const char *json_api_repr_pipe = "{\n    \"types\": [],\n    \"messages\": [\n        [\n            \"pipe_create\",\n            [\n                \"u16\",\n                \"_vl_msg_id\"\n            ],\n            [\n                \"u32\",\n                \"client_index\"\n            ],\n            [\n                \"u32\",\n                \"context\"\n            ],\n            [\n                \"bool\",\n                \"is_specified\"\n            ],\n            [\n                \"u32\",\n                \"user_instance\"\n            ],\n            {\n                \"crc\": \"0xbb263bd3\",\n                \"options\": {},\n                \"comment\": \"/** \\\\brief Initialize a new pipe interface with the given parameters\\n    @param client_index - opaque cookie to identify the sender\\n    @param context - sender context, to match reply w/ request\\n    @param is_specified - if non-0, a specific user_instance is being requested\\n    @param user_instance - requested instance, ~0 => dynamically allocate\\n*/\"\n            }\n        ],\n        [\n            \"pipe_create_reply\",\n            [\n                \"u16\",\n                \"_vl_msg_id\"\n            ],\n            [\n                \"u32\",\n                \"context\"\n            ],\n            [\n                \"i32\",\n                \"retval\"\n            ],\n            [\n                \"vl_api_interface_index_t\",\n                \"sw_if_index\"\n            ],\n            [\n                \"vl_api_interface_index_t\",\n                \"pipe_sw_if_index\",\n                2\n            ],\n            {\n                \"crc\": \"0xb7ce310c\",\n                \"options\": {},\n                \"comment\": \"/** \\\\brief Reply for pipe create reply\\n    @param context - returned sender context, to match reply w/ request\\n    @param retval - return code\\n    @param sw_if_index - software index allocated for the new pipe parent interface\\n                         Use the parent interface for link up/down and to delete\\n    @param pipe_sw_if_index - the two SW indicies that form the ends of the pipe.\\n*/\"\n            }\n        ],\n        [\n            \"pipe_delete\",\n            [\n                \"u16\",\n                \"_vl_msg_id\"\n            ],\n            [\n                \"u32\",\n                \"client_index\"\n            ],\n            [\n                \"u32\",\n                \"context\"\n            ],\n            [\n                \"vl_api_interface_index_t\",\n                \"sw_if_index\"\n            ],\n            {\n                \"crc\": \"0xf9e6675e\",\n                \"options\": {},\n                \"comment\": \"/** \\\\brief Delete pipe interface\\n    @param client_index - opaque cookie to identify the sender\\n    @param context - sender context, to match reply w/ request\\n    @param sw_if_index - interface index of existing parent pipe interface\\n*/\"\n            }\n        ],\n        [\n            \"pipe_delete_reply\",\n            [\n                \"u16\",\n                \"_vl_msg_id\"\n            ],\n            [\n                \"u32\",\n                \"context\"\n            ],\n            [\n                \"i32\",\n                \"retval\"\n            ],\n            {\n                \"crc\": \"0xe8d4e804\",\n                \"options\": {}\n            }\n        ],\n        [\n            \"pipe_dump\",\n            [\n                \"u16\",\n                \"_vl_msg_id\"\n            ],\n            [\n                \"u32\",\n                \"client_index\"\n            ],\n            [\n                \"u32\",\n                \"context\"\n            ],\n            {\n                \"crc\": \"0x51077d14\",\n                \"options\": {},\n                \"comment\": \"/** \\\\brief Dump pipe interfaces request */\"\n            }\n        ],\n        [\n            \"pipe_details\",\n            [\n                \"u16\",\n                \"_vl_msg_id\"\n            ],\n            [\n                \"u32\",\n                \"context\"\n            ],\n            [\n                \"vl_api_interface_index_t\",\n                \"sw_if_index\"\n            ],\n            [\n                \"vl_api_interface_index_t\",\n                \"pipe_sw_if_index\",\n                2\n            ],\n            [\n                \"u32\",\n                \"instance\"\n            ],\n            {\n                \"crc\": \"0xc52b799d\",\n                \"options\": {},\n                \"comment\": \"/** \\\\brief Reply for pipe dump request\\n    @param sw_if_index - software index of pipe interface\\n    @param sw_if_index - software index allocated for the pipe parent interface\\n    @param pipe_sw_if_index - the two SW indicies that form the ends of the pipe.\\n    @param instance - instance allocated\\n*/\"\n            }\n        ]\n    ],\n    \"unions\": [],\n    \"enums\": [\n        [\n            \"if_status_flags\",\n            [\n                \"IF_STATUS_API_FLAG_ADMIN_UP\",\n                1\n            ],\n            [\n                \"IF_STATUS_API_FLAG_LINK_UP\",\n                2\n            ],\n            {\n                \"enumtype\": \"u32\"\n            }\n        ],\n        [\n            \"mtu_proto\",\n            [\n                \"MTU_PROTO_API_L3\",\n                0\n            ],\n            [\n                \"MTU_PROTO_API_IP4\",\n                1\n            ],\n            [\n                \"MTU_PROTO_API_IP6\",\n                2\n            ],\n            [\n                \"MTU_PROTO_API_MPLS\",\n                3\n            ],\n            {\n                \"enumtype\": \"u32\"\n            }\n        ],\n        [\n            \"link_duplex\",\n            [\n                \"LINK_DUPLEX_API_UNKNOWN\",\n                0\n            ],\n            [\n                \"LINK_DUPLEX_API_HALF\",\n                1\n            ],\n            [\n                \"LINK_DUPLEX_API_FULL\",\n                2\n            ],\n            {\n                \"enumtype\": \"u32\"\n            }\n        ],\n        [\n            \"sub_if_flags\",\n            [\n                \"SUB_IF_API_FLAG_NO_TAGS\",\n                1\n            ],\n            [\n                \"SUB_IF_API_FLAG_ONE_TAG\",\n                2\n            ],\n            [\n                \"SUB_IF_API_FLAG_TWO_TAGS\",\n                4\n            ],\n            [\n                \"SUB_IF_API_FLAG_DOT1AD\",\n                8\n            ],\n            [\n                \"SUB_IF_API_FLAG_EXACT_MATCH\",\n                16\n            ],\n            [\n                \"SUB_IF_API_FLAG_DEFAULT\",\n                32\n            ],\n            [\n                \"SUB_IF_API_FLAG_OUTER_VLAN_ID_ANY\",\n                64\n            ],\n            [\n                \"SUB_IF_API_FLAG_INNER_VLAN_ID_ANY\",\n                128\n            ],\n            [\n                \"SUB_IF_API_FLAG_MASK_VNET\",\n                254\n            ],\n            [\n                \"SUB_IF_API_FLAG_DOT1AH\",\n                256\n            ],\n            {\n                \"enumtype\": \"u32\"\n            }\n        ],\n        [\n            \"rx_mode\",\n            [\n                \"RX_MODE_API_UNKNOWN\",\n                0\n            ],\n            [\n                \"RX_MODE_API_POLLING\",\n                1\n            ],\n            [\n                \"RX_MODE_API_INTERRUPT\",\n                2\n            ],\n            [\n                \"RX_MODE_API_ADAPTIVE\",\n                3\n            ],\n            [\n                \"RX_MODE_API_DEFAULT\",\n                4\n            ],\n            {\n                \"enumtype\": \"u32\"\n            }\n        ],\n        [\n            \"if_type\",\n            [\n                \"IF_API_TYPE_HARDWARE\",\n                0\n            ],\n            [\n                \"IF_API_TYPE_SUB\",\n                1\n            ],\n            [\n                \"IF_API_TYPE_P2P\",\n                2\n            ],\n            [\n                \"IF_API_TYPE_PIPE\",\n                3\n            ],\n            {\n                \"enumtype\": \"u32\"\n            }\n        ],\n        [\n            \"direction\",\n            [\n                \"RX\",\n                0\n            ],\n            [\n                \"TX\",\n                1\n            ],\n            {\n                \"enumtype\": \"u8\"\n            }\n        ]\n    ],\n    \"enumflags\": [],\n    \"services\": {\n        \"pipe_create\": {\n            \"reply\": \"pipe_create_reply\"\n        },\n        \"pipe_delete\": {\n            \"reply\": \"pipe_delete_reply\"\n        },\n        \"pipe_dump\": {\n            \"reply\": \"pipe_details\",\n            \"stream\": true\n        }\n    },\n    \"options\": {\n        \"version\": \"1.0.1\"\n    },\n    \"aliases\": {\n        \"interface_index\": {\n            \"type\": \"u32\"\n        }\n    },\n    \"vl_api_version\": \"0xc159134a\",\n    \"imports\": [\n        \"vnet/interface_types.api\"\n    ],\n    \"counters\": [],\n    \"paths\": []\n}";
