const char *json_api_repr_virtio = "{\n    \"types\": [\n        [\n            \"pci_address\",\n            [\n                \"u16\",\n                \"domain\"\n            ],\n            [\n                \"u8\",\n                \"bus\"\n            ],\n            [\n                \"u8\",\n                \"slot\"\n            ],\n            [\n                \"u8\",\n                \"function\"\n            ]\n        ]\n    ],\n    \"messages\": [\n        [\n            \"virtio_pci_create\",\n            [\n                \"u16\",\n                \"_vl_msg_id\"\n            ],\n            [\n                \"u32\",\n                \"client_index\"\n            ],\n            [\n                \"u32\",\n                \"context\"\n            ],\n            [\n                \"vl_api_pci_address_t\",\n                \"pci_addr\"\n            ],\n            [\n                \"bool\",\n                \"use_random_mac\"\n            ],\n            [\n                \"vl_api_mac_address_t\",\n                \"mac_address\"\n            ],\n            [\n                \"bool\",\n                \"gso_enabled\"\n            ],\n            [\n                \"bool\",\n                \"checksum_offload_enabled\"\n            ],\n            [\n                \"u64\",\n                \"features\"\n            ],\n            {\n                \"crc\": \"0x1944f8db\",\n                \"options\": {\n                    \"deprecated\": null\n                },\n                \"comment\": \"/** \\\\brief Initialize a new virtio pci interface with the given parameters\\n    @param client_index - opaque cookie to identify the sender\\n    @param context - sender context, to match reply w/ request\\n    @param pci_addr - pci address\\n    @param use_random_mac - let the system generate a unique mac address\\n    @param mac_address - mac addr to assign to the interface if use_random not set\\n    @param gso_enabled - enable gso feature if available, 1 to enable\\n    @param checksum_offload_enabled - enable checksum feature if available, 1 to enable\\n    @param features - the virtio features which driver should negotiate with device\\n*/\"\n            }\n        ],\n        [\n            \"virtio_pci_create_reply\",\n            [\n                \"u16\",\n                \"_vl_msg_id\"\n            ],\n            [\n                \"u32\",\n                \"context\"\n            ],\n            [\n                \"i32\",\n                \"retval\"\n            ],\n            [\n                \"vl_api_interface_index_t\",\n                \"sw_if_index\"\n            ],\n            {\n                \"crc\": \"0x5383d31f\",\n                \"options\": {\n                    \"deprecated\": null\n                },\n                \"comment\": \"/** \\\\brief Reply for virtio pci create reply\\n    @param context - returned sender context, to match reply w/ request\\n    @param retval - return code\\n    @param sw_if_index - software index allocated for the new virtio pci interface\\n*/\"\n            }\n        ],\n        [\n            \"virtio_pci_create_v2\",\n            [\n                \"u16\",\n                \"_vl_msg_id\"\n            ],\n            [\n                \"u32\",\n                \"client_index\"\n            ],\n            [\n                \"u32\",\n                \"context\"\n            ],\n            [\n                \"vl_api_pci_address_t\",\n                \"pci_addr\"\n            ],\n            [\n                \"bool\",\n                \"use_random_mac\"\n            ],\n            [\n                \"vl_api_mac_address_t\",\n                \"mac_address\"\n            ],\n            [\n                \"vl_api_virtio_flags_t\",\n                \"virtio_flags\"\n            ],\n            [\n                \"u64\",\n                \"features\"\n            ],\n            {\n                \"crc\": \"0x5d096e1a\",\n                \"options\": {},\n                \"comment\": \"/** \\\\brief Initialize a new virtio pci interface with the given parameters\\n    @param client_index - opaque cookie to identify the sender\\n    @param context - sender context, to match reply w/ request\\n    @param pci_addr - pci address\\n    @param use_random_mac - let the system generate a unique mac address\\n    @param mac_address - mac addr to assign to the interface if use_random not set\\n    @param virtio_flags - feature flags to enable\\n    @param features - the virtio features which driver should negotiate with device\\n*/\"\n            }\n        ],\n        [\n            \"virtio_pci_create_v2_reply\",\n            [\n                \"u16\",\n                \"_vl_msg_id\"\n            ],\n            [\n                \"u32\",\n                \"context\"\n            ],\n            [\n                \"i32\",\n                \"retval\"\n            ],\n            [\n                \"vl_api_interface_index_t\",\n                \"sw_if_index\"\n            ],\n            {\n                \"crc\": \"0x5383d31f\",\n                \"options\": {},\n                \"comment\": \"/** \\\\brief Reply for virtio pci create reply\\n    @param context - returned sender context, to match reply w/ request\\n    @param retval - return code\\n    @param sw_if_index - software index allocated for the new virtio pci interface\\n*/\"\n            }\n        ],\n        [\n            \"virtio_pci_delete\",\n            [\n                \"u16\",\n                \"_vl_msg_id\"\n            ],\n            [\n                \"u32\",\n                \"client_index\"\n            ],\n            [\n                \"u32\",\n                \"context\"\n            ],\n            [\n                \"vl_api_interface_index_t\",\n                \"sw_if_index\"\n            ],\n            {\n                \"crc\": \"0xf9e6675e\",\n                \"options\": {},\n                \"comment\": \"/** \\\\brief Delete virtio pci interface\\n    @param client_index - opaque cookie to identify the sender\\n    @param context - sender context, to match reply w/ request\\n    @param sw_if_index - interface index of existing virtio pci interface\\n*/\"\n            }\n        ],\n        [\n            \"virtio_pci_delete_reply\",\n            [\n                \"u16\",\n                \"_vl_msg_id\"\n            ],\n            [\n                \"u32\",\n                \"context\"\n            ],\n            [\n                \"i32\",\n                \"retval\"\n            ],\n            {\n                \"crc\": \"0xe8d4e804\",\n                \"options\": {}\n            }\n        ],\n        [\n            \"sw_interface_virtio_pci_dump\",\n            [\n                \"u16\",\n                \"_vl_msg_id\"\n            ],\n            [\n                \"u32\",\n                \"client_index\"\n            ],\n            [\n                \"u32\",\n                \"context\"\n            ],\n            {\n                \"crc\": \"0x51077d14\",\n                \"options\": {},\n                \"comment\": \"/** \\\\brief Dump virtio pci interfaces request */\"\n            }\n        ],\n        [\n            \"sw_interface_virtio_pci_details\",\n            [\n                \"u16\",\n                \"_vl_msg_id\"\n            ],\n            [\n                \"u32\",\n                \"context\"\n            ],\n            [\n                \"vl_api_interface_index_t\",\n                \"sw_if_index\"\n            ],\n            [\n                \"vl_api_pci_address_t\",\n                \"pci_addr\"\n            ],\n            [\n                \"vl_api_mac_address_t\",\n                \"mac_addr\"\n            ],\n            [\n                \"u16\",\n                \"tx_ring_sz\"\n            ],\n            [\n                \"u16\",\n                \"rx_ring_sz\"\n            ],\n            [\n                \"u64\",\n                \"features\"\n            ],\n            {\n                \"crc\": \"0x6ca9c167\",\n                \"options\": {},\n                \"comment\": \"/** \\\\brief Reply for virtio pci interface dump request\\n    @param sw_if_index - software index of virtio pci interface\\n    @param pci_addr - pci address\\n    @param mac_addr - native virtio device mac address\\n    @param tx_ring_sz - the number of entries of TX ring\\n    @param rx_ring_sz - the number of entries of RX ring\\n    @param features - the virtio features which driver have negotiated with device\\n*/\"\n            }\n        ]\n    ],\n    \"unions\": [],\n    \"enums\": [\n        [\n            \"if_status_flags\",\n            [\n                \"IF_STATUS_API_FLAG_ADMIN_UP\",\n                1\n            ],\n            [\n                \"IF_STATUS_API_FLAG_LINK_UP\",\n                2\n            ],\n            {\n                \"enumtype\": \"u32\"\n            }\n        ],\n        [\n            \"mtu_proto\",\n            [\n                \"MTU_PROTO_API_L3\",\n                0\n            ],\n            [\n                \"MTU_PROTO_API_IP4\",\n                1\n            ],\n            [\n                \"MTU_PROTO_API_IP6\",\n                2\n            ],\n            [\n                \"MTU_PROTO_API_MPLS\",\n                3\n            ],\n            {\n                \"enumtype\": \"u32\"\n            }\n        ],\n        [\n            \"link_duplex\",\n            [\n                \"LINK_DUPLEX_API_UNKNOWN\",\n                0\n            ],\n            [\n                \"LINK_DUPLEX_API_HALF\",\n                1\n            ],\n            [\n                \"LINK_DUPLEX_API_FULL\",\n                2\n            ],\n            {\n                \"enumtype\": \"u32\"\n            }\n        ],\n        [\n            \"sub_if_flags\",\n            [\n                \"SUB_IF_API_FLAG_NO_TAGS\",\n                1\n            ],\n            [\n                \"SUB_IF_API_FLAG_ONE_TAG\",\n                2\n            ],\n            [\n                \"SUB_IF_API_FLAG_TWO_TAGS\",\n                4\n            ],\n            [\n                \"SUB_IF_API_FLAG_DOT1AD\",\n                8\n            ],\n            [\n                \"SUB_IF_API_FLAG_EXACT_MATCH\",\n                16\n            ],\n            [\n                \"SUB_IF_API_FLAG_DEFAULT\",\n                32\n            ],\n            [\n                \"SUB_IF_API_FLAG_OUTER_VLAN_ID_ANY\",\n                64\n            ],\n            [\n                \"SUB_IF_API_FLAG_INNER_VLAN_ID_ANY\",\n                128\n            ],\n            [\n                \"SUB_IF_API_FLAG_MASK_VNET\",\n                254\n            ],\n            [\n                \"SUB_IF_API_FLAG_DOT1AH\",\n                256\n            ],\n            {\n                \"enumtype\": \"u32\"\n            }\n        ],\n        [\n            \"rx_mode\",\n            [\n                \"RX_MODE_API_UNKNOWN\",\n                0\n            ],\n            [\n                \"RX_MODE_API_POLLING\",\n                1\n            ],\n            [\n                \"RX_MODE_API_INTERRUPT\",\n                2\n            ],\n            [\n                \"RX_MODE_API_ADAPTIVE\",\n                3\n            ],\n            [\n                \"RX_MODE_API_DEFAULT\",\n                4\n            ],\n            {\n                \"enumtype\": \"u32\"\n            }\n        ],\n        [\n            \"if_type\",\n            [\n                \"IF_API_TYPE_HARDWARE\",\n                0\n            ],\n            [\n                \"IF_API_TYPE_SUB\",\n                1\n            ],\n            [\n                \"IF_API_TYPE_P2P\",\n                2\n            ],\n            [\n                \"IF_API_TYPE_PIPE\",\n                3\n            ],\n            {\n                \"enumtype\": \"u32\"\n            }\n        ],\n        [\n            \"direction\",\n            [\n                \"RX\",\n                0\n            ],\n            [\n                \"TX\",\n                1\n            ],\n            {\n                \"enumtype\": \"u8\"\n            }\n        ]\n    ],\n    \"enumflags\": [\n        [\n            \"virtio_flags\",\n            [\n                \"VIRTIO_API_FLAG_GSO\",\n                1\n            ],\n            [\n                \"VIRTIO_API_FLAG_CSUM_OFFLOAD\",\n                2\n            ],\n            [\n                \"VIRTIO_API_FLAG_GRO_COALESCE\",\n                4\n            ],\n            [\n                \"VIRTIO_API_FLAG_PACKED\",\n                8\n            ],\n            [\n                \"VIRTIO_API_FLAG_IN_ORDER\",\n                16\n            ],\n            [\n                \"VIRTIO_API_FLAG_BUFFERING\",\n                32\n            ],\n            [\n                \"VIRTIO_API_FLAG_RSS\",\n                64\n            ],\n            {\n                \"enumtype\": \"u32\"\n            }\n        ]\n    ],\n    \"services\": {\n        \"virtio_pci_create\": {\n            \"reply\": \"virtio_pci_create_reply\"\n        },\n        \"virtio_pci_create_v2\": {\n            \"reply\": \"virtio_pci_create_v2_reply\"\n        },\n        \"virtio_pci_delete\": {\n            \"reply\": \"virtio_pci_delete_reply\"\n        },\n        \"sw_interface_virtio_pci_dump\": {\n            \"reply\": \"sw_interface_virtio_pci_details\",\n            \"stream\": true\n        }\n    },\n    \"options\": {\n        \"version\": \"3.0.0\"\n    },\n    \"aliases\": {\n        \"interface_index\": {\n            \"type\": \"u32\"\n        },\n        \"mac_address\": {\n            \"type\": \"u8\",\n            \"length\": 6\n        }\n    },\n    \"vl_api_version\": \"0xa507d784\",\n    \"imports\": [\n        \"vnet/interface_types.api\",\n        \"vnet/ethernet/ethernet_types.api\",\n        \"vlib/pci/pci_types.api\"\n    ],\n    \"counters\": [],\n    \"paths\": []\n}";
