/* Copyright (c) 2023 Cisco and/or its affiliates.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at:
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an  BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. */

/* Test vectors published in NIST Cryptographic Algorithm Validation Program
 * (CAVP)
 * https://csrc.nist.gov/Projects/Cryptographic-Algorithm-Validation-Program/CAVP-TESTING-BLOCK-CIPHER-MODES#GCMVS
 */

#include <vppinfra/clib.h>
#include <vnet/crypto/crypto.h>
#include <unittest/crypto/crypto.h>

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad0_tc0) = {
  .name = "128-GMAC 0-aad NIST CAVS TC0",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x11\x75\x4c\xd7\x2a\xec\x30\x9b\xf5\x2f\x76\x87\x21\x2e\x89\x57"),
  .iv = TEST_DATA_STR ("\x3c\x81\x9d\x9a\x9b\xed\x08\x76\x15\x03\x0b\x65"),
  .tag = TEST_DATA_STR (
    "\x25\x03\x27\xc6\x74\xaa\xf4\x77\xae\xf2\x67\x57\x48\xcf\x69\x71"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad0_tc1) = {
  .name = "128-GMAC 0-aad NIST CAVS TC1",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xca\x47\x24\x8a\xc0\xb6\xf8\x37\x2a\x97\xac\x43\x50\x83\x08\xed"),
  .iv = TEST_DATA_STR ("\xff\xd2\xb5\x98\xfe\xab\xc9\x01\x92\x62\xd2\xbe"),
  .tag = TEST_DATA_STR (
    "\x60\xd2\x04\x04\xaf\x52\x7d\x24\x8d\x89\x3a\xe4\x95\x70\x7d\x1a"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad0_tc2) = {
  .name = "128-GMAC 0-aad NIST CAVS TC2",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xdb\x1a\xd0\xbd\x1c\xf6\xdb\x0b\x5d\x86\xef\xdd\x89\x14\xb2\x18"),
  .iv = TEST_DATA_STR ("\x36\xfa\xd6\xac\xb3\xc9\x8e\x01\x38\xae\xb9\xb1"),
  .tag = TEST_DATA_STR (
    "\x5e\xe2\xba\x73\x7d\x3f\x2a\x94\x4b\x33\x5a\x81\xf6\x65\x3c\xce"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad0_tc3) = {
  .name = "128-GMAC 0-aad NIST CAVS TC3",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x1c\x71\x35\xaf\x62\x7c\x04\xc3\x29\x57\xf3\x3f\x9a\xc0\x85\x90"),
  .iv = TEST_DATA_STR ("\x35\x5c\x09\x4f\xa0\x9c\x8e\x92\x81\x17\x8d\x34"),
  .tag = TEST_DATA_STR (
    "\xb6\xab\x2c\x7d\x90\x6c\x9d\x9e\xc4\xc1\x49\x8d\x2c\xbb\x50\x29"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad0_tc4) = {
  .name = "128-GMAC 0-aad NIST CAVS TC4",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x6c\xa2\xc1\x12\x05\xa6\xe5\x5a\xb5\x04\xdb\xf3\x49\x1f\x8b\xdc"),
  .iv = TEST_DATA_STR ("\xb1\x00\x8b\x65\x0a\x2f\xee\x64\x21\x75\xc6\x0d"),
  .tag = TEST_DATA_STR (
    "\x7a\x9a\x22\x5d\x5f\x9a\x0e\xbf\xe0\xe6\x9f\x37\x18\x71\xa6\x72"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad0_tc5) = {
  .name = "128-GMAC 0-aad NIST CAVS TC5",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x69\xf2\xca\x78\xbb\x56\x90\xac\xc6\x58\x73\x02\x62\x88\x28\xd5"),
  .iv = TEST_DATA_STR ("\x70\x1d\xa2\x82\xcb\x6b\x60\x18\xda\xbd\x00\xd3"),
  .tag = TEST_DATA_STR (
    "\xab\x1d\x40\xdd\xa1\x79\x8d\x56\x68\x78\x92\xe2\x15\x9d\xec\xfd"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad0_tc6) = {
  .name = "128-GMAC 0-aad NIST CAVS TC6",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xdc\xf4\xe3\x39\xc4\x87\xb6\x79\x7a\xac\xa9\x31\x72\x5f\x7b\xbd"),
  .iv = TEST_DATA_STR ("\x2c\x1d\x95\x5e\x35\x36\x67\x60\xea\xd8\x81\x7c"),
  .tag = TEST_DATA_STR (
    "\x32\xb5\x42\xc5\xf3\x44\xcc\xec\xeb\x46\x0a\x02\x93\x8d\x6b\x0c"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad0_tc7) = {
  .name = "128-GMAC 0-aad NIST CAVS TC7",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x76\x58\xcd\xbb\x81\x57\x2a\x23\xa7\x8e\xe4\x59\x6f\x84\x4e\xe9"),
  .iv = TEST_DATA_STR ("\x1c\x3b\xaa\xe9\xb9\x06\x59\x61\x84\x2c\xbe\x52"),
  .tag = TEST_DATA_STR (
    "\x70\xc7\x12\x3f\xc8\x19\xaa\x06\x0e\xd2\xd3\xc1\x59\xb6\xea\x41"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad0_tc8) = {
  .name = "128-GMAC 0-aad NIST CAVS TC8",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x28\x1a\x57\x0b\x1e\x8f\x26\x5e\xe0\x93\x03\xec\xae\x0c\xc4\x6d"),
  .iv = TEST_DATA_STR ("\x8c\x29\x41\xf7\x3c\xf8\x71\x3a\xd5\xbc\x13\xdf"),
  .tag = TEST_DATA_STR (
    "\xa4\x2e\x5e\x5f\x6f\xb0\x0a\x9f\x12\x06\xb3\x02\xed\xbf\xd8\x7c"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad0_tc9) = {
  .name = "128-GMAC 0-aad NIST CAVS TC9",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xcd\x33\x2a\x98\x6f\x82\xd9\x8c\x21\x52\x78\x13\x1a\xd3\x87\xb7"),
  .iv = TEST_DATA_STR ("\x1d\x12\xb2\x59\xf4\x4b\x87\x3d\x39\x42\xbc\x11"),
  .tag = TEST_DATA_STR (
    "\x34\x23\x80\x23\x64\x81\x85\xd7\xef\x0c\xfc\xf5\x83\x6e\x93\xcc"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad0_tc10) = {
  .name = "128-GMAC 0-aad NIST CAVS TC10",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x80\xe1\xd9\x8d\x10\xb2\x72\x37\x38\x6f\x02\x91\x89\xec\x04\x48"),
  .iv = TEST_DATA_STR ("\x23\x9e\xba\xb2\xf5\x24\xfd\x62\xc5\x54\xa1\x90"),
  .tag = TEST_DATA_STR (
    "\x4c\x0f\x29\xd9\x63\xf0\xed\x68\xdc\xcf\x34\x49\x6c\xf4\x3d\x00"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad0_tc11) = {
  .name = "128-GMAC 0-aad NIST CAVS TC11",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x40\x65\x0c\xdb\x61\xe3\xe1\x9a\x1a\x98\xfb\x4e\x05\x37\x7d\x35"),
  .iv = TEST_DATA_STR ("\x69\xf0\xa8\x1a\xaf\x6b\xb8\x48\x62\x82\xf1\xb9"),
  .tag = TEST_DATA_STR (
    "\x26\x57\xe1\x2d\xec\x21\xc3\xec\xf0\x71\xaf\x61\x79\x52\x9f\xb4"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad0_tc12) = {
  .name = "128-GMAC 0-aad NIST CAVS TC12",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x1e\x89\xa6\xcd\x75\x28\xcc\xe1\xe2\xb2\xb5\xf7\xfd\x2b\x6b\x52"),
  .iv = TEST_DATA_STR ("\xe1\x1f\xd4\x27\xa7\x82\xd5\x43\xf7\x8e\xfc\x60"),
  .tag = TEST_DATA_STR (
    "\xee\xed\xff\x87\x4c\x8e\xde\xea\x53\xe8\xbe\x2a\x13\xaf\xd8\x1b"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad0_tc13) = {
  .name = "128-GMAC 0-aad NIST CAVS TC13",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x2a\x7a\xd6\x14\x66\x76\x05\x7d\xb7\x77\xde\xa4\x68\x3d\x0d\x45"),
  .iv = TEST_DATA_STR ("\xed\x72\x1e\xa6\x74\x56\xd4\x59\x4a\xaf\xbd\x51"),
  .tag = TEST_DATA_STR (
    "\xee\x3c\xab\x57\x78\x88\x84\x39\xd9\x0f\xa7\x18\xb7\x57\x38\xad"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad0_tc14) = {
  .name = "128-GMAC 0-aad NIST CAVS TC14",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xa3\x64\xf4\x94\xa4\xcd\x01\x47\xc3\x47\x31\x07\x4d\xc1\xa8\x5b"),
  .iv = TEST_DATA_STR ("\x4a\xa8\x47\x0d\xd4\x04\xe4\x05\x4b\x30\x09\x3a"),
  .tag = TEST_DATA_STR (
    "\xd8\xa7\xbb\xa3\xa4\x51\x90\x2e\x3a\xdc\x01\x06\x0c\x3c\x91\xa7"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad128_tc0) = {
  .name = "128-GMAC 128-aad NIST CAVS TC0",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x77\xbe\x63\x70\x89\x71\xc4\xe2\x40\xd1\xcb\x79\xe8\xd7\x7f\xeb"),
  .iv = TEST_DATA_STR ("\xe0\xe0\x0f\x19\xfe\xd7\xba\x01\x36\xa7\x97\xf3"),
  .aad = TEST_DATA_STR (
    "\x7a\x43\xec\x1d\x9c\x0a\x5a\x78\xa0\xb1\x65\x33\xa6\x21\x3c\xab"),
  .tag = TEST_DATA_STR (
    "\x20\x9f\xcc\x8d\x36\x75\xed\x93\x8e\x9c\x71\x66\x70\x9d\xd9\x46"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad128_tc1) = {
  .name = "128-GMAC 128-aad NIST CAVS TC1",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x76\x80\xc5\xd3\xca\x61\x54\x75\x8e\x51\x0f\x4d\x25\xb9\x88\x20"),
  .iv = TEST_DATA_STR ("\xf8\xf1\x05\xf9\xc3\xdf\x49\x65\x78\x03\x21\xf8"),
  .aad = TEST_DATA_STR (
    "\xc9\x4c\x41\x01\x94\xc7\x65\xe3\xdc\xc7\x96\x43\x79\x75\x8e\xd3"),
  .tag = TEST_DATA_STR (
    "\x94\xdc\xa8\xed\xfc\xf9\x0b\xb7\x4b\x15\x3c\x8d\x48\xa1\x79\x30"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad128_tc2) = {
  .name = "128-GMAC 128-aad NIST CAVS TC2",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xa8\x2b\xb1\xed\xc7\xc0\x1a\x36\x89\x00\x6f\x34\xbf\xed\x78\x3e"),
  .iv = TEST_DATA_STR ("\x96\x38\x36\xb6\x7b\x18\x8b\xec\xf9\xba\x14\x11"),
  .aad = TEST_DATA_STR (
    "\x9d\x11\x5b\xb9\xbb\xd1\x19\xfb\x77\x7b\x63\x16\x06\x5a\x9a\xc8"),
  .tag = TEST_DATA_STR (
    "\xc4\x91\x88\x9f\xa3\xec\xa4\x54\x4b\xa0\xd5\x1b\x8e\x0f\x38\x37"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad128_tc3) = {
  .name = "128-GMAC 128-aad NIST CAVS TC3",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xb9\x78\x2d\x0a\x59\x86\xc6\x3f\x35\x2d\x3b\xc4\xc7\xec\xc9\x6d"),
  .iv = TEST_DATA_STR ("\x45\x41\xe1\x5b\x92\xed\xea\x44\xec\xeb\x1f\x2a"),
  .aad = TEST_DATA_STR (
    "\xf1\xa9\xf0\x72\x34\x29\xc5\xb2\x61\x85\xac\x3e\xa7\xe1\x3d\x7a"),
  .tag = TEST_DATA_STR (
    "\x74\xd0\xd3\x69\x49\xf0\x27\x66\x70\xf9\xdd\xc5\x79\xe9\x4f\x3a"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad128_tc4) = {
  .name = "128-GMAC 128-aad NIST CAVS TC4",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x59\xb9\x57\x85\xb3\x0f\x20\x56\x79\xfc\x4f\x3f\x9a\x90\x10\x2f"),
  .iv = TEST_DATA_STR ("\x19\x08\x78\x7c\xc1\xe1\x88\x0a\x6e\xf5\xdd\x17"),
  .aad = TEST_DATA_STR (
    "\x39\x85\x2d\x31\x82\x94\x4a\x51\x77\xdb\x27\x7b\x63\x91\x07\x02"),
  .tag = TEST_DATA_STR (
    "\x8f\x9a\x96\xc0\x13\x99\x24\x85\xb4\x3e\x2b\x62\x74\x5a\xd1\x73"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad128_tc5) = {
  .name = "128-GMAC 128-aad NIST CAVS TC5",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x34\xdd\x79\x26\xab\x13\xd4\x07\x81\x60\xd8\x7d\xe2\xe3\xc7\x24"),
  .iv = TEST_DATA_STR ("\xc1\x1c\xcd\xaf\x79\x8a\xb0\x3a\xf2\xd9\x7e\xf9"),
  .aad = TEST_DATA_STR (
    "\xaf\x69\x87\x17\xa6\xd7\x90\xb3\xbf\xc3\x91\x95\x85\x7b\xb5\xff"),
  .tag = TEST_DATA_STR (
    "\x48\x11\x60\x50\xbb\xd9\x11\x82\x70\xd0\xbe\x25\x2d\x29\xd5\xd4"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad128_tc6) = {
  .name = "128-GMAC 128-aad NIST CAVS TC6",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x8e\xc8\x6f\xab\x55\xaa\xab\x0e\x77\x45\x5e\x9c\xd3\xdb\xc7\x8e"),
  .iv = TEST_DATA_STR ("\x15\xfd\x90\xa9\x86\x7e\x14\xf0\xd6\x3b\x53\xb9"),
  .aad = TEST_DATA_STR (
    "\xe7\x50\x9e\x27\x62\x09\xa6\xd3\xec\xfa\xbb\x53\xcc\xdc\xd2\x36"),
  .tag = TEST_DATA_STR (
    "\xd9\x6d\x6a\xc0\xd3\x09\xce\xbe\xde\xba\x2a\xf9\xf2\x62\x13\x2f"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad128_tc7) = {
  .name = "128-GMAC 128-aad NIST CAVS TC7",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x66\xb2\x47\x3d\x9e\x01\x21\x66\x6d\x47\x63\x3f\x70\x08\xeb\x1c"),
  .iv = TEST_DATA_STR ("\xc1\x71\x6c\x68\xa2\x4d\x57\x77\x0b\x86\x7e\x51"),
  .aad = TEST_DATA_STR (
    "\xc2\x0f\x68\x63\x17\xd6\x7e\x53\xdd\x79\xba\xe5\xc4\x6d\xc1\x11"),
  .tag = TEST_DATA_STR (
    "\x9a\x08\x61\x68\x09\xcf\x15\x24\x7d\xfe\xb9\x75\x6b\xa4\xf6\x09"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad128_tc8) = {
  .name = "128-GMAC 128-aad NIST CAVS TC8",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x5b\x26\x2a\x9d\x00\x90\x4d\x30\xa2\x58\x7c\xaa\xde\x09\x13\x81"),
  .iv = TEST_DATA_STR ("\xf7\xbc\x15\x4c\xa5\x62\xe8\xf2\xc1\x84\x55\x98"),
  .aad = TEST_DATA_STR (
    "\x23\x11\x2d\x07\x8c\x99\x14\xfa\x3d\xfe\x52\x18\xcd\x19\x10\x16"),
  .tag = TEST_DATA_STR (
    "\x98\x85\x4d\x19\x3a\x06\xdb\xe3\x2c\xe4\x49\x7e\xec\x5c\x9a\x8b"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad128_tc9) = {
  .name = "128-GMAC 128-aad NIST CAVS TC9",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x2e\x4f\xb9\xcc\x32\x01\x88\xa6\xf1\xfa\x89\xa7\xa2\x52\x27\x3a"),
  .iv = TEST_DATA_STR ("\x7a\x6d\x4e\xe6\x9c\x72\x56\xc1\x4f\xba\x8f\x5e"),
  .aad = TEST_DATA_STR (
    "\x80\xba\x4a\x20\x2a\x68\xc3\x59\x0d\x65\x57\x91\x2c\x6f\x87\x8e"),
  .tag = TEST_DATA_STR (
    "\x92\x80\x31\x32\x73\xbe\xfb\x8a\xfa\x0b\xce\xca\x5a\x96\x6d\x85"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad128_tc10) = {
  .name = "128-GMAC 128-aad NIST CAVS TC10",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x5e\xa9\x49\x73\xd8\x61\x6d\xaf\xa7\xf3\x1d\xb0\x71\x6d\x17\x29"),
  .iv = TEST_DATA_STR ("\xa0\x5b\x62\x66\x9d\x25\x0e\x61\xb0\x77\xd2\x8a"),
  .aad = TEST_DATA_STR (
    "\x96\x20\xba\xf2\xf5\x8d\x01\x3f\x8a\x4c\x48\x71\x98\x9c\x1b\x17"),
  .tag = TEST_DATA_STR (
    "\x7e\x55\x03\x98\xde\xe7\x28\x25\x6d\x69\x28\xcd\xaa\xc4\x3b\x73"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad128_tc11) = {
  .name = "128-GMAC 128-aad NIST CAVS TC11",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x91\x03\x85\xf6\xf0\x7f\x9e\x57\xe4\x83\xc4\x7d\xd5\x20\x6b\xcc"),
  .iv = TEST_DATA_STR ("\x51\x8f\x56\xe3\x36\x58\xdf\x31\x1d\x42\xd9\xfe"),
  .aad = TEST_DATA_STR (
    "\x5d\x15\x79\x09\xa2\xa4\x60\x71\x17\xe7\x7d\xa0\xe4\x49\x3b\x88"),
  .tag = TEST_DATA_STR (
    "\xa7\x04\x1e\xa4\xa1\xd7\x4d\x9e\x66\xb9\x57\x1b\x59\xb6\xa1\xd8"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad128_tc12) = {
  .name = "128-GMAC 128-aad NIST CAVS TC12",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xca\xb3\xaf\x7a\x15\xb4\x30\xe0\x34\xe7\x93\xbb\x30\xdb\x8a\xb2"),
  .iv = TEST_DATA_STR ("\x96\x3a\x56\xe2\xe1\x2f\x38\x70\x62\xe1\x84\x98"),
  .aad = TEST_DATA_STR (
    "\xa0\x94\xa1\xdd\x11\x21\xd3\xaa\x52\xc8\x1e\x8f\x10\xbf\x9f\x0c"),
  .tag = TEST_DATA_STR (
    "\x1a\x31\xd2\x95\x60\x1e\xb3\xc8\x2a\x54\xb2\x34\x98\x4f\xfd\xf5"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad128_tc13) = {
  .name = "128-GMAC 128-aad NIST CAVS TC13",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x89\xc9\x49\xe9\xc8\x04\xaf\x01\x4d\x56\x04\xb3\x94\x59\xf2\xc8"),
  .iv = TEST_DATA_STR ("\xd1\xb1\x04\xc8\x15\xbf\x1e\x94\xe2\x8c\x8f\x16"),
  .aad = TEST_DATA_STR (
    "\x82\xad\xcd\x63\x8d\x3f\xa9\xd9\xf3\xe8\x41\x00\xd6\x1e\x07\x77"),
  .tag = TEST_DATA_STR (
    "\x88\xdb\x9d\x62\x17\x2e\xd0\x43\xaa\x10\xf1\x6d\x22\x7d\xc4\x1b"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad128_tc14) = {
  .name = "128-GMAC 128-aad NIST CAVS TC14",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xa4\xd9\x94\xc4\xac\x5a\xc0\xf0\x29\x13\x24\x57\x14\xfb\xe2\x35"),
  .iv = TEST_DATA_STR ("\xa9\x47\x2d\xad\xcc\xa8\xd7\xe0\xe3\xb8\x08\x4d"),
  .aad = TEST_DATA_STR (
    "\xeb\x31\x8b\x9e\x17\x57\x52\x03\xdd\x29\xeb\xed\x20\xec\x82\xf9"),
  .tag = TEST_DATA_STR (
    "\x32\x3d\xf7\xf3\x36\x94\x10\x6f\x56\x73\x9d\xe0\x97\x32\x16\xa3"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad160_tc0) = {
  .name = "128-GMAC 160-aad NIST CAVS TC0",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x2f\xb4\x5e\x5b\x8f\x99\x3a\x2b\xfe\xbc\x4b\x15\xb5\x33\xe0\xb4"),
  .iv = TEST_DATA_STR ("\x5b\x05\x75\x5f\x98\x4d\x2b\x90\xf9\x4b\x80\x27"),
  .aad = TEST_DATA_STR ("\xe8\x54\x91\xb2\x20\x2c\xaf\x1d\x7d\xce\x03\xb9\x7e"
			"\x09\x33\x1c\x32\x47\x39\x41"),
  .tag = TEST_DATA_STR (
    "\xc7\x5b\x78\x32\xb2\xa2\xd9\xbd\x82\x74\x12\xb6\xef\x57\x69\xdb"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad160_tc1) = {
  .name = "128-GMAC 160-aad NIST CAVS TC1",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x95\x21\x17\x04\x8f\x77\xe2\x76\xc2\xef\x65\x80\x53\x7c\x14\x03"),
  .iv = TEST_DATA_STR ("\x07\x0b\x8f\xb4\x6a\x7a\xd5\x28\x85\xbe\x1b\x26"),
  .aad = TEST_DATA_STR ("\x34\xb0\x88\xf9\x82\x81\x8b\x5f\x07\xda\xbe\x2b\x62"
			"\xf9\x54\x7f\x4e\xd0\x99\x12"),
  .tag = TEST_DATA_STR (
    "\xbe\xdd\x4c\xf3\x0f\xd7\xa4\xab\xc4\x9b\xdc\xc3\xf3\xb2\x48\xb1"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad160_tc2) = {
  .name = "128-GMAC 160-aad NIST CAVS TC2",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x7f\x64\x53\xb3\x9b\xde\x01\x85\x60\xa1\x6a\x27\x04\x21\x75\x43"),
  .iv = TEST_DATA_STR ("\x0f\x3e\xec\xf4\x8d\x68\x35\x32\x26\xa7\x7f\xe4"),
  .aad = TEST_DATA_STR ("\x11\xe4\xec\xb2\x56\xeb\xff\x56\x45\x3f\xa2\xe7\x5e"
			"\x43\xeb\x9d\x64\x10\x49\xe6"),
  .tag = TEST_DATA_STR (
    "\xb5\x12\x62\x3a\x12\xd5\x49\x2b\x7d\x76\xd3\x9b\xe0\xdf\x57\x77"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad160_tc3) = {
  .name = "128-GMAC 160-aad NIST CAVS TC3",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x93\x32\xe4\x33\xbf\x61\x00\xc6\xcc\x23\xb0\x87\x10\x62\x7c\x40"),
  .iv = TEST_DATA_STR ("\xaa\xb3\xdb\x30\x15\xb2\x9d\x24\xf3\x29\xbe\xb4"),
  .aad = TEST_DATA_STR ("\xbd\x84\x3a\x08\xf0\xa8\x22\xf8\xf4\xf7\x6c\x36\x48"
			"\x38\x0a\xab\x76\x22\xe7\x19"),
  .tag = TEST_DATA_STR (
    "\xe5\x4f\x1d\x18\xc6\x1d\x8b\xe1\x54\x84\x72\x76\x05\xb5\xa5\xdc"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad160_tc4) = {
  .name = "128-GMAC 160-aad NIST CAVS TC4",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x57\x73\x75\x0a\x49\x30\x96\xa9\x9d\x84\xc0\x56\x3f\xc2\x93\xe9"),
  .iv = TEST_DATA_STR ("\xc3\x90\xed\x70\xdc\x94\x97\x23\x44\x13\xad\x52"),
  .aad = TEST_DATA_STR ("\x60\x12\x51\x72\x58\x71\x6c\x1f\x00\x35\xef\xa6\x0a"
			"\x0f\x36\xb5\xc6\x5e\x73\x79"),
  .tag = TEST_DATA_STR (
    "\xb0\x11\xb2\x64\x61\x0e\x58\x08\x27\x05\x47\x6f\x04\x0b\x8c\x86"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad160_tc5) = {
  .name = "128-GMAC 160-aad NIST CAVS TC5",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x41\xb0\xd0\xfc\xe5\xd3\x13\x59\xcf\xd5\xdb\x40\x64\xe2\xd4\x6b"),
  .iv = TEST_DATA_STR ("\xb9\x03\xe9\xd0\xce\xa2\x57\x95\xa8\x2e\x73\xe3"),
  .aad = TEST_DATA_STR ("\x4c\xba\x50\x18\x76\xf3\x3e\x1f\xda\x9c\xd4\x56\xe3"
			"\x18\x06\x83\xe3\x86\x3b\xd9"),
  .tag = TEST_DATA_STR (
    "\x18\xbc\x39\xd0\xb9\x5c\xf0\x59\xcd\x8c\x25\x00\x4f\x5e\x50\x7c"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad160_tc6) = {
  .name = "128-GMAC 160-aad NIST CAVS TC6",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x47\x48\xb7\x82\xe3\xfe\x5e\x4e\xff\xeb\x7c\x67\x23\x2d\x2b\x07"),
  .iv = TEST_DATA_STR ("\xc5\xe4\xdc\xf1\x8f\x86\x07\x6b\x88\xa5\xd5\xe9"),
  .aad = TEST_DATA_STR ("\x3b\x2f\xca\xd8\x73\x9e\xd8\x7e\x1d\x02\xe8\x08\x45"
			"\xf1\x20\xe2\x49\xea\x92\xb1"),
  .tag = TEST_DATA_STR (
    "\xb8\xae\x71\x8e\x28\x79\xc9\xcb\x65\x8d\x5d\x11\x22\xe6\x9b\xb7"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad160_tc7) = {
  .name = "128-GMAC 160-aad NIST CAVS TC7",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xe3\x0c\xc2\x20\x77\xd5\x95\x12\x16\xd0\x7f\x37\xc5\x1b\x58\xf9"),
  .iv = TEST_DATA_STR ("\xfc\x58\x3a\xd1\x59\xb5\x2e\x0b\x63\x78\x15\x7e"),
  .aad = TEST_DATA_STR ("\xc3\xcb\x7b\xe8\x88\x8e\xf4\x4c\xa5\xaa\x93\xdd\xe2"
			"\x6d\x27\x51\x28\x8e\x1f\x5a"),
  .tag = TEST_DATA_STR (
    "\xa8\xce\x25\xb5\xdc\x8f\x84\xe2\xf5\xda\xe5\xf0\x85\xaa\xcc\xd4"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad160_tc8) = {
  .name = "128-GMAC 160-aad NIST CAVS TC8",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x7c\x8b\x10\xba\x75\xee\x6a\xb4\xa9\x97\xd3\xf5\x98\xb7\x9d\x40"),
  .iv = TEST_DATA_STR ("\x6f\xb5\x51\x88\xdd\xf0\x0d\xde\x09\x59\x65\x87"),
  .aad = TEST_DATA_STR ("\x2d\xdc\x0a\xcf\x97\x05\xf8\xd1\x8f\x90\x5b\x8f\x9d"
			"\x47\x2e\x7d\xbf\x6b\x91\xe3"),
  .tag = TEST_DATA_STR (
    "\x57\x91\xd3\x80\x51\x09\xc5\xe1\x8a\xdf\xf4\xe8\x09\x06\xa0\x18"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad160_tc9) = {
  .name = "128-GMAC 160-aad NIST CAVS TC9",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x72\xc7\xdb\x6c\xa2\x9f\x83\x64\x1c\x3f\xff\x5b\x71\xc4\xbc\x30"),
  .iv = TEST_DATA_STR ("\xf2\x00\x07\x42\xe2\x49\xac\x56\xd5\xb2\xf6\x5f"),
  .aad = TEST_DATA_STR ("\xcd\x99\x4d\x2d\x08\x23\x27\x70\x92\x7d\x85\x4e\xf2"
			"\xb6\xca\x2f\x08\x73\x70\xcf"),
  .tag = TEST_DATA_STR (
    "\xa5\x96\x6d\xf3\x9f\xee\xba\x03\x36\xf0\xb9\xa3\xf4\xff\xe6\xc3"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad160_tc10) = {
  .name = "128-GMAC 160-aad NIST CAVS TC10",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x28\x33\xcc\x10\x19\x50\x30\xe4\xa1\x15\x55\x32\x66\x6c\xb0\x49"),
  .iv = TEST_DATA_STR ("\xad\x80\x2b\x9a\x5c\x94\x09\xfa\x3e\x7d\xcf\xcc"),
  .aad = TEST_DATA_STR ("\xb3\xec\xbe\xa2\x79\x7d\x00\x6c\x07\xb8\xce\x62\x1b"
			"\xe3\xb0\xec\xcd\x37\xc3\xec"),
  .tag = TEST_DATA_STR (
    "\x81\xde\xab\x8b\xde\xe0\xd3\x91\x49\x5e\xed\x40\x29\xa6\xd2\x05"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad160_tc11) = {
  .name = "128-GMAC 160-aad NIST CAVS TC11",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xd8\x98\x5b\xb5\xac\x02\x58\xad\xad\x86\x66\x0e\xbb\xc6\xd1\x9f"),
  .iv = TEST_DATA_STR ("\xb5\xee\x26\xf8\xc4\x63\xbb\xfc\x27\x11\x5b\x0a"),
  .aad = TEST_DATA_STR ("\x61\x3f\x51\xf8\x32\xfb\xf4\x34\xb8\xe3\xfe\x94\x54"
			"\xae\x46\xa8\x62\xd8\x31\xf0"),
  .tag = TEST_DATA_STR (
    "\xfe\x9f\x0b\x1b\xdc\x68\xde\xe6\xe8\xdc\x2c\xe1\x26\x65\xd3\x36"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad160_tc12) = {
  .name = "128-GMAC 160-aad NIST CAVS TC12",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x9b\x8f\x69\x24\xdc\x22\xf1\x07\x3c\x1a\x38\x44\x8a\x2f\x04\x47"),
  .iv = TEST_DATA_STR ("\x09\xcd\xab\xf8\x7d\x82\x82\x8e\xca\x1c\x0c\x7f"),
  .aad = TEST_DATA_STR ("\x69\x21\x0e\x4e\x0a\x1c\xfd\x50\x38\x75\x66\x52\x79"
			"\x0b\x9a\x8c\xfb\xbd\x94\x3d"),
  .tag = TEST_DATA_STR (
    "\xa6\x0c\x10\x4a\x6f\xb4\x63\x84\x27\xa8\x8a\x86\xc0\x49\x23\xbd"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad160_tc13) = {
  .name = "128-GMAC 160-aad NIST CAVS TC13",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x72\x13\x22\x13\xd5\xd9\x53\x09\xbf\x7e\x10\xf8\x31\x8d\x7c\x20"),
  .iv = TEST_DATA_STR ("\xfb\x90\xbf\x28\x3c\x54\x11\x23\x03\x55\xd7\xa1"),
  .aad = TEST_DATA_STR ("\xa3\x0b\xb1\x7c\x80\x89\xc6\xf5\xf6\x1b\x25\x0a\x94"
			"\xcb\xbb\xfd\xf5\xf2\xa3\xe6"),
  .tag = TEST_DATA_STR (
    "\x09\x19\x1a\xf4\x18\x94\x9f\xe6\xbe\x8d\xbf\x13\xe0\x06\x52\x7a"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad160_tc14) = {
  .name = "128-GMAC 160-aad NIST CAVS TC14",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x65\x2f\xfb\xad\x4e\x1f\xcb\xe7\x55\x64\x39\x5e\x6c\x1c\x39\x24"),
  .iv = TEST_DATA_STR ("\x11\x13\x49\x63\x6d\x10\x6f\xd5\xf6\xa1\xe0\x88"),
  .aad = TEST_DATA_STR ("\x5f\x52\xaa\x85\xdc\x3a\xc0\x42\x64\x7e\x32\xad\xa0"
			"\x50\xd6\x7e\x59\xb5\x19\xaa"),
  .tag = TEST_DATA_STR (
    "\x28\xd9\x80\xd7\xbf\xd8\x78\xc2\x27\xc1\x40\xde\x34\x82\x76\x5b"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad384_tc0) = {
  .name = "128-GMAC 384-aad NIST CAVS TC0",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x99\xe3\xe8\x79\x3e\x68\x6e\x57\x1d\x82\x85\xc5\x64\xf7\x5e\x2b"),
  .iv = TEST_DATA_STR ("\xc2\xdd\x0a\xb8\x68\xda\x6a\xa8\xad\x9c\x0d\x23"),
  .aad = TEST_DATA_STR (
    "\xb6\x68\xe4\x2d\x4e\x44\x4c\xa8\xb2\x3c\xfd\xd9\x5a\x9f\xed\xd5\x17\x8a"
    "\xa5\x21\x14\x48\x90\xb0\x93\x73\x3c\xf5\xcf\x22\x52\x6c\x59\x17\xee\x47"
    "\x65\x41\x80\x9a\xc6\x86\x7a\x8c\x39\x93\x09\xfc"),
  .tag = TEST_DATA_STR (
    "\x3f\x4f\xba\x10\x0e\xaf\x1f\x34\xb0\xba\xad\xaa\xe9\x99\x5d\x85"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad384_tc1) = {
  .name = "128-GMAC 384-aad NIST CAVS TC1",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xf8\xe2\x9e\xfd\x00\xa4\x23\xc4\xea\x94\x56\x86\x3f\x83\xc5\x4f"),
  .iv = TEST_DATA_STR ("\x2d\x3c\xf6\x7c\xbc\xe6\x9d\x63\x9b\xd1\xc0\x92"),
  .aad = TEST_DATA_STR (
    "\x02\xc7\x0f\xc8\xa2\x54\x46\x19\xc1\xc3\xe9\xfc\xe6\xb3\xc6\xc3\xbc\x24"
    "\x64\x3e\x0f\x14\x0e\x6b\x48\xac\x50\x5e\xa6\x66\xcd\x9a\x20\x10\xc3\xa8"
    "\xe2\xf5\xf1\x04\x37\x88\x7f\xe8\x03\xb5\x4d\xb3"),
  .tag = TEST_DATA_STR (
    "\x96\x3c\xb5\x0a\xca\x3e\x09\xdd\x0d\x9a\x01\x3c\x87\x34\x15\x5f"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad384_tc2) = {
  .name = "128-GMAC 384-aad NIST CAVS TC2",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x00\xe3\x49\x1d\xfc\xf3\xbe\xc3\x9c\x89\xcc\xfd\x80\xa5\xa8\x96"),
  .iv = TEST_DATA_STR ("\x29\xf6\xff\x4e\xdc\x4a\xc3\xe9\x7f\xfb\x16\x80"),
  .aad = TEST_DATA_STR (
    "\x73\x81\x33\x51\xb3\x9f\x5e\x40\x00\xa9\xee\x8d\x2b\x85\xf1\x31\x63\x4a"
    "\xca\xed\xe0\xdd\x25\xd6\x91\xa2\xb8\x29\xad\x4f\xe9\xea\x69\x9f\x12\x24"
    "\x25\x19\x84\x7c\xb0\x83\xb0\xb4\xd3\xd8\xb3\xbc"),
  .tag = TEST_DATA_STR (
    "\x01\xb2\xe9\xba\x71\x9a\xd7\x7c\x75\x3b\x36\x4e\xcc\x5a\xab\xeb"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad384_tc3) = {
  .name = "128-GMAC 384-aad NIST CAVS TC3",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x0a\xd0\x6f\x4c\x19\xaf\x1d\x5f\x60\x2b\x38\xf8\x6e\x56\x29\x1c"),
  .iv = TEST_DATA_STR ("\x0b\x23\x5c\x6a\x75\xce\xcd\xfc\xba\x90\x01\xce"),
  .aad = TEST_DATA_STR (
    "\x7d\x4f\x26\xf7\x89\x5b\x2e\xf3\xda\x2e\x4f\x93\xe4\x11\xcd\xb7\x40\x25"
    "\xc7\x75\x9c\x03\x8d\x87\x23\x44\xa4\x5c\xe5\x6d\x92\xa5\x81\x86\x2c\x3b"
    "\xac\xe0\x39\x09\x0a\x2c\xcf\xa4\x3b\x62\x3d\xcb"),
  .tag = TEST_DATA_STR (
    "\xb4\xbc\x9c\xe1\x47\x5d\x0c\x93\xdf\xd5\xa5\xd8\xd4\x5b\xd8\xe5"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad384_tc4) = {
  .name = "128-GMAC 384-aad NIST CAVS TC4",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xee\xeb\x33\xe0\xc8\xa4\x06\xea\x23\x6a\x07\x5c\xdb\xe9\xd6\xf9"),
  .iv = TEST_DATA_STR ("\xb9\x35\xe8\xee\xd6\x62\x27\x83\x6e\xde\x18\x9a"),
  .aad = TEST_DATA_STR (
    "\x9a\x42\x91\xac\xb9\x92\x4b\xba\x42\x41\xb0\xc9\xc3\xc2\xe1\x26\x2b\x25"
    "\xa7\xc7\xf0\x2c\x92\xad\xea\xdf\x92\x25\x4d\x61\x8a\xb5\x93\x88\xaa\x30"
    "\xb4\x7e\xaf\xa5\x88\x99\xc3\x57\xcf\x28\x1e\x31"),
  .tag = TEST_DATA_STR (
    "\x14\x3d\x69\x54\xeb\x6f\xe7\x0a\xff\x70\xda\x97\x8c\xcd\x45\x09"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad384_tc5) = {
  .name = "128-GMAC 384-aad NIST CAVS TC5",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x60\x0b\x54\x42\xa0\xb5\x50\xa3\x8f\x85\xd2\xfb\x0a\xcc\x9c\x96"),
  .iv = TEST_DATA_STR ("\x5e\x65\xdd\x6e\x8b\x20\xd6\xb2\x93\x1f\xe6\xc2"),
  .aad = TEST_DATA_STR (
    "\x46\x1e\x54\xa0\x92\xf8\x39\x24\x66\x84\x9f\xb0\x37\x0a\xe3\x0c\x14\xc1"
    "\xbf\x39\x87\xab\x2e\xbb\xe9\x8e\x18\xd1\x3f\x04\x1d\x09\xd0\x43\xf7\xae"
    "\xa7\x8b\xfc\xc4\x2f\x86\x4a\x9f\xb4\x0f\x00\x31"),
  .tag = TEST_DATA_STR (
    "\x2c\xd6\x26\xf9\xa0\x68\x63\x00\xcf\x23\xc0\xbc\x59\x7c\x63\xb4"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad384_tc6) = {
  .name = "128-GMAC 384-aad NIST CAVS TC6",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xce\x8d\x11\x03\x10\x0f\xa2\x90\xf9\x53\xfb\xb4\x39\xef\xde\xe4"),
  .iv = TEST_DATA_STR ("\x48\x74\xc6\xf8\x08\x23\x66\xfc\x7e\x49\xb9\x33"),
  .aad = TEST_DATA_STR (
    "\xd6\x9d\x03\x3c\x32\x02\x97\x89\x26\x3c\x68\x9e\x11\xff\x7e\x9e\x8e\xef"
    "\xc4\x8d\xdb\xc4\xe1\x0e\xea\xe1\xc9\xed\xbb\x44\xf0\x4e\x7c\xc6\x47\x15"
    "\x01\xea\xdd\xa3\x94\x0a\xb4\x33\xd0\xa8\xc2\x10"),
  .tag = TEST_DATA_STR (
    "\xa5\x96\x4b\x77\xaf\x0b\x8a\xec\xd8\x44\xd6\xad\xec\x8b\x7b\x1c"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad384_tc7) = {
  .name = "128-GMAC 384-aad NIST CAVS TC7",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xae\x71\x14\xc0\x9f\xfa\x04\x29\x88\x34\x41\x2f\x6a\x8d\xe4\x53"),
  .iv = TEST_DATA_STR ("\xf3\x80\xc2\xd8\x60\xbe\x2a\xf4\x1e\x1b\xe5\xc6"),
  .aad = TEST_DATA_STR (
    "\x7e\x16\x08\x2f\x68\x9c\x63\xe8\xad\xdd\xd5\xcb\x2d\xa6\x10\xbb\xfb\x88"
    "\xd0\x73\xcf\x8b\x20\x43\x84\xa9\x37\xaa\xb0\x37\x65\x23\xa5\x0d\x3d\x5f"
    "\x13\x92\x97\x8f\x79\x60\x9f\x12\xdf\x8f\xc2\x88"),
  .tag = TEST_DATA_STR (
    "\x40\xd3\xa3\x63\x58\xa6\xf6\xca\xaa\x6a\xf9\x2c\xfd\x87\x4a\x22"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad384_tc8) = {
  .name = "128-GMAC 384-aad NIST CAVS TC8",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xd8\xf5\x20\xb6\xf3\xcf\x6b\x83\x5c\xe4\xcc\xe4\x8f\x4c\xb0\x33"),
  .iv = TEST_DATA_STR ("\x01\x9a\x55\xc9\x86\x15\xc0\x22\xaf\xff\x96\x44"),
  .aad = TEST_DATA_STR (
    "\xc3\xfb\x51\x8d\xdb\x2d\x73\x41\x7e\x24\x33\x59\xa0\xed\x8c\x12\x67\x50"
    "\xeb\x16\x3e\x7b\xd8\x45\x63\x71\x59\x39\x70\x75\xe3\xdb\x1d\xb7\x2f\xe2"
    "\xf0\xe1\x3b\x59\x9c\x33\x3c\x47\x3f\xeb\x22\x45"),
  .tag = TEST_DATA_STR (
    "\x46\x7c\xfa\xd5\xaf\x11\x85\x2d\x6e\xca\x28\x9c\x86\xf9\x67\xad"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad384_tc9) = {
  .name = "128-GMAC 384-aad NIST CAVS TC9",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x13\xba\x95\x60\x6b\x01\xaf\x03\x5b\xf9\x61\xe3\x98\x52\xe3\x4b"),
  .iv = TEST_DATA_STR ("\x9e\xc9\xcf\x3b\x00\x2c\xfe\xd9\xe7\x61\x93\x4f"),
  .aad = TEST_DATA_STR (
    "\xbb\x9d\xe5\x63\x83\x6d\x1f\x1b\x1d\xe9\x64\x51\x4e\xce\xbb\x8a\xd1\x05"
    "\x01\xdb\x56\x22\x80\xb7\xbd\x98\x80\x48\x14\x73\x58\x17\x90\x8b\x28\x56"
    "\xca\xfa\xde\xcd\x40\xb0\x48\x32\xfb\xde\x2b\xfb"),
  .tag = TEST_DATA_STR (
    "\x17\x2a\x3b\xcb\xc5\x00\x1d\xfd\x38\x15\x17\x5a\x88\xf7\x05\x6c"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad384_tc10) = {
  .name = "128-GMAC 384-aad NIST CAVS TC10",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x1c\x97\xda\x5f\xc5\xa9\x64\x0f\x28\x96\x22\x84\x24\x08\xcb\xa2"),
  .iv = TEST_DATA_STR ("\x6d\x76\x5a\x98\x8e\x93\x45\x88\x16\x3e\x29\xb7"),
  .aad = TEST_DATA_STR (
    "\x10\x26\xa5\x90\x81\x6d\x2e\x1a\xa6\x7a\xa0\xd1\x3d\x50\xa8\x41\x3a\xf4"
    "\xd8\xee\x9b\x1f\xa5\xce\xb8\xde\xac\xc9\xf4\x1e\x8e\x76\x4b\x3a\xc1\x5f"
    "\x98\x29\x5e\x88\x00\xad\xf6\xa7\x17\x54\x48\xcd"),
  .tag = TEST_DATA_STR (
    "\x49\x45\xa7\x9d\x5e\xdb\xb9\x34\xc5\xcf\x94\x39\x5c\x35\x9d\xeb"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad384_tc11) = {
  .name = "128-GMAC 384-aad NIST CAVS TC11",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x8d\xd4\x6f\x27\x1a\x20\x1c\xc2\x1c\xa0\x82\x32\x48\x15\x7e\x6b"),
  .iv = TEST_DATA_STR ("\x18\x21\xb3\x10\xce\x2d\xba\x99\x9c\xdf\x75\x76"),
  .aad = TEST_DATA_STR (
    "\x34\xba\x40\x99\x97\xce\xba\x06\x5f\x4a\x54\x57\x07\x8a\x9e\x23\x2a\x84"
    "\xf5\x94\x01\x1a\xec\xfd\xbf\xbd\x24\xa8\x02\xca\x12\x9e\x01\xcb\x13\x27"
    "\xe2\x65\xb4\xa9\x00\x4f\xb4\xc5\x00\x3f\xff\xd3"),
  .tag = TEST_DATA_STR (
    "\x30\x4c\xc2\xcd\x2f\xcd\xd4\xab\xc8\x44\xbc\x9c\x1c\xbe\x02\x41"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad384_tc12) = {
  .name = "128-GMAC 384-aad NIST CAVS TC12",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x0c\x54\x5d\x95\x33\x3b\x6a\xcf\x8b\x29\x28\xf3\xef\xd0\x83\xde"),
  .iv = TEST_DATA_STR ("\x31\xde\x89\xd0\x7e\x75\x77\x95\x6f\xa9\x5e\xf3"),
  .aad = TEST_DATA_STR (
    "\x55\x74\xd6\x5f\x5a\xff\xfb\x2d\x31\xcc\xa8\xf5\x8c\xf5\x94\x5b\x83\x55"
    "\x3c\xd4\x5d\x2d\xba\x0e\x05\xfa\x54\xe4\x2a\xa3\xf5\xa0\x51\xe1\x62\x4d"
    "\xe1\x6d\x4b\x93\xcb\xab\x79\x88\xc6\xd9\x5f\x8c"),
  .tag = TEST_DATA_STR (
    "\x4e\xd9\x1c\xfe\x90\xa4\x99\x00\xe0\x56\x56\x97\xbc\x82\xb6\x59"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad384_tc13) = {
  .name = "128-GMAC 384-aad NIST CAVS TC13",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x79\x0b\x39\xf3\x01\x38\x3a\x82\xb3\x77\xf5\x85\xd3\xbf\x0f\x26"),
  .iv = TEST_DATA_STR ("\x2f\xd9\xc1\x42\xb5\xfc\x62\xe8\x7e\xff\xf1\xfd"),
  .aad = TEST_DATA_STR (
    "\x45\x63\x4e\x0a\xfc\x59\xae\x9f\x6e\x30\xf7\xf5\xfe\x43\xcf\x5a\x4e\x1f"
    "\x78\xd0\xae\xbb\x9e\x5a\x7a\xd9\xd8\x6f\x25\x27\x8e\x52\x1f\x48\x45\xd4"
    "\x9d\x6c\xb5\x33\xca\xc6\x43\x98\x39\x64\x7f\xd0"),
  .tag = TEST_DATA_STR (
    "\x69\x63\x7c\x3f\x92\x33\xda\x23\xf8\xdf\x7b\x09\xe8\xcf\xb2\x52"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad384_tc14) = {
  .name = "128-GMAC 384-aad NIST CAVS TC14",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x8f\x63\x65\x26\x32\xd0\x7b\x2a\x4a\x83\xc2\x6d\xed\xd3\x26\x57"),
  .iv = TEST_DATA_STR ("\x74\x7b\xee\x0e\x1d\x46\x2a\x90\x16\xf1\x46\x8d"),
  .aad = TEST_DATA_STR (
    "\x9c\x00\xff\x96\x9b\x55\xa4\x97\xdc\x52\x3f\xa0\xce\xda\xa3\x39\xdc\x3c"
    "\x6c\xe1\x8e\x61\xc7\xbf\x80\x0c\x36\x12\x01\x35\x1b\xc4\x97\x28\xc3\xbb"
    "\x15\x06\x7e\x90\x61\x62\xee\x79\x1b\x8d\x33\x3a"),
  .tag = TEST_DATA_STR (
    "\xbd\x5a\x0c\xbf\x85\x9a\x61\x33\xa7\xf2\xd5\x04\xd9\x7c\xae\x05"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad720_tc0) = {
  .name = "128-GMAC 720-aad NIST CAVS TC0",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x20\xb5\xb6\xb8\x54\xe1\x87\xb0\x58\xa8\x4d\x57\xbc\x15\x38\xb6"),
  .iv = TEST_DATA_STR ("\x94\xc1\x93\x5a\xfc\x06\x1c\xbf\x25\x4b\x93\x6f"),
  .aad = TEST_DATA_STR (
    "\xca\x41\x8e\x71\xdb\xf8\x10\x03\x81\x74\xea\xa3\x71\x9b\x3f\xcb\x80\x53"
    "\x1c\x71\x10\xad\x91\x92\xd1\x05\xee\xaa\xfa\x15\xb8\x19\xac\x00\x56\x68"
    "\x75\x2b\x34\x4e\xd1\xb2\x2f\xaf\x77\x04\x8b\xaf\x03\xdb\xdd\xb3\xb4\x7d"
    "\x6b\x00\xe9\x5c\x4f\x00\x5e\x0c\xc9\xb7\x62\x7c\xca\xfd\x3f\x21\xb3\x31"
    "\x2a\xa8\xd9\x1d\x3f\xa0\x89\x3f\xe5\xbf\xf7\xd4\x4c\xa4\x6f\x23\xaf"
    "\xe0"),
  .tag = TEST_DATA_STR (
    "\xb3\x72\x86\xeb\xaf\x4a\x54\xe0\xff\xc2\xa1\xde\xaf\xc9\xf6\xdb"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad720_tc1) = {
  .name = "128-GMAC 720-aad NIST CAVS TC1",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x7a\xa5\x31\x88\xa9\xc5\x97\x12\x6a\x10\xd2\x48\x60\x3e\xbb\x62"),
  .iv = TEST_DATA_STR ("\xaa\x45\xca\x5d\xac\x41\xa8\x25\xc4\x5d\x36\xbf"),
  .aad = TEST_DATA_STR (
    "\x41\x7f\xd5\x14\x7d\x56\xde\x0c\x74\x32\x95\x97\x82\x4e\xc2\x78\x8a\x34"
    "\x4f\xb6\x0b\x40\x3e\xdf\x01\x87\xaf\xa1\x2e\x72\xa0\x50\x09\xbb\x70\xf8"
    "\x3c\xca\xd1\x1e\xfa\x48\x7c\x19\x65\xcf\x84\xfe\xac\x06\x7c\x1f\xfd\xbf"
    "\x53\x1f\xca\x97\xc5\x54\xf8\x75\xc4\xa1\xa1\xd3\xab\x3c\x53\xc8\xa7\x4e"
    "\xf3\xee\x94\x15\xa8\x7e\x23\x16\x99\xc8\x2d\x76\x4d\xeb\xed\xa1\x81"
    "\x32"),
  .tag = TEST_DATA_STR (
    "\x99\x7b\xf8\x46\x54\xbb\x96\x16\xc0\xcc\x9b\x45\xf8\x2c\x76\x73"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad720_tc2) = {
  .name = "128-GMAC 720-aad NIST CAVS TC2",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x72\xb5\x84\x8e\xd1\xd2\xba\xdb\xd4\x27\xe1\x6f\xc3\xb3\xe4\x4d"),
  .iv = TEST_DATA_STR ("\xa8\x4c\x7e\x92\x8d\xc6\xe6\x37\x9a\x51\x3a\x20"),
  .aad = TEST_DATA_STR (
    "\x1c\x0d\xfc\xec\xbd\x7b\xb0\xe6\x80\xce\x04\x2d\x08\xb2\xd9\xa7\x41\x26"
    "\x7b\xd1\xda\x76\x8d\xf2\xba\x08\x37\x92\x33\xa9\x97\x3f\x14\x92\x8e\x9d"
    "\xa6\x35\x37\x68\xb9\xb2\x60\x1c\x03\x3f\xd9\x64\xb1\x6a\x16\xda\xaa\x3e"
    "\xa3\x5a\xd7\xce\xf7\xe3\x1e\xb1\xf7\x34\x0a\xa3\x4e\x8b\xfc\x08\xb0\xa6"
    "\xe6\x20\x52\x92\x57\x0c\xed\x43\x31\x68\x76\xd0\xd4\x99\xd9\x19\x2e"
    "\x6b"),
  .tag = TEST_DATA_STR (
    "\x27\x0c\xd7\x86\xb9\x5e\x68\x20\xcd\xb6\x5a\x23\x1b\x75\x30\xed"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad720_tc3) = {
  .name = "128-GMAC 720-aad NIST CAVS TC3",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x6d\x05\x12\xeb\xf2\xe7\x3d\x63\xf4\x28\x49\xc5\x7f\x07\x3f\xd0"),
  .iv = TEST_DATA_STR ("\xc1\xc4\x69\x27\xc7\x4c\x03\xf1\x93\x42\xc3\x3a"),
  .aad = TEST_DATA_STR (
    "\x28\xbf\x89\x03\xb2\xdf\xb7\xe6\x9f\x1a\x73\x51\x21\xc7\xef\xe9\xa4\xc4"
    "\x2b\x6a\x29\x53\x27\xbc\xeb\x02\x46\xc8\x5d\x78\x2c\xe6\x2b\xf0\x75\xdb"
    "\xdf\x6e\x8e\xc6\x58\x9c\x26\xd3\x06\x96\xcc\xce\xef\x03\x87\x0b\xd0\xab"
    "\xfd\x26\xd3\x06\x00\xea\xfc\x65\x61\x37\x40\xb5\x4d\x77\x7d\x37\x9e\x8a"
    "\xac\xf2\x41\xec\xfb\xa1\x1b\x06\x01\x86\xac\x06\x5d\xb1\x71\xaa\xb0"
    "\x99"),
  .tag = TEST_DATA_STR (
    "\xa6\x86\xf5\x94\x1c\xeb\x51\x0e\x12\x6a\x63\x16\xe3\x40\x4d\xc0"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad720_tc4) = {
  .name = "128-GMAC 720-aad NIST CAVS TC4",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x64\x38\xbc\x79\x52\x0d\xef\x5d\xb5\x8e\x49\x63\x97\x74\x68\x7a"),
  .iv = TEST_DATA_STR ("\xd6\x82\xb4\x74\x18\xce\xb5\xbc\x09\xc7\x13\xc2"),
  .aad = TEST_DATA_STR (
    "\xd2\x52\xb1\x64\xae\x55\x9e\xd1\x55\xc8\x41\x7b\x96\x65\x25\x29\xdf\x15"
    "\x1f\x24\xcc\xf1\xce\x98\xd0\xc7\xdd\xf2\x93\xf4\xf1\x23\x66\x30\xa1\x9b"
    "\x24\xdc\x23\x97\x8d\x33\x77\xa0\x99\x06\x5d\x0b\xa7\x1d\x4b\xb8\xa7\xdc"
    "\x0c\xb7\x67\x60\xca\x7c\x4a\x0e\x12\xc8\xcb\x56\xc6\x10\x26\x46\x32\x3c"
    "\x08\xc4\xf4\xf5\x62\x26\xfd\x5b\x71\xa8\x45\x90\x91\x3a\xd2\x0d\xa2"
    "\x87"),
  .tag = TEST_DATA_STR (
    "\x04\xe7\x87\x96\xdb\xf4\x2e\x9f\xfa\x6b\xb9\xe3\x46\x58\x1f\x13"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad720_tc5) = {
  .name = "128-GMAC 720-aad NIST CAVS TC5",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x11\x7a\x0a\xa5\x92\xff\xf1\x7a\xe3\x6c\x94\x91\x7d\xb1\x6c\x65"),
  .iv = TEST_DATA_STR ("\xc3\x53\x7b\xe6\x02\x9d\x54\xff\xef\xab\x27\x30"),
  .aad = TEST_DATA_STR (
    "\x29\xe9\x59\xb9\x68\x17\x54\x7a\xe0\x6b\xf8\x5f\xe1\x64\xe8\x2a\x26\x93"
    "\xf8\x2a\x7a\xeb\x66\xd5\x35\xf0\xd2\xc3\xbf\xfd\x1b\xa1\x8e\x94\xef\x45"
    "\x79\x39\xf0\xc0\x73\x3e\xda\x47\x38\xd1\x36\x38\x0f\xc8\x76\x07\x5c\x49"
    "\x43\x22\x02\x37\xa5\x92\x9b\x01\xb3\x2d\xa2\xbc\x2a\x6a\xfd\x6a\xe1\xd8"
    "\x9f\xd4\x70\x09\x38\x35\x96\x2f\xf6\x70\x8b\xb3\x9b\xa3\x65\x20\x2f"
    "\x56"),
  .tag = TEST_DATA_STR (
    "\xb8\x7f\xcc\x4d\x5c\x48\x4e\x68\xea\x52\xc0\x1b\x55\xff\xa4\x38"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad720_tc6) = {
  .name = "128-GMAC 720-aad NIST CAVS TC6",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x5d\x99\x5a\x33\x8e\xd6\x0f\x8a\xb0\xb5\x9d\xa6\xc9\xa4\x0c\x52"),
  .iv = TEST_DATA_STR ("\x27\x23\xc5\x4e\x31\xc5\xc5\x7f\x02\x36\xe8\x16"),
  .aad = TEST_DATA_STR (
    "\x23\x9c\x80\x68\x3f\xeb\x6a\xfd\x38\xf8\x75\x9a\x27\xcb\x5f\x35\x0f\xbc"
    "\x2f\x75\x78\x38\xc4\x08\x58\xc9\xd0\x8f\x69\x9c\xc5\x6c\x42\x36\xf4\xa7"
    "\x7b\xd8\x0d\xf0\xe8\xe4\x1d\x5f\x9b\xa7\x32\xdb\x2e\x0a\x3a\x5e\x95\x2e"
    "\xde\x7b\xfd\xd5\xfc\xbe\xbd\x23\xd0\x72\x71\x13\x4d\xb5\xb8\x24\x61\x53"
    "\x7c\x47\xe2\xca\x51\xb3\x48\xb0\x83\x0f\x5e\xe5\x75\xad\x4b\x44\x14"
    "\xdc"),
  .tag = TEST_DATA_STR (
    "\x94\x35\x6a\x3b\xfa\xf0\x7f\x2e\xf0\xeb\xe3\xa5\x07\x07\x6b\x16"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad720_tc7) = {
  .name = "128-GMAC 720-aad NIST CAVS TC7",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xc8\xa8\x63\xa1\xeb\xaf\x10\xc0\xfc\x0e\x80\xdf\x12\x44\x4e\x6e"),
  .iv = TEST_DATA_STR ("\xc3\xe8\xcd\xf0\x86\x82\x7f\xee\x70\x95\xd0\xea"),
  .aad = TEST_DATA_STR (
    "\x99\x27\xda\x88\xc5\xd3\x36\x25\x66\x99\xc7\x68\x45\xe9\x46\xdc\x53\xc8"
    "\x7b\xf0\xe1\x1e\x4b\xec\x94\x50\x98\x16\x02\xb3\x20\x10\xd2\xb5\x2b\xfc"
    "\x91\x28\x3a\x63\x29\xd4\x55\x59\x89\x98\xed\xe2\xe6\x1e\x35\x2e\x55\x31"
    "\x10\x15\x4b\x4d\xa5\xce\x66\x8d\x66\x4b\x83\xf6\x71\xc0\x10\xbf\x22\x0b"
    "\x7d\x32\xb3\x4f\x4c\xa6\x9b\x66\xcc\x87\x23\x3d\x79\x23\x37\xcb\x2b"
    "\xff"),
  .tag = TEST_DATA_STR (
    "\x09\x88\x37\xde\x27\x70\x7e\xa3\x59\x3e\x31\xce\xb8\x27\x67\x32"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad720_tc8) = {
  .name = "128-GMAC 720-aad NIST CAVS TC8",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x69\xcc\x28\xb1\x61\xf2\x14\xa5\x80\xe6\xba\x4b\xc2\xe3\xde\x9d"),
  .iv = TEST_DATA_STR ("\xf2\xa5\x66\xf9\xcf\x83\xfd\x28\x0c\x8f\xe0\x8e"),
  .aad = TEST_DATA_STR (
    "\xf8\xc5\x26\x3a\x4e\x06\xb4\x9e\x18\x45\x89\xa1\xe0\x71\x97\x86\x43\xc3"
    "\x53\xaa\x27\xb4\x81\x7f\xe3\x9e\x45\xab\xc4\x42\xe2\x2a\xb5\xd6\x83\xbc"
    "\xee\x5d\xbb\xd5\x89\xfa\x58\x3f\x17\x1b\xb5\x95\x36\xad\xdd\x2b\x6c\xef"
    "\xd4\x98\x23\x41\x30\x05\xef\xb2\xa6\x65\xe2\x6a\x60\x29\xc9\x27\xd3\x89"
    "\x1c\xb0\xd4\xf2\x3e\x8c\xcc\x60\xcf\xd0\x2c\xe8\x97\x8c\x45\x1d\xdc"
    "\x11"),
  .tag = TEST_DATA_STR (
    "\xc9\xc8\x06\xcb\x8b\x1a\x88\x98\x09\x69\x5c\x2e\xc5\xa7\xa8\x6e"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad720_tc9) = {
  .name = "128-GMAC 720-aad NIST CAVS TC9",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xbb\xf3\x59\x20\xfc\xab\x2c\xed\xaa\xfd\xf3\xf0\x03\x21\xf5\x44"),
  .iv = TEST_DATA_STR ("\x2c\x7e\xe3\xff\x1d\xf8\x4f\x36\x50\xbc\x92\x98"),
  .aad = TEST_DATA_STR (
    "\xa7\x5f\x50\xba\x9a\x50\xf4\x87\x99\x59\x4b\x61\x95\xb3\x12\x5e\xd9\x2d"
    "\xf7\x31\x44\xbf\xcb\x62\x4c\xe6\x73\x23\xd8\x34\xba\x1a\xfa\xf0\xdf\x4c"
    "\x6c\x02\x2c\x11\xd4\x8b\xd7\x5c\x86\x67\x5a\x59\x27\xac\x12\x50\x03\x0f"
    "\x72\x0f\x97\x49\x8d\x4f\xe0\x78\x7b\xae\x65\x5d\xc5\x53\x7a\xc1\xbc\xac"
    "\x19\x8a\x89\x3f\x9a\xf7\xc2\xef\x9b\x97\x1d\xd6\x4f\x7e\x7b\x62\x60"
    "\x3e"),
  .tag = TEST_DATA_STR (
    "\xc7\xcd\x3f\x93\x8f\x4a\xb1\x86\x42\xd8\x62\x34\xed\xfc\x17\xed"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad720_tc10) = {
  .name = "128-GMAC 720-aad NIST CAVS TC10",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x96\x90\xde\x66\x97\x02\xba\x72\xae\xb9\x34\xf5\xac\x50\xe0\x3c"),
  .iv = TEST_DATA_STR ("\xda\x87\x13\xfe\x2b\x20\x58\xc4\x38\xaf\xf2\x60"),
  .aad = TEST_DATA_STR (
    "\xf3\x0e\xe9\x50\xda\x37\xc7\x22\x4b\x5c\x93\xe9\xa2\x9c\xaf\xdb\xf8\xe2"
    "\x07\x0f\x65\xc2\x26\x24\x4b\x1a\x68\x34\x59\xe0\xc5\xc1\x1c\x9b\x77\xc8"
    "\xfc\x28\x6d\x42\x98\xa5\xb9\xcd\x1f\xee\x3e\x13\xd4\x69\x0a\x88\x78\x0d"
    "\x35\xb5\x58\xb5\xd9\xe5\x2b\x1a\x67\xfc\x88\x57\x07\x66\x91\xdc\xa7\xf5"
    "\xfe\x8e\xf2\x20\x65\xcc\x5d\x9c\x00\x3f\xfd\x25\xeb\xe2\x3e\x61\x44"
    "\x0e"),
  .tag = TEST_DATA_STR (
    "\x7f\x92\x91\x45\x18\xdd\xbe\x84\x2b\x06\x77\x1f\x64\xc4\x0f\x59"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad720_tc11) = {
  .name = "128-GMAC 720-aad NIST CAVS TC11",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xe5\xd8\xc6\xe2\xac\x69\x35\xc8\x5e\x81\xee\x0e\xf7\x23\xea\xcf"),
  .iv = TEST_DATA_STR ("\xc7\x31\x40\xee\x90\xcc\x1d\xcf\x88\x45\x7d\xa2"),
  .aad = TEST_DATA_STR (
    "\xf6\xc2\x67\xa6\xae\x5c\xe3\xcf\x4b\xcd\xf5\x9c\xfd\x1f\x77\x7c\x66\x13"
    "\x3e\x0e\xc4\x77\x27\x85\xf3\x3e\x5f\xa8\x00\xd3\x10\xb2\x4b\x57\x73\xbc"
    "\x60\x3a\x76\xb3\x0f\xc3\x23\x28\xa8\xe4\x0f\x02\xf8\x23\xa8\x13\xa9\xe4"
    "\xb4\xfa\xc7\x26\xe9\x92\xc1\x83\xbd\x08\x15\x11\x1c\x1d\x3a\x35\x88\x4a"
    "\x4e\xff\x32\x02\x7b\xa6\x0d\xba\x67\x9b\x46\x9a\xf3\x1b\xc5\x0c\x05"
    "\x91"),
  .tag = TEST_DATA_STR (
    "\xf9\x38\xfd\x0d\x8c\x14\x8d\x81\x76\x51\x09\xdf\x66\xda\xc9\xaa"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad720_tc12) = {
  .name = "128-GMAC 720-aad NIST CAVS TC12",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xe2\x34\x58\xf6\xb3\x04\xc2\xd8\xfe\xb3\xde\xdd\x37\x41\xbc\x24"),
  .iv = TEST_DATA_STR ("\x46\x19\x03\x6b\x50\xba\x01\x2f\xe5\x0b\xe1\xd7"),
  .aad = TEST_DATA_STR (
    "\x74\xbf\xdc\x6b\xc4\xbf\xc3\x8d\x66\x6b\x98\x5c\xfe\x04\x3c\x67\x79\x8b"
    "\x2d\xb9\x8f\x14\x92\x68\xdb\xa2\x44\x36\xca\xb8\x3e\x9a\x91\xf2\x44\xff"
    "\xc5\x74\x8c\x93\xf8\xdf\x33\x9a\xe2\x4b\xa4\x31\x8c\x50\xda\x01\x1a\xb3"
    "\x68\xd3\x16\x7c\x16\xe5\x03\x30\x9b\x01\x35\x1a\x11\xf1\x4d\x06\x7c\xc6"
    "\x76\x9b\x99\x89\xc7\xd9\x52\xe3\x31\x50\x11\xee\x2e\xa0\x34\xdb\x8c"
    "\xb8"),
  .tag = TEST_DATA_STR (
    "\x60\x53\xab\x80\xc7\x46\x82\x1e\xc5\x0c\x97\xe5\xa1\x42\x4a\x85"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad720_tc13) = {
  .name = "128-GMAC 720-aad NIST CAVS TC13",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x53\x72\xac\x5d\x3b\x08\xd8\x60\x91\x91\x10\xbd\xeb\x7f\x31\xdf"),
  .iv = TEST_DATA_STR ("\x06\xca\x97\x9d\x8c\x25\x0d\x9b\x7b\xe4\x55\x73"),
  .aad = TEST_DATA_STR (
    "\xe1\xf9\x58\x83\x4e\x63\xc7\x5c\x8c\x75\x8b\xaf\xaa\x2f\x25\x7e\xa5\x68"
    "\x9d\x0d\x55\xb8\x77\xb4\xd6\x7b\x8b\x73\xc2\x5c\xe2\x4e\x9b\x09\x4b\x97"
    "\x6d\xb9\x20\xa1\x59\x96\x8d\xa9\xd3\x3c\x51\x1a\xa8\x99\x9a\xba\x42\xb8"
    "\xbb\x88\x6e\x65\x45\xdd\x10\x86\x93\x15\x0a\xf3\x57\x49\x6b\xb5\x89\x8b"
    "\x4e\x8f\x72\x5d\x50\xef\x47\x4a\xfb\x83\x6a\x33\x58\xda\x22\x17\xbb"
    "\x93"),
  .tag = TEST_DATA_STR (
    "\x93\x38\xe1\x4f\xe0\xb0\x8a\x96\x9a\x10\x4c\x82\x85\x28\xa6\xa4"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac128_aad720_tc14) = {
  .name = "128-GMAC 720-aad NIST CAVS TC14",
  .alg = VNET_CRYPTO_ALG_AES_128_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xbf\x1c\xb4\x9e\x98\x0c\xec\x0b\x15\x3f\xe3\x57\x38\x75\xac\x6c"),
  .iv = TEST_DATA_STR ("\x54\x26\x66\x9d\x25\x52\x40\x36\xfb\xe8\x1e\x89"),
  .aad = TEST_DATA_STR (
    "\xb3\x36\x94\x97\x66\xe9\x94\x8a\x7e\x6f\x36\xa2\xd3\x77\xb8\x4a\x25\xc4"
    "\xb4\x98\x87\x94\xf3\xde\xab\x7a\xf4\xb1\x4a\x12\xda\xc6\x41\xe2\x5f\xe2"
    "\xae\x9f\xf5\x34\x50\xac\xe1\x51\x3a\xcd\x0b\x28\x4a\x49\x0b\x45\x5f\x04"
    "\xf4\x0a\xf9\x44\x18\xc8\x79\x2e\xc1\xa0\x98\x3f\xb1\xd9\xa3\x1d\x93\xdc"
    "\x3e\xd2\xc7\x5e\x6a\x6c\xe0\x92\x11\x1e\xab\xad\x03\x9b\xac\x2a\x49"
    "\xf6"),
  .tag = TEST_DATA_STR (
    "\xe2\x99\x6a\x2b\x3b\x6b\xf5\x22\x17\xcf\xc4\xd0\xf5\xbb\x35\x1b"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad0_tc0) = {
  .name = "192-GMAC 0-aad NIST CAVS TC0",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xaa\x74\x0a\xbf\xad\xcd\xa7\x79\x22\x0d\x3b\x40\x6c"
			"\x5d\x7e\xc0\x9a\x77\xfe\x9d\x94\x10\x45\x39"),
  .iv = TEST_DATA_STR ("\xab\x22\x65\xb4\xc1\x68\x95\x55\x61\xf0\x43\x15"),
  .tag = TEST_DATA_STR (
    "\xf1\x49\xe2\xb5\xf0\xad\xaa\x98\x42\xca\x5f\x45\xb7\x68\xa8\xfc"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad0_tc1) = {
  .name = "192-GMAC 0-aad NIST CAVS TC1",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x1b\xb1\xd6\xa3\xbf\xc7\x48\x78\x6f\x39\x51\xe4\x3c"
			"\x18\x05\x4b\xfc\x8c\xe6\xab\x3d\xc3\xd3\x98"),
  .iv = TEST_DATA_STR ("\xfe\xa5\x6a\x5c\xe5\xf7\xd4\xc8\x16\x80\x19\x5d"),
  .tag = TEST_DATA_STR (
    "\x0b\x0b\xc0\x76\x8b\x02\xf1\x26\xa2\x9b\xcb\x14\x4a\xbc\x6e\x4b"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad0_tc2) = {
  .name = "192-GMAC 0-aad NIST CAVS TC2",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xb6\x28\x0d\x41\xff\x79\xec\x0a\x8d\xb1\x7c\x1e\x22"
			"\xf4\x3c\x5f\xdc\x92\x89\x88\x46\xe6\x46\xb5"),
  .iv = TEST_DATA_STR ("\xc4\x78\x88\xc1\x48\xb7\xdc\x05\x5f\x99\xaa\x08"),
  .tag = TEST_DATA_STR (
    "\xe3\x05\xfa\x02\x47\x24\x60\x6e\x14\x03\x26\x95\x9b\xfc\x53\x18"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad0_tc3) = {
  .name = "192-GMAC 0-aad NIST CAVS TC3",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x02\x87\xa6\xe3\xf9\x00\x56\xcb\x4b\x3d\x3f\x72\x23"
			"\xe8\xd0\xaf\x58\xc0\x9f\x15\x6c\xb3\xb8\x05"),
  .iv = TEST_DATA_STR ("\xea\x0c\xd8\x6c\x79\x7c\x34\x2c\xfb\xb5\xa9\xcc"),
  .tag = TEST_DATA_STR (
    "\x54\x9e\xc0\xd0\xff\xcf\xc1\x38\x1e\xd7\x09\xc3\xea\x8e\xac\xda"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad0_tc4) = {
  .name = "192-GMAC 0-aad NIST CAVS TC4",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x47\x3e\xa0\xd2\x98\xf6\x45\xcb\x01\xfa\x64\x83\x7b"
			"\x47\x09\xb2\x83\x74\xf3\xe9\x55\xe7\xfb\xa2"),
  .iv = TEST_DATA_STR ("\x25\x5b\x70\x4f\x33\xad\x24\xbb\x93\xd2\x67\x3a"),
  .tag = TEST_DATA_STR (
    "\xec\xbe\xe0\x40\xa6\xba\x2b\xd2\x71\x0f\x0d\xa5\x8b\x10\x96\xc2"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad0_tc5) = {
  .name = "192-GMAC 0-aad NIST CAVS TC5",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x6d\xa0\xbd\x47\x43\x24\x4d\xc4\xb3\xbe\x99\x3a\xb3"
			"\x95\x4c\xcd\xc9\x00\x77\xff\x31\x1e\xa6\x57"),
  .iv = TEST_DATA_STR ("\xd1\x9f\xaf\x29\xfe\xaf\xd0\x70\x83\x94\x38\x16"),
  .tag = TEST_DATA_STR (
    "\xbd\xd8\xdb\xa9\x1c\xf8\xaf\xfc\x30\x9f\x91\xb3\xe2\x39\x18\xaa"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad0_tc6) = {
  .name = "192-GMAC 0-aad NIST CAVS TC6",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xc9\xaa\x36\xf1\x7b\x9e\x57\xcf\xac\xb9\x28\xe1\x34"
			"\x66\x00\xc2\x2d\x36\x7d\x18\x28\x54\xb4\x64"),
  .iv = TEST_DATA_STR ("\x6d\x12\x0c\x4a\x05\xe2\xb4\x52\xcc\x22\x13\xc4"),
  .tag = TEST_DATA_STR (
    "\xd2\x3a\x67\x29\x61\x9f\xa7\xc3\x58\x79\x4b\x63\xbf\x5e\xe8\xd7"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad0_tc7) = {
  .name = "192-GMAC 0-aad NIST CAVS TC7",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x5e\xde\xf8\xc4\x17\x1c\x44\x58\x7c\xbb\xf0\xf4\x2d"
			"\xc7\xdc\x05\xb3\x6c\x84\x91\xd6\x7c\x75\x82"),
  .iv = TEST_DATA_STR ("\x21\xf2\x65\xfe\x1e\x26\xff\xe4\x91\xa0\xe5\x94"),
  .tag = TEST_DATA_STR (
    "\x9e\x73\xbc\x5f\x26\xd5\xf1\xc8\x5f\xd5\xc0\xdf\x48\x63\x61\x04"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad0_tc8) = {
  .name = "192-GMAC 0-aad NIST CAVS TC8",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x44\x17\x16\x09\x6e\x80\xa1\xa8\x62\x86\xd2\xe1\xdf"
			"\x48\x94\xae\xf5\xfa\x91\x4d\x7f\x6b\xde\xda"),
  .iv = TEST_DATA_STR ("\xb0\x10\x72\xd8\x25\xeb\x24\xba\x3c\x0a\xbc\x95"),
  .tag = TEST_DATA_STR (
    "\x28\xaf\x88\xfe\x13\x40\x68\xe9\xc8\x44\x5a\x19\x47\x84\x3e\xd2"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad0_tc9) = {
  .name = "192-GMAC 0-aad NIST CAVS TC9",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xd3\x6e\x99\x04\xe0\xab\x25\x51\xc8\xa4\x12\x56\xbc"
			"\x66\xad\x25\x37\xf4\xd7\x8c\x56\x18\x33\x73"),
  .iv = TEST_DATA_STR ("\xfc\xc8\x50\xfa\xc3\x38\x79\x4f\x3a\xdf\x50\x50"),
  .tag = TEST_DATA_STR (
    "\xa8\x8c\x92\xd6\xec\x0a\xbe\x95\x0c\x14\x7a\xf3\xb6\xbf\xae\xca"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad0_tc10) = {
  .name = "192-GMAC 0-aad NIST CAVS TC10",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x50\x04\xb2\x3a\xa5\x4b\x45\x3c\xe7\x98\xd0\xfa\xe7"
			"\x07\x35\x00\xc4\xfb\xd4\x94\xc3\x50\xc5\xd0"),
  .iv = TEST_DATA_STR ("\x31\x11\x9a\xc3\x7e\x06\x63\x25\x05\x48\xd8\x9a"),
  .tag = TEST_DATA_STR (
    "\x1b\x47\x45\x91\x10\x76\x4a\xae\x49\xf9\x44\xaf\x0c\x74\xd1\xf3"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad0_tc11) = {
  .name = "192-GMAC 0-aad NIST CAVS TC11",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xd7\xfe\x59\x41\x1c\x9a\x7e\xf5\x5c\xf5\x6e\xee\xe2"
			"\xd9\x60\xc5\xcd\x65\x5f\x4f\xab\xce\x69\x69"),
  .iv = TEST_DATA_STR ("\x56\x00\x39\x99\x1d\x07\x81\x71\xb8\x6e\x2e\x36"),
  .tag = TEST_DATA_STR (
    "\x33\x25\xa6\x83\x1b\x9a\x8d\xa5\x26\xad\x3a\x9c\x30\xbf\x89\x64"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad0_tc12) = {
  .name = "192-GMAC 0-aad NIST CAVS TC12",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x00\x60\xc6\x4b\x6b\x16\x81\xe4\x60\x30\x0a\x17\x63"
			"\x39\x88\xee\xfc\x6f\xc2\xcb\xd4\x7a\xe6\xc9"),
  .iv = TEST_DATA_STR ("\x14\xd0\x54\x6f\xda\x5d\x9c\x36\x46\x18\x9d\xd4"),
  .tag = TEST_DATA_STR (
    "\x66\x78\x4d\x25\xfb\x39\xfa\xcb\xdd\x80\xae\xfa\x7d\xa0\xf0\x2f"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad0_tc13) = {
  .name = "192-GMAC 0-aad NIST CAVS TC13",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x9c\x4f\x21\xe3\x7b\x91\xfe\x41\x9e\x35\xb6\xfc\xdc"
			"\x4e\x70\xd0\x32\x55\x75\xf9\x11\xc1\x3b\x43"),
  .iv = TEST_DATA_STR ("\xf5\x7a\xf6\x28\x46\xb2\x71\xe8\x02\xd6\x70\x1f"),
  .tag = TEST_DATA_STR (
    "\x6b\x1e\x00\x9e\x6e\xdf\x78\x9f\xc9\x43\x85\x73\x4d\xd5\x7d\x2f"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad0_tc14) = {
  .name = "192-GMAC 0-aad NIST CAVS TC14",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x8b\xed\xa5\x6e\xe0\xe1\xe6\xd8\x09\x8b\xa2\x67\xf0"
			"\x9b\x96\xd8\x9a\x3a\x46\x22\xa6\x41\xe7\x9d"),
  .iv = TEST_DATA_STR ("\x81\x29\x96\x6d\x15\xbd\xb7\x0e\x0d\x2f\xcc\xef"),
  .tag = TEST_DATA_STR (
    "\x28\x73\xdf\x0e\x03\x54\x86\x46\x81\x46\x30\xe0\xca\xc1\xe4\xe5"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad128_tc0) = {
  .name = "192-GMAC 128-aad NIST CAVS TC0",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x41\xc5\xda\x86\x67\xef\x72\x52\x20\xff\xe3\x9a\xe0"
			"\xac\x59\x0a\xc9\xfc\xa7\x29\xab\x60\xad\xa0"),
  .iv = TEST_DATA_STR ("\x05\xad\x13\xa5\xe2\xc2\xab\x66\x7e\x1a\x6f\xbc"),
  .aad = TEST_DATA_STR (
    "\x8b\x5c\x12\x4b\xef\x6e\x2f\x0f\xe4\xd8\xc9\x5c\xd5\xfa\x4c\xf1"),
  .tag = TEST_DATA_STR (
    "\x20\x4b\xdb\x1b\xd6\x21\x54\xbf\x08\x92\x2a\xaa\x54\xee\xd7\x05"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad128_tc1) = {
  .name = "192-GMAC 128-aad NIST CAVS TC1",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xae\xf7\x60\xf0\xcb\x88\x11\xf4\xab\x4a\x05\xfa\xa3"
			"\x5f\xe8\xb9\x85\x4a\xde\x54\x8e\x04\x0e\x7f"),
  .iv = TEST_DATA_STR ("\xd5\xda\xed\xc1\xd4\x95\x9a\x5d\x74\x4b\xc5\xf2"),
  .aad = TEST_DATA_STR (
    "\xae\xa8\xce\x76\xe3\xcf\x40\xd4\x73\xf6\x1a\x08\xd5\x9e\x53\xf5"),
  .tag = TEST_DATA_STR (
    "\x92\x6c\x61\xde\xf0\x2e\xd3\x0e\xd1\x58\xe3\x55\xac\x5d\x57\x10"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad128_tc2) = {
  .name = "192-GMAC 128-aad NIST CAVS TC2",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x12\x00\x0b\x16\x46\x22\x25\x44\x6c\xfe\x6e\x80\xd8"
			"\x4d\x47\x1d\xbc\x9a\xa3\xf4\x78\xd4\x65\x83"),
  .iv = TEST_DATA_STR ("\x6c\x88\x3c\xb8\xf9\xff\x7e\x57\x90\x34\x52\x4e"),
  .aad = TEST_DATA_STR (
    "\x23\x27\x13\xb8\xde\x07\x44\xb0\x82\x51\x54\x9a\xaa\x19\x15\x4f"),
  .tag = TEST_DATA_STR (
    "\x20\x94\xf4\x98\x9f\x85\x0a\xf3\xbb\xfc\x48\xb1\x89\x5e\xc1\xde"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad128_tc3) = {
  .name = "192-GMAC 128-aad NIST CAVS TC3",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xb3\xe8\xa0\xd0\x5d\xbc\xd4\x53\xbd\xc8\xec\x09\x75"
			"\xf2\xbb\x06\x3a\x21\xd0\x39\x1d\xc9\x46\x45"),
  .iv = TEST_DATA_STR ("\x3f\xaf\x8b\xf9\x1d\x4d\x95\xa7\xf9\x62\x8a\x65"),
  .aad = TEST_DATA_STR (
    "\x6e\x69\x26\x61\x76\x14\xbe\xf6\x15\x3a\x4c\xe6\x29\xa9\x1b\x69"),
  .tag = TEST_DATA_STR (
    "\xac\xbb\x55\xb7\x10\x2e\x86\x17\x75\x42\xbc\x5a\x7f\xc7\x17\xa1"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad128_tc4) = {
  .name = "192-GMAC 128-aad NIST CAVS TC4",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xdf\x91\x67\x43\xd0\x21\x80\xa2\x88\x80\xbc\x45\xe0"
			"\xd0\xb7\x37\x47\x74\x69\x26\x01\xeb\x55\x56"),
  .iv = TEST_DATA_STR ("\x48\x31\x18\x9f\x72\x75\x17\xd7\xf4\x22\xf1\x2f"),
  .aad = TEST_DATA_STR (
    "\x4a\x4e\x9b\x8c\xda\x53\x57\xf9\x5c\x37\x26\x48\x64\xe3\x89\xa9"),
  .tag = TEST_DATA_STR (
    "\xde\x3b\x10\xaf\x6d\x8c\x1e\x3c\xd5\x80\x20\xce\xac\x9c\x5f\x41"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad128_tc5) = {
  .name = "192-GMAC 128-aad NIST CAVS TC5",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xcd\x58\xa2\xca\x49\x5b\x28\x14\x79\x61\x61\x91\x1a"
			"\xf0\x82\xa8\x52\xc8\xcb\xd3\xfe\x1c\xed\xb4"),
  .iv = TEST_DATA_STR ("\x78\x66\xd4\x58\x8a\xce\x52\xed\x1d\x07\xd3\x46"),
  .aad = TEST_DATA_STR (
    "\xe0\xe6\xf8\x5c\x52\xab\xa6\x87\x3d\x7d\xb5\x0d\x80\x2e\xd6\x16"),
  .tag = TEST_DATA_STR (
    "\x0d\x2b\xd7\xc5\x1f\x7f\x88\x16\x20\xbf\x50\x8f\x4b\x66\x2d\xa6"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad128_tc6) = {
  .name = "192-GMAC 128-aad NIST CAVS TC6",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xd8\x0f\x1f\x01\xe6\x6c\xa7\x04\x1f\x12\xa9\xde\xc4"
			"\x6c\xed\xfd\xf7\x5a\xef\x66\x4c\x58\xb2\x33"),
  .iv = TEST_DATA_STR ("\x24\x08\xb3\x5e\x9b\xa6\x93\xe0\x89\x31\xf7\xf3"),
  .aad = TEST_DATA_STR (
    "\x28\x03\x5a\x77\xe8\xb1\xdf\x98\x20\x9b\xd5\x29\xe4\x72\xbe\x1c"),
  .tag = TEST_DATA_STR (
    "\xc1\xa2\x9c\xb9\x1f\x13\x12\xb8\xc6\xc8\x6a\xd8\x33\xa9\x73\x74"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad128_tc7) = {
  .name = "192-GMAC 128-aad NIST CAVS TC7",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x58\x29\x1d\xe8\x38\xf4\x7f\x6b\x30\xf2\xeb\x4f\x55"
			"\x6b\xf2\xfd\x81\xb8\x49\xb9\xe8\x76\xf0\x48"),
  .iv = TEST_DATA_STR ("\xdb\x4f\xa1\xcc\xc0\xed\x55\xdb\xe5\x33\xee\x90"),
  .aad = TEST_DATA_STR (
    "\x57\x6b\x6e\xaf\x76\x21\x1b\xe4\xd5\x40\x36\x3b\x23\xac\x29\x9d"),
  .tag = TEST_DATA_STR (
    "\x9d\x3f\xc9\x6b\xa6\xcc\x39\xaa\x30\x74\xb0\x1e\xe2\xcb\xa4\xd4"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad128_tc8) = {
  .name = "192-GMAC 128-aad NIST CAVS TC8",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xab\x3c\x5c\x4a\x37\x2e\xc0\x5f\xeb\x74\x23\xa5\x55"
			"\xed\x6c\xc6\x6c\x5d\x3b\xd8\x55\x7e\xff\xa7"),
  .iv = TEST_DATA_STR ("\xfb\x79\x7a\x5f\xa6\x3a\x38\x88\x0e\xd3\x80\xc6"),
  .aad = TEST_DATA_STR (
    "\x67\x63\xc5\x0c\x5d\xe0\xdb\x7f\x67\x5f\xe1\x6d\x0a\x5d\x5a\x79"),
  .tag = TEST_DATA_STR (
    "\x6a\xe6\xc7\x8d\xe5\xdf\xea\x5c\xb3\xe9\x6e\xe9\x59\x71\x37\x41"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad128_tc9) = {
  .name = "192-GMAC 128-aad NIST CAVS TC9",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x86\xcb\xf5\x79\xe9\xb0\x77\x0e\xc2\xa1\x37\x21\x8e"
			"\x9f\xf7\x1a\xeb\xf0\x51\xf6\x4a\x31\x8c\x74"),
  .iv = TEST_DATA_STR ("\x4d\xf9\xe3\xdd\x72\x0d\xce\x9e\xcc\xb3\x81\x76"),
  .aad = TEST_DATA_STR (
    "\xba\xbf\x21\xb7\x2e\x05\x67\xf2\x2e\x6f\xb1\x72\x11\x5b\x61\x2f"),
  .tag = TEST_DATA_STR (
    "\x51\x86\xbd\x05\x73\x93\x81\x1b\xc9\xc2\x8e\x8e\xb7\x71\x4b\x32"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad128_tc10) = {
  .name = "192-GMAC 128-aad NIST CAVS TC10",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xfd\x39\xed\xc5\x1d\xf0\x8e\x69\xf3\x97\x68\xdd\xff"
			"\x3f\xa9\xa7\xf6\x76\x42\xd7\x3b\x2c\x33\xdd"),
  .iv = TEST_DATA_STR ("\xf0\xf8\x07\x50\x24\xbb\x50\x97\x82\x79\xc5\x37"),
  .aad = TEST_DATA_STR (
    "\x7c\x95\xd4\xa5\x59\x15\xcf\x13\x7d\x3f\xa2\xbc\x0b\x9d\x5e\x99"),
  .tag = TEST_DATA_STR (
    "\xe9\x61\xe7\x9c\xd3\x49\x46\x1a\x14\x3b\x13\xe6\x2c\xf6\x9d\x3f"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad128_tc11) = {
  .name = "192-GMAC 128-aad NIST CAVS TC11",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x31\xc4\x45\x71\x32\x99\x96\x28\x7e\x98\xfc\x3d\xc7"
			"\x18\x15\x68\xcd\x48\xa3\x35\xfd\x37\x97\x2f"),
  .iv = TEST_DATA_STR ("\xb2\x34\x1f\xaa\x66\x1d\xc0\x49\x25\xf5\xa6\xb5"),
  .aad = TEST_DATA_STR (
    "\x2a\x0e\x83\xf4\xff\x96\x7e\xdd\xdc\x09\xdd\xc4\xc1\x69\xd5\x5d"),
  .tag = TEST_DATA_STR (
    "\x9b\xd9\x1d\x5d\xf6\x8a\xfc\x6d\x45\xbe\xbd\xe9\x0f\xcd\xb1\xee"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad128_tc12) = {
  .name = "192-GMAC 128-aad NIST CAVS TC12",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x23\xf9\x54\x34\x4d\x93\xa2\x7d\x5a\xbb\xb1\x92\x0a"
			"\xd8\xe8\x9a\x05\x10\x67\x21\x71\x31\x64\x48"),
  .iv = TEST_DATA_STR ("\x56\x0f\x42\x9c\x3d\xf4\x31\x41\x3e\x08\x6a\x75"),
  .aad = TEST_DATA_STR (
    "\x2d\x78\x32\xa2\xc4\x6b\x63\x44\xfe\x35\xf1\x48\xb5\xbf\x64\x1d"),
  .tag = TEST_DATA_STR (
    "\x6d\xd3\x5d\x46\x8e\xfd\xc9\xc9\x73\x97\x82\x33\x20\xc9\xb0\x69"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad128_tc13) = {
  .name = "192-GMAC 128-aad NIST CAVS TC13",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x11\xfa\xfc\xf1\x40\x9c\x57\x52\x05\x3d\x5c\xd1\x07"
			"\x70\xce\xf2\x70\x77\xdf\x64\x55\xfb\x27\x3b"),
  .iv = TEST_DATA_STR ("\xa6\x90\xc8\x70\x54\x65\x8e\xdc\x49\x94\x14\xd8"),
  .aad = TEST_DATA_STR (
    "\xd1\x51\x2c\x14\x46\x12\xb4\x5a\x77\x42\x65\x88\xc1\xc0\x25\x4d"),
  .tag = TEST_DATA_STR (
    "\xb1\x40\x66\x1e\xa4\xa7\x93\xbc\x67\xda\xa0\xfa\x00\x9a\x18\x5b"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad128_tc14) = {
  .name = "192-GMAC 128-aad NIST CAVS TC14",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xd7\x1e\x1c\x94\xb2\x11\x84\x08\x8e\x6a\x63\xf3\xca"
			"\xba\x9a\x9c\xcf\x4a\x15\xf0\xbc\x53\xfb\x02"),
  .iv = TEST_DATA_STR ("\x82\x7c\xd7\x65\xa6\xdc\x8e\x4d\xe2\xe7\x66\x49"),
  .aad = TEST_DATA_STR (
    "\x4f\xc6\x66\xa1\xcf\x04\xcf\xdb\x0f\x5f\x68\x1b\x6f\x19\x86\xbb"),
  .tag = TEST_DATA_STR (
    "\x9c\xf4\x07\xee\x84\x47\x6d\x54\x8e\x05\x93\x9c\x3b\xeb\x9f\x53"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad160_tc0) = {
  .name = "192-GMAC 160-aad NIST CAVS TC0",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x09\x3e\xf7\x55\x1e\xbb\xff\x8e\xb0\xc0\xa8\xa4\xa6"
			"\x2b\x19\x8f\x0c\x2e\x83\x8d\xe1\x0e\xee\xee"),
  .iv = TEST_DATA_STR ("\xe6\x56\xe9\x39\x30\xed\x52\x10\xba\x3f\x03\x22"),
  .aad = TEST_DATA_STR ("\x3d\xa2\x2d\xac\xfd\x11\xb2\x1b\x0a\x71\x31\x57\xf6"
			"\x0a\xec\x0c\xd2\x2f\x1a\xdd"),
  .tag = TEST_DATA_STR (
    "\x1b\x2d\x27\x64\x57\x3e\x20\xae\x64\x0b\xf2\x9d\x48\xe5\xfe\x05"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad160_tc1) = {
  .name = "192-GMAC 160-aad NIST CAVS TC1",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x08\x63\xe6\xe0\xe9\x76\x3a\x30\x21\xed\x49\x45\x81"
			"\x2b\xec\xf2\x7b\x82\x20\xd4\xb3\x29\x73\x57"),
  .iv = TEST_DATA_STR ("\xad\xb4\xe4\xe6\x29\xcf\x4a\x86\x54\x0e\xfe\x1e"),
  .aad = TEST_DATA_STR ("\x58\x14\x24\xd6\x33\xf3\xf9\x69\xd1\xb4\xf8\x35\x19"
			"\x7a\x74\x0a\x69\x5b\x2c\x3b"),
  .tag = TEST_DATA_STR (
    "\x79\x47\x52\x19\xe6\x34\x9b\x68\xac\x71\x27\xfb\x55\x11\xe9\x20"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad160_tc2) = {
  .name = "192-GMAC 160-aad NIST CAVS TC2",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xb4\x33\x29\x9e\x54\xca\xcc\x5e\x6f\x7b\x34\xc0\xa1"
			"\xe0\x55\x52\x24\xa6\xa7\x66\xf8\xae\x21\x01"),
  .iv = TEST_DATA_STR ("\x8f\x36\x11\xd4\xf6\x97\xae\x52\x48\x25\xd2\x39"),
  .aad = TEST_DATA_STR ("\xee\xbc\x05\x29\x17\xef\xb0\x31\x65\x0f\x38\xce\x70"
			"\x4e\x92\xd4\xbf\x59\xf9\x41"),
  .tag = TEST_DATA_STR (
    "\x91\xd1\x76\x2a\x7f\x19\xfe\x1e\x75\xdc\xa5\x1e\xc8\x5c\xe3\x19"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad160_tc3) = {
  .name = "192-GMAC 160-aad NIST CAVS TC3",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xf3\x73\xef\xaf\xb2\x39\x44\x55\xe0\x64\xb4\x2d\x23"
			"\x4d\x21\xbd\xf4\x52\xdf\x03\x64\x52\xcf\x2c"),
  .iv = TEST_DATA_STR ("\xd6\x63\x86\x02\x84\xd5\xb8\x33\x32\xa3\xa0\x25"),
  .aad = TEST_DATA_STR ("\xee\x1e\x7f\x47\x29\x85\xa6\x39\x74\x39\xb2\x8a\x52"
			"\x6b\x6c\xed\xf5\x95\x3a\xb1"),
  .tag = TEST_DATA_STR (
    "\xd8\x19\x77\x43\x49\xbf\x96\x3a\x9d\xf7\xed\x46\x26\x1f\xc5\xca"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad160_tc4) = {
  .name = "192-GMAC 160-aad NIST CAVS TC4",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xd5\x04\x53\xfd\x22\x3b\x11\x0d\x2d\xf3\x12\xc4\x81"
			"\x7d\x97\x6a\x59\x19\x4a\xda\x77\x7f\x97\x41"),
  .iv = TEST_DATA_STR ("\x6a\x88\xcd\x46\x85\x86\xcb\x22\x7b\x92\x85\x40"),
  .aad = TEST_DATA_STR ("\x3d\x7e\x66\x93\xe1\x63\x9d\xed\x24\x2e\x3c\x0b\x93"
			"\x1b\x32\xe7\x2a\xdc\x70\x5f"),
  .tag = TEST_DATA_STR (
    "\xb3\x88\xde\x6f\x83\x48\xee\xea\xda\x78\xf6\xe9\xe0\x04\xdb\xe5"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad160_tc5) = {
  .name = "192-GMAC 160-aad NIST CAVS TC5",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xe6\xe6\x86\xc7\xbe\xf0\x1b\x17\x04\xab\x1b\xf1\x14"
			"\x99\x0c\xea\xad\x41\xbe\x84\x80\x17\x74\x60"),
  .iv = TEST_DATA_STR ("\xbe\x55\x59\xdc\x38\xe9\x79\x4c\x24\xac\x7a\x83"),
  .aad = TEST_DATA_STR ("\xd8\x21\x8c\xc4\x56\xa5\x4f\x79\x1a\x3f\xd8\x78\x90"
			"\x89\x09\x46\x35\x3b\x9b\xfb"),
  .tag = TEST_DATA_STR (
    "\xe3\x94\xe6\xff\x9e\x9a\x75\x40\xb7\x42\xff\xf8\xdc\x92\x3a\x59"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad160_tc6) = {
  .name = "192-GMAC 160-aad NIST CAVS TC6",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x85\xba\x70\xce\x5a\xa7\xcb\x81\x95\x70\x74\xfa\xb4"
			"\xdf\x72\xc4\x6c\x9e\xc4\x46\x6b\xa0\xb4\x8a"),
  .iv = TEST_DATA_STR ("\xb9\x1d\x11\xf7\x2f\x6c\x5f\xab\x56\x53\x5e\x5c"),
  .aad = TEST_DATA_STR ("\x4e\x8a\xfc\x98\xf8\x27\x20\xcd\x8e\x35\xea\x8f\x8a"
			"\xff\x20\xd2\x39\xde\x14\x07"),
  .tag = TEST_DATA_STR (
    "\x41\x3f\xc7\x28\x3d\x57\x8b\xaa\x1e\x62\x13\xf5\x41\x59\x04\x62"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad160_tc7) = {
  .name = "192-GMAC 160-aad NIST CAVS TC7",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x7e\xd5\xb2\xa4\xbf\x74\x27\xdb\xb2\xf6\x4e\xfb\xcd"
			"\x06\x76\xab\x91\x2f\x12\xaf\xae\x85\xf8\xf7"),
  .iv = TEST_DATA_STR ("\x1a\x06\xb0\xa2\x07\xb8\x9c\x19\x3c\xfb\xdb\x20"),
  .aad = TEST_DATA_STR ("\x93\xbf\x5a\xbe\x39\x7e\xe6\xa9\x79\xc3\x88\x7c\xb5"
			"\x7a\xf0\x1d\xf8\x3d\xf2\x91"),
  .tag = TEST_DATA_STR (
    "\x10\xca\x8e\xe1\x68\x70\xb9\x51\xc9\x18\x0e\xa1\x85\x36\x50\xbb"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad160_tc8) = {
  .name = "192-GMAC 160-aad NIST CAVS TC8",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x28\x1c\x87\xcc\x27\x80\x53\x75\x78\x6e\x11\x78\xd1"
			"\xb0\xe2\x2a\x7a\x85\x46\xcf\x6f\x2f\xe1\x2b"),
  .iv = TEST_DATA_STR ("\xb9\xc5\x70\x39\x2f\x02\x53\x89\x05\x5c\x9c\x35"),
  .aad = TEST_DATA_STR ("\x54\xc8\x26\xa0\xca\x02\x76\x33\x51\x59\xa7\x54\x2e"
			"\x22\x8c\x3d\xae\xbd\x38\x9a"),
  .tag = TEST_DATA_STR (
    "\x56\x0a\x32\x1c\xff\x6a\x8c\x1e\xac\x06\x01\x49\xc5\x95\x5f\xf8"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad160_tc9) = {
  .name = "192-GMAC 160-aad NIST CAVS TC9",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x90\xa1\x96\xe8\xc5\xd5\x77\xa6\xc5\x14\x38\x1b\xad"
			"\xdb\xba\x7e\xd8\xe4\xd1\xe0\xa7\x96\x1f\x32"),
  .iv = TEST_DATA_STR ("\x1c\x2c\x7c\x8b\xd0\x15\x33\x68\xb2\xa8\xc3\x49"),
  .aad = TEST_DATA_STR ("\x0f\x40\x9b\xa3\x68\xc2\xef\x04\x33\xb9\xbd\x96\xff"
			"\x73\x51\x1f\xce\x63\x93\x18"),
  .tag = TEST_DATA_STR (
    "\x7a\x7a\xc7\x70\xa5\xa4\xc6\x6b\x78\x7a\xa3\xa1\x26\x7b\xa3\x45"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad160_tc10) = {
  .name = "192-GMAC 160-aad NIST CAVS TC10",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x36\x37\x62\x07\xa2\xc1\xf2\x5e\xc2\x0f\x62\x1b\x06"
			"\xdb\x26\x0c\x20\xbe\x7b\x39\xe7\x0b\x89\x3c"),
  .iv = TEST_DATA_STR ("\xc9\xe9\x2a\x66\x62\xa7\xd6\xbb\x84\x94\x5c\x95"),
  .aad = TEST_DATA_STR ("\xb2\xcd\xe6\x02\x9e\x0f\x93\x30\x92\xe9\x74\x74\xdc"
			"\x8b\x0b\x17\x4d\xe5\x53\x52"),
  .tag = TEST_DATA_STR (
    "\x80\x3b\x69\x53\x80\x1b\xf9\x79\x09\x64\x37\xe0\x2f\x3c\xb1\x31"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad160_tc11) = {
  .name = "192-GMAC 160-aad NIST CAVS TC11",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x68\xe6\x21\xb4\x76\xdd\x7d\xda\xda\x51\x56\xc0\xc5"
			"\x65\xc4\xc0\x05\xbc\xf9\x93\x08\x2a\x6c\x68"),
  .iv = TEST_DATA_STR ("\xac\x0b\xd5\x9b\xf7\xb9\x34\x5d\x01\xec\x7d\x99"),
  .aad = TEST_DATA_STR ("\xe3\x18\xce\x39\xbe\xae\x93\x72\xde\xe2\xba\xc3\x56"
			"\x8c\xa3\x7e\xf8\x71\x4b\x1f"),
  .tag = TEST_DATA_STR (
    "\xf8\x42\x6c\x2c\x39\x0a\x5b\xd5\xde\x2f\x4f\x31\xb8\x9a\x8f\xf8"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad160_tc12) = {
  .name = "192-GMAC 160-aad NIST CAVS TC12",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xe5\x76\x06\xe3\x9d\xc3\x6d\xca\xcc\xfb\x3a\x13\x23"
			"\xb5\xc1\x80\x3c\xea\x6d\x76\xcd\x96\x44\x18"),
  .iv = TEST_DATA_STR ("\xf6\x35\xc6\xe1\x2b\xf9\x39\x46\x5c\xd7\x10\x45"),
  .aad = TEST_DATA_STR ("\xdf\x1b\xef\x3c\xeb\x77\xb6\x7f\xf6\xdc\x7a\x16\x63"
			"\xc5\x72\xfb\x00\x22\x05\x59"),
  .tag = TEST_DATA_STR (
    "\x59\xd5\xf4\x63\x40\x2a\x08\x95\x4f\xa9\xd1\x65\x44\x9d\x95\x1c"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad160_tc13) = {
  .name = "192-GMAC 160-aad NIST CAVS TC13",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x09\xab\x6b\x91\x2e\x3b\x17\x4d\xea\x38\xa7\x27\x0b"
			"\x36\xc3\x88\xe1\x08\xbc\x76\x0b\xa4\x81\xbf"),
  .iv = TEST_DATA_STR ("\x13\x2e\x01\x26\x95\xf1\xe9\xb7\x99\x50\x5c\xef"),
  .aad = TEST_DATA_STR ("\xd7\x91\xd5\x0c\xd3\x13\xdb\x40\x60\x75\xc9\x7b\x12"
			"\x8b\x07\x8d\xa5\xb6\x8c\xa1"),
  .tag = TEST_DATA_STR (
    "\x72\x15\x2f\x6d\x3a\x95\x0d\x32\x3f\xd6\x19\xbe\x3d\x5b\x0c\x6f"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad160_tc14) = {
  .name = "192-GMAC 160-aad NIST CAVS TC14",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xc1\x3e\xf7\x2b\x80\xac\xfd\x6a\xdc\x0b\xb8\xf0\x37"
			"\x72\x23\xcb\xa7\x33\x93\x9b\x50\x58\xf3\x36"),
  .iv = TEST_DATA_STR ("\xc4\x57\x6f\x76\xb3\x8c\x9c\x91\xbb\x08\xb8\x3f"),
  .aad = TEST_DATA_STR ("\x3f\x89\x65\x17\x15\x6c\xde\x96\xb2\x39\x09\xf2\x98"
			"\xa7\x6c\xde\x59\x04\x7a\xe0"),
  .tag = TEST_DATA_STR (
    "\xba\x43\xda\x6f\x40\xaa\x9c\x3a\x66\xdc\x37\x2e\x3b\x3f\x94\x0c"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad384_tc0) = {
  .name = "192-GMAC 384-aad NIST CAVS TC0",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xa8\x3a\xc8\x25\x54\xd5\xc3\x47\x5d\x03\x51\x4a\x94"
			"\x21\xc4\x09\xcc\xad\x9b\xc4\x4a\x5f\x83\x07"),
  .iv = TEST_DATA_STR ("\x38\xaa\x26\xf7\xb6\x8d\xc6\x74\xca\xe9\x0b\x84"),
  .aad = TEST_DATA_STR (
    "\x03\x97\xa7\x14\x93\x9f\x55\xc1\x8d\xa0\x13\x27\x13\x48\xbd\x23\x1e\x14"
    "\xd0\x7f\x39\x7c\xa0\xdb\x20\xd3\xa7\x7c\x42\xf3\xf4\x1c\x25\xc6\x4f\xd9"
    "\x3b\xd3\xbd\x9f\xcd\x35\x5a\x0b\xde\x4f\x19\x61"),
  .tag = TEST_DATA_STR (
    "\xe4\x8b\x36\xdc\x68\x3f\x32\xdb\xae\x3b\x13\xc3\xad\xb1\xb7\x89"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad384_tc1) = {
  .name = "192-GMAC 384-aad NIST CAVS TC1",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x5f\x1a\x7f\x44\x39\xf8\xc7\xbc\x30\x1e\xa5\xb8\x95"
			"\x5f\x3c\x67\x98\x26\x3b\xe4\x7d\xac\xe3\x9c"),
  .iv = TEST_DATA_STR ("\x2d\xcc\x19\xde\x07\x65\x5f\x72\x43\xfa\xb0\x45"),
  .aad = TEST_DATA_STR (
    "\x7e\x76\x78\x36\xe5\xd1\x6d\xfc\x44\x26\x23\x7e\xfc\x91\x4f\xc4\x0b\xfe"
    "\x59\x4b\x54\x94\x6e\xd5\xf2\x00\x20\x3c\x93\xce\x58\x5c\x4c\xb4\xa2\x4a"
    "\x33\x64\xcc\xb9\x80\x3a\x64\xac\x4e\x38\xde\x2d"),
  .tag = TEST_DATA_STR (
    "\x9d\x34\xef\xdb\x37\x01\x49\x49\x13\xe3\x86\x36\x81\xa9\xb4\x02"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad384_tc2) = {
  .name = "192-GMAC 384-aad NIST CAVS TC2",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x8d\xce\x48\xb9\x16\x91\x63\x83\x5a\x0b\x6a\x4a\x62"
			"\x7c\x29\x03\x0b\x5b\xef\x3f\xf3\xba\xe1\xca"),
  .iv = TEST_DATA_STR ("\xa0\x0d\x44\x38\xe9\x6e\x7a\x22\xe5\x72\x65\xce"),
  .aad = TEST_DATA_STR (
    "\x7c\xc8\x27\x3e\x62\x59\x55\x83\xd4\x27\xbb\xf4\x59\x2c\xd2\xc2\x52\x5a"
    "\x28\xbb\x9e\x14\x3a\x9c\x9a\xf0\x63\x41\x10\xf2\xb6\x9c\xcb\x4e\xc0\x0c"
    "\xc2\xaf\xaa\x86\xc9\x86\xd3\xef\x2c\x44\x76\xa9"),
  .tag = TEST_DATA_STR (
    "\xe2\x76\xfe\xda\x74\x32\xfa\xa0\xe4\xab\xd4\x6d\x59\x2b\x8f\xee"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad384_tc3) = {
  .name = "192-GMAC 384-aad NIST CAVS TC3",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x0e\xce\x58\x69\x34\x92\x86\xef\x12\x2b\xb4\xf0\x5a"
			"\x0c\x0d\xf5\xc7\xc6\xdf\x49\x60\x79\x20\xff"),
  .iv = TEST_DATA_STR ("\x8d\xa2\x36\x71\xb7\x2e\xc6\xaa\xab\x27\x46\x3e"),
  .aad = TEST_DATA_STR (
    "\x95\x82\x4e\xa2\xb8\x0c\x4e\x97\x91\x1f\xff\xa3\x9e\x3f\x0c\x21\xfc\x81"
    "\xed\xd2\x68\x8a\x5a\x1e\x58\x3b\xa3\x62\xb4\x7b\x97\x97\x31\xbb\x25\x6c"
    "\xff\x1a\x47\x9f\x27\xa3\x12\x40\x89\x1e\x57\xe2"),
  .tag = TEST_DATA_STR (
    "\xfb\xd7\x57\xb8\x96\x3b\xbb\x32\x6c\xda\x80\xf3\xd5\x08\xf8\x9b"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad384_tc4) = {
  .name = "192-GMAC 384-aad NIST CAVS TC4",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x0a\xbd\x4a\xac\x60\x34\x31\x26\x17\x38\x61\x53\x4e"
			"\x8f\x46\xfc\x46\x0f\x8f\x3e\x21\x69\xf5\xc5"),
  .iv = TEST_DATA_STR ("\x5e\x46\xfe\x13\xe6\xc2\x44\xe8\x34\x19\x8d\x3d"),
  .aad = TEST_DATA_STR (
    "\x62\x48\x64\xae\xa6\x0c\x08\xe9\xa1\x3a\x8a\x9c\x09\x44\x57\xc9\xda\x22"
    "\x26\x24\x7a\x77\x1a\xae\xd5\x97\xc5\xe2\xcb\xc3\xd6\xe6\x17\x9d\xef\x86"
    "\xc9\xd0\x4f\x1f\x6e\x8c\xe5\xb9\x9f\x78\x9e\x3b"),
  .tag = TEST_DATA_STR (
    "\x67\x74\x56\xc4\x84\xab\x6b\xb1\xc3\x22\xf1\x00\xff\x9f\x8c\x43"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad384_tc5) = {
  .name = "192-GMAC 384-aad NIST CAVS TC5",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x82\xea\xe3\xc1\xde\xac\x84\x84\xe9\x9e\x97\xe6\x97"
			"\xc7\xa4\x2d\xb0\x26\xd7\x11\xc3\xdb\x60\x0e"),
  .iv = TEST_DATA_STR ("\x8f\xa3\xf1\x6b\xb6\xce\xf8\x75\x2c\x8e\x31\xef"),
  .aad = TEST_DATA_STR (
    "\x61\xe8\xf8\x8a\xe8\xc0\x55\xf7\xd9\xe6\x7e\x0f\x1d\x49\x93\xa3\xe5\xf7"
    "\x3f\x36\x62\xdc\x1c\xa8\x88\x66\x33\xab\x9b\x2a\x8c\x69\x28\xdb\x5b\x7a"
    "\x30\xfd\xec\xaa\x29\xdb\xbe\x01\xfd\xb1\x20\xbb"),
  .tag = TEST_DATA_STR (
    "\x7d\xe2\x16\x8f\x5c\x43\x4c\x06\xb7\xc4\xaf\x15\x37\x27\x45\x22"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad384_tc6) = {
  .name = "192-GMAC 384-aad NIST CAVS TC6",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x04\x2d\x69\x65\x4b\x27\xa1\x38\x0a\xde\xcc\x9b\xa7"
			"\x50\x90\xf1\xca\x42\x2b\x72\x5a\x47\x93\xe0"),
  .iv = TEST_DATA_STR ("\x87\xdb\x23\x7e\x9b\xf6\xcc\xbd\x08\x69\xf0\xf9"),
  .aad = TEST_DATA_STR (
    "\x49\x6e\xff\x4c\x74\xac\x08\xbc\xcd\xec\xec\x7a\x49\x40\xdd\xbe\xb8\x0b"
    "\xa1\xa5\x58\x24\x7e\xaa\x18\xa4\x66\x72\xd8\x74\xd7\xde\x6d\xd3\xa5\x77"
    "\x9e\xbc\xd9\x84\xc2\x29\x91\x3d\x10\xf6\xf7\xcc"),
  .tag = TEST_DATA_STR (
    "\xba\x06\xea\xab\x5b\x16\x66\x20\xef\xc8\x07\x2f\xa3\xa5\xb4\xb8"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad384_tc7) = {
  .name = "192-GMAC 384-aad NIST CAVS TC7",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xc1\x79\x23\x25\x76\xee\xb3\x8c\x98\xf8\x47\x87\x3d"
			"\x00\x4b\x96\x46\x65\xa3\x87\xa0\xa7\xf0\x14"),
  .iv = TEST_DATA_STR ("\x85\xd2\x99\x6d\x00\x3e\xf9\xfd\xc4\xa5\x4c\xe9"),
  .aad = TEST_DATA_STR (
    "\x92\x68\x24\x53\x5c\x61\x3f\xde\x98\x69\xdf\x1a\xaf\x76\x4a\x54\xc1\x36"
    "\x16\x67\x7f\x09\x92\x09\x14\x2d\xa4\xb6\x5d\x9a\x86\x64\xd1\x78\x53\xec"
    "\x10\x2f\xfa\x1b\x16\x88\x80\x6d\xbe\x50\x3a\x33"),
  .tag = TEST_DATA_STR (
    "\xdc\x13\x50\x36\xf7\x4e\x62\x34\xc4\xe3\x27\xfb\xb0\xae\xb9\x25"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad384_tc8) = {
  .name = "192-GMAC 384-aad NIST CAVS TC8",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xda\x39\xc0\xef\xb1\x00\xfd\x9c\xf2\xd9\x01\x70\x5a"
			"\xa6\x35\x90\x3c\xe5\x3b\x66\x9e\xbd\xb5\xca"),
  .iv = TEST_DATA_STR ("\x0e\x95\xbd\xcb\x66\x98\x24\xdb\xd7\xff\xc8\x8f"),
  .aad = TEST_DATA_STR (
    "\x46\x42\x87\x5e\x8e\x20\xc1\x65\xb5\xb1\x7f\x12\xfd\xc6\x30\x99\x6b\x58"
    "\xb8\x57\x1c\x5a\x15\x94\x4c\xe1\x94\x50\x8c\x87\x12\x3a\xd5\x00\x41\xf5"
    "\x9a\xfe\x02\xea\xc3\xac\x1e\x6b\xa5\xed\x92\x8b"),
  .tag = TEST_DATA_STR (
    "\x59\xf9\x96\xe9\xa7\x23\x14\xfc\x76\x75\xe5\xa9\x13\xfe\x8e\x36"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad384_tc9) = {
  .name = "192-GMAC 384-aad NIST CAVS TC9",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x5a\x41\x3f\xa7\x5a\x13\xb0\x36\x53\x81\x82\xad\x51"
			"\x50\x6f\xdd\x77\x33\xf1\xba\x39\x00\x04\x84"),
  .iv = TEST_DATA_STR ("\xa6\xcd\xa5\xb0\x22\xec\xfc\x5a\x2b\x75\x90\x13"),
  .aad = TEST_DATA_STR (
    "\x1e\xed\x51\xef\xc1\xf5\xca\xe5\x76\x90\xe0\x32\x06\xb4\x5a\x7b\x5c\xb4"
    "\x58\x56\xab\x36\x31\x32\x34\x94\x85\x01\xdd\x02\xea\x4f\x24\xae\x90\xb5"
    "\xb2\x46\x28\x91\xe4\x93\x3a\x1b\xd0\x38\x74\x63"),
  .tag = TEST_DATA_STR (
    "\x57\x29\x61\xb6\xe8\x50\xad\xb4\x60\x16\x64\xe0\xeb\x3e\x07\x36"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad384_tc10) = {
  .name = "192-GMAC 384-aad NIST CAVS TC10",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x80\x08\xf9\xd2\x5a\x1a\x70\x5b\x5f\x00\x79\xd3\xe3"
			"\x9c\x49\x87\x28\x65\x37\x10\x06\x61\xde\x6e"),
  .iv = TEST_DATA_STR ("\xa9\xdd\x20\xd7\x51\x2c\xe5\xb3\x54\x83\xa0\x82"),
  .aad = TEST_DATA_STR (
    "\xfb\xd2\x16\x02\x37\x74\x2f\x4c\xa7\x2f\x0b\x7e\xd6\x16\xa8\x47\xaf\x65"
    "\xed\xd2\x81\x67\x97\xb1\xc9\xc8\xb0\xb7\x37\x7b\x57\x59\x3c\x56\xc5\x80"
    "\x63\xc9\x6a\x30\x69\x8c\x51\xbe\xb6\x78\x6e\x74"),
  .tag = TEST_DATA_STR (
    "\xb2\xe2\x25\x89\x00\xd7\xfd\x7a\xc4\xe9\x63\x92\x38\xd6\x63\x8a"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad384_tc11) = {
  .name = "192-GMAC 384-aad NIST CAVS TC11",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xcc\x95\x61\xf6\xd3\x0d\x9a\x2f\x25\x75\x07\x52\xd3"
			"\x9a\x1f\x0b\xc8\x00\xe0\xe7\x24\x42\x7e\x64"),
  .iv = TEST_DATA_STR ("\x3c\x56\x51\x80\x3f\xee\x90\x98\xbd\x69\x04\xed"),
  .aad = TEST_DATA_STR (
    "\x0e\x28\x55\x40\x35\x82\x98\xa1\x87\xd4\xf6\x82\x3f\xf8\x6c\xea\xb1\x23"
    "\x4d\xbc\xef\xc0\x9b\x23\x33\xe7\x45\xf2\x3b\xb6\x0e\x63\x65\xcd\x36\x3d"
    "\x9e\x9b\x3d\xfa\x9f\xb9\x27\x0d\x6a\x9a\x52\xab"),
  .tag = TEST_DATA_STR (
    "\x01\x53\xf9\x5c\x4c\x0b\x4b\x47\x98\x9d\xa7\x1e\xe7\x2c\x34\xc6"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad384_tc12) = {
  .name = "192-GMAC 384-aad NIST CAVS TC12",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x61\xab\xa2\x81\xeb\x81\x20\x5d\xc6\xd9\xbb\x6b\x17"
			"\x87\xb7\x42\x51\x63\x87\x24\x1c\x15\x3c\xc2"),
  .iv = TEST_DATA_STR ("\x72\x4b\x42\x24\x31\x2a\x59\x6f\xf2\x30\x03\x93"),
  .aad = TEST_DATA_STR (
    "\xdd\x06\xc9\xe0\x6a\x6f\xd9\xd8\xfe\xa3\x56\x25\x5c\xbf\x90\x93\x86\xf7"
    "\xac\x5e\x9b\x5e\xaa\x5c\x55\x28\x20\x54\x82\x7f\x74\xe9\xe7\x43\x46\xac"
    "\xff\x57\x25\x09\x73\x53\xe8\x6b\xff\xeb\x6d\xc6"),
  .tag = TEST_DATA_STR (
    "\x26\x44\xe8\xe6\x52\xc2\x58\xab\x02\x8b\x86\xcd\x7e\xf5\x5f\x5c"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad384_tc13) = {
  .name = "192-GMAC 384-aad NIST CAVS TC13",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x73\xa4\x90\x52\xef\x91\x18\xf0\x54\x88\x81\x0c\x20"
			"\x80\xd7\x38\x99\x39\x38\x6c\x18\x6d\x92\xb3"),
  .iv = TEST_DATA_STR ("\x38\xf0\x06\xb2\xe8\x5e\x7a\xa2\xf4\xc8\x81\x89"),
  .aad = TEST_DATA_STR (
    "\x21\xfa\x5a\xbb\x18\xb2\xfb\xcc\xe3\xa1\x9b\x2e\xac\x8b\xe7\xa3\x01\x92"
    "\x3f\xa2\x58\x10\x52\x86\x13\x3e\xd5\xf4\x78\x34\x84\x2a\x63\x84\xc4\xfc"
    "\x0a\x39\x86\xe1\xa2\x5b\xba\x83\x47\x9f\x68\x16"),
  .tag = TEST_DATA_STR (
    "\x9d\xbd\x74\x84\xc9\xaa\xed\x54\xdf\x7e\xd6\x4b\xbe\xd2\x0c\x68"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad384_tc14) = {
  .name = "192-GMAC 384-aad NIST CAVS TC14",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x1e\x11\x8d\x10\x94\x26\xb2\xab\x64\x46\xb0\x65\x99"
			"\xa4\xc9\x71\xf6\x68\x3a\x34\x35\x68\xef\x97"),
  .iv = TEST_DATA_STR ("\xcc\x87\x23\x42\x15\xc9\x74\xfd\x44\x68\x9e\x25"),
  .aad = TEST_DATA_STR (
    "\x48\x67\x4b\xf3\x86\x06\x46\x02\xd0\x0f\xd7\x2a\x17\x39\x20\xaf\x9b\x4c"
    "\x4f\x9a\xfb\xf1\x9e\xa7\x63\xff\x44\xe4\x7e\xf8\x9a\x10\x65\x80\xc2\x89"
    "\xc3\x98\xf9\x7f\xaa\x60\xba\xf4\x9d\xc1\xa2\xaf"),
  .tag = TEST_DATA_STR (
    "\x97\xf1\x3f\x94\x2a\xf7\xb7\x79\x7e\xa0\x9c\xea\xbd\xc7\xdc\x9c"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad720_tc0) = {
  .name = "192-GMAC 720-aad NIST CAVS TC0",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xc2\x5d\x34\x7f\xfb\x5b\x7b\xa0\x79\xbe\x22\x79\xa0"
			"\xa7\xf2\x20\xf1\x9c\x74\xbb\x9c\x5a\x15\xb6"),
  .iv = TEST_DATA_STR ("\xb3\x5f\x14\x21\x82\xfe\xa6\x5c\x64\x23\x68\xed"),
  .aad = TEST_DATA_STR (
    "\x19\x67\xa0\xbd\x80\xcf\x2c\x9c\x58\xe4\x41\xe1\x2c\xba\x78\x8f\x9c\x07"
    "\x21\x77\xe1\xce\x02\xf3\x0d\x58\xae\x98\x1a\xb3\x7e\xac\x45\x2c\x0d\x9f"
    "\x1c\x5f\x34\x85\xd7\xb1\x6a\xe0\x93\x66\x82\x1d\x23\xd4\x44\x79\xd5\x2c"
    "\xcc\x4a\xcd\x8f\xa6\xf5\xb9\x01\x38\x45\xc6\x29\xf6\x9c\x61\x2c\x9c\xbb"
    "\xcd\xca\x3b\xdf\x43\x85\x5f\xa7\xc7\x1b\xff\x45\x8a\x7d\x4c\x01\x9a"
    "\xd9"),
  .tag = TEST_DATA_STR (
    "\xf5\xa0\xd6\x49\x24\xae\xab\x15\xa6\x36\xc7\xce\x4d\xb5\x22\x43"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad720_tc1) = {
  .name = "192-GMAC 720-aad NIST CAVS TC1",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xcb\x00\x0f\xdd\xd6\x7b\xf5\xa2\x4b\x03\xc8\xb0\x89"
			"\x65\xfc\x56\x89\x62\xd7\xb2\xa0\xb4\xe6\x8e"),
  .iv = TEST_DATA_STR ("\xac\xad\xc8\xf8\x22\xb8\x37\xb8\xfc\xd5\xac\x53"),
  .aad = TEST_DATA_STR (
    "\xce\x0e\x3e\x4e\x6f\xfe\xae\x66\xc5\x35\x66\x7e\x8a\x8c\xf1\x2f\xca\x0e"
    "\x9d\xae\x69\x87\x83\x5e\x8e\xc6\x2f\xb9\x5b\x38\xf3\x1e\xc5\xe9\x37\xbd"
    "\xfe\xd5\xb5\x51\x74\x83\x4b\x03\x8b\xa3\x32\x2b\x4a\x25\x65\xac\x41\x3b"
    "\x6e\x20\x4f\x88\xc3\xa9\x32\x16\xb8\x81\x06\x49\x4e\xaa\x14\xa8\x20\x68"
    "\xf0\x0a\x3b\xf2\x27\xb6\x27\x07\x53\x83\x68\x2b\xd6\xbe\xd6\x23\x1e"
    "\xaf"),
  .tag = TEST_DATA_STR (
    "\x2c\x1c\xdf\xc8\xaf\xb7\x56\x9b\x87\x7b\xa5\xae\x13\xd6\x23\x5b"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad720_tc2) = {
  .name = "192-GMAC 720-aad NIST CAVS TC2",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x94\x9f\x77\x6b\x66\xb5\x28\x34\xde\x80\xe2\x4d\xa2"
			"\xc6\x68\x3c\x00\x94\x74\x3c\x6b\x4b\x57\xd1"),
  .iv = TEST_DATA_STR ("\x75\x57\xf7\xb9\xa8\x55\x4e\x79\xf8\x69\x52\x9b"),
  .aad = TEST_DATA_STR (
    "\xe3\x6d\xb9\xd5\x1d\xed\xe1\x0f\x17\xe4\xba\x3a\xa2\x0e\xee\x49\xc2\x06"
    "\x24\x4f\x89\xf6\x7f\xfa\x7d\x49\x94\x58\x93\xa0\x5f\xb6\xb5\x94\x8c\x53"
    "\x61\xdc\x84\xb3\x3a\x4c\x35\x76\x8c\xb6\x54\x74\x08\xba\x61\x7e\xdb\xa4"
    "\x17\x82\xa6\x5e\x4f\xca\x1a\x02\x79\x68\xf4\x4c\x43\x3f\x84\x53\xdb\xef"
    "\xb3\x5a\xa4\xc2\x1b\x6c\x52\x0b\x10\x2a\xe4\xfd\xf2\x07\x9f\x81\xdd"
    "\x0c"),
  .tag = TEST_DATA_STR (
    "\x88\xc0\x61\x2c\x1d\xde\xf9\x14\xb3\x43\x95\x05\x3f\x7f\x63\x2e"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad720_tc3) = {
  .name = "192-GMAC 720-aad NIST CAVS TC3",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x25\x87\x2c\x71\xd8\x70\x0c\x6e\x0a\x74\xf4\x4e\x95"
			"\x46\x8b\x12\xf2\xdc\xeb\x94\xc2\x57\x57\x5d"),
  .iv = TEST_DATA_STR ("\x20\xa8\x7b\xaf\xff\x89\x83\xae\x72\x5a\x6f\xf1"),
  .aad = TEST_DATA_STR (
    "\xc0\x9c\x11\x84\xd0\xfb\xe3\xaf\x22\x20\x2a\x59\xdf\xef\xd6\x6f\xcd\xa2"
    "\x29\x3c\x90\x62\x6f\x14\x93\xd6\xfd\x79\xed\x5b\x5d\x01\xbf\x8a\xc9\x09"
    "\x5f\x44\xa3\x1f\x9d\xb4\xa2\x6f\x79\x75\x4d\x75\xec\xf4\xfe\x02\x5f\x2c"
    "\x1a\xdf\x3c\xe5\xf3\xae\x76\x72\x1d\xaf\x3d\xcc\x9d\xd8\x99\xe3\xf9\x6c"
    "\x82\x73\xb2\x9b\xc1\x8f\xc3\x8a\xae\x1a\xaa\x12\x4d\xb3\x71\xaa\x47"
    "\xfd"),
  .tag = TEST_DATA_STR (
    "\xb1\x66\x3e\xb5\xb6\x98\xae\x8a\x7a\x18\xa6\xee\x74\x81\xb9\x8b"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad720_tc4) = {
  .name = "192-GMAC 720-aad NIST CAVS TC4",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x18\x8c\xa6\x91\x49\x83\xd3\xc1\xe5\x6c\x05\x9d\x0d"
			"\x70\x1d\x57\x3a\x61\xdf\x2d\xea\xee\xb1\xa6"),
  .iv = TEST_DATA_STR ("\x6c\x2f\xed\xb5\xf7\xf9\xf1\x15\x3a\xc3\x6c\xd8"),
  .aad = TEST_DATA_STR (
    "\x1d\xb4\xb3\x12\x70\x44\x94\x98\xba\x03\x97\x31\xb7\x33\x07\x68\xd1\x4c"
    "\x27\xe3\x73\xb7\xde\xbd\xb9\x8f\x2a\x41\xb6\xae\xc3\xb2\x98\xa0\x3e\xa5"
    "\xde\x8f\xed\x8f\xf2\x17\x96\x75\xea\x08\xe3\xc9\x81\x2c\x3f\x4f\x63\x76"
    "\x5f\x40\x39\x53\x4c\x5c\xcf\x98\xfd\xc3\xe7\x0c\xb1\x30\x9a\xd4\x16\x1e"
    "\x37\xe7\x14\xe6\x97\x28\x72\xfa\x65\x83\x72\x83\x25\xac\x52\x0d\x56"
    "\x69"),
  .tag = TEST_DATA_STR (
    "\x29\xc5\x6f\x77\xd8\x26\x0c\xa2\x94\x83\x37\xb2\x1c\x0c\x37\xa2"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad720_tc5) = {
  .name = "192-GMAC 720-aad NIST CAVS TC5",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xce\xcc\xe8\xae\x97\x77\x18\x7e\x5a\x87\xec\xb2\xd7"
			"\x35\xf7\x82\xf9\xf7\xaa\xb8\xb8\x7b\x13\x7d"),
  .iv = TEST_DATA_STR ("\x7d\x56\x4d\xb2\x02\xd0\xfa\xb3\x8d\xed\x36\xdd"),
  .aad = TEST_DATA_STR (
    "\xf6\xfb\xd1\xb5\x75\x5d\x70\x91\x54\x31\x2e\x11\x0f\xd4\x60\x85\xa4\xb6"
    "\xf6\x17\xc1\x27\xfe\xa7\x76\x36\xbf\xb8\xa5\x8a\x6a\x6d\x90\x30\xb2\xa6"
    "\xc4\xe7\x0d\x7a\x3a\x89\x4a\x75\x96\x7f\x65\x02\xe0\xc8\x16\xfb\x30\x69"
    "\xf2\xed\x94\xc8\x88\xd3\x07\x4c\x1c\x63\xc5\x95\x12\xbe\x45\x3e\x57\x5c"
    "\xec\x11\x5c\x49\xeb\x4d\xba\x44\xd2\xf7\xc7\x8b\x33\x55\xb1\xe6\x77"
    "\x87"),
  .tag = TEST_DATA_STR (
    "\xb2\x7c\x0b\xe6\x89\x85\x66\x26\xe5\x5e\x03\x77\xa0\x83\x34\x13"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad720_tc6) = {
  .name = "192-GMAC 720-aad NIST CAVS TC6",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x94\x47\x03\x91\xde\xf6\x95\xfe\x5d\xe5\xb8\x23\x3a"
			"\x20\xfe\x52\x11\xbb\x1d\xbb\xb2\x73\x31\x3f"),
  .iv = TEST_DATA_STR ("\x57\xce\x3a\x88\xf6\xd2\x72\x15\xc9\x43\x7c\x30"),
  .aad = TEST_DATA_STR (
    "\x51\x05\x96\x5c\xed\xe3\x1c\x1e\x2f\xbb\x1f\x5f\xb6\x41\xaa\x45\x65\xf8"
    "\x15\xbf\x18\x1a\x42\x9c\xdc\x35\x3b\xcf\x41\x7a\x0e\x57\xb9\x57\x49\xb4"
    "\x88\x6a\x80\x19\x01\x37\xf7\x7b\x99\xff\xe2\x80\x88\xa8\xa7\xf9\xf1\x2f"
    "\xf4\xc6\x16\x53\xdf\x30\x57\x2b\xde\xed\x92\xf2\xfa\xc5\xc4\x93\xce\x6f"
    "\xad\x20\xc0\xee\xd6\x6f\x95\x02\x6c\x76\x33\x48\x89\x20\xb9\x02\x32"
    "\xa0"),
  .tag = TEST_DATA_STR (
    "\x50\x31\x79\x2c\xa7\x0d\xc4\x9e\xeb\xd8\xea\xd3\x76\xe6\xe3\x33"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad720_tc7) = {
  .name = "192-GMAC 720-aad NIST CAVS TC7",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x22\x0a\x24\x93\x89\x79\x3c\x97\xfb\xe2\x8b\xa6\xaf"
			"\xeb\xf1\x2a\xc0\xde\x55\xed\x71\xaf\xfa\x68"),
  .iv = TEST_DATA_STR ("\xb5\xa5\x71\x95\x1a\x37\x30\x30\xfc\xf0\xeb\x4d"),
  .aad = TEST_DATA_STR (
    "\xa2\x75\x20\x58\xa8\x46\x9b\x60\xd6\x99\x7a\x31\x5e\x5c\x88\x25\xec\xb2"
    "\xf6\xfd\x1f\x60\x8d\x1a\xe5\xb5\xa4\xf5\xb4\xb9\x28\x62\xb8\x4d\x6b\x3e"
    "\x74\x4e\x92\x3b\x02\x44\xb7\xb0\xfd\x6d\x6f\x36\xa8\xc1\x73\xd4\x6a\xd2"
    "\x01\xdd\x8d\x8a\x55\xc0\x8d\x95\x49\x30\x26\x69\xb9\xd3\x3f\x46\x61\x80"
    "\xf0\x58\x1e\xb3\x00\xbb\x8a\xb8\xb0\x61\x11\x32\x34\xd9\x68\xce\xcc"
    "\xce"),
  .tag = TEST_DATA_STR (
    "\xb2\xcf\x3f\xa8\xca\x8d\x3e\xea\xaa\x3f\x82\x41\x10\x64\xc9\x87"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad720_tc8) = {
  .name = "192-GMAC 720-aad NIST CAVS TC8",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xe5\x4c\x36\xdb\xb4\x67\xfe\xb4\x30\xf4\x08\x7f\xe4"
			"\xcf\x12\xba\xfc\x94\xa1\x78\x00\x68\x38\xe8"),
  .iv = TEST_DATA_STR ("\x31\x9e\x97\x14\xb4\x92\x5c\xb1\x61\xbc\xfd\x91"),
  .aad = TEST_DATA_STR (
    "\xfc\x50\x39\x7c\xc9\x92\xfd\xe3\xd4\x44\xd2\xfd\xf3\x87\x77\xf2\x9a\xb6"
    "\x04\x99\x63\xea\x08\xc4\xe2\xf0\x0c\x15\x98\xb8\xc0\xbe\xa7\xe9\x4f\x59"
    "\x1b\xb8\x3e\xb5\x35\x1f\xfa\x4b\xff\xef\x3e\x3e\xc3\x57\xfe\x47\xb1\x7d"
    "\xb7\xee\xc0\x4a\xd4\x66\x9b\x92\x13\x02\xe5\xc4\x1a\xc6\x9f\xe4\x45\x83"
    "\x8f\xcf\xd5\xb8\xd5\x1e\x89\xb3\xef\xdf\x2e\x7a\xf4\xf0\x57\x6d\xfc"
    "\x69"),
  .tag = TEST_DATA_STR (
    "\x45\x35\x3a\x04\x31\x39\x23\x75\x54\xb5\x11\x7d\x0b\x8d\x52\xa7"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad720_tc9) = {
  .name = "192-GMAC 720-aad NIST CAVS TC9",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xaa\xd8\x0b\x3b\xb6\xe2\x2d\x9d\x18\xf1\x07\x8f\x54"
			"\x29\x73\xaa\x8f\xff\x28\xab\xfa\x2e\xd6\x37"),
  .iv = TEST_DATA_STR ("\x6b\x53\x35\x92\x9a\x6f\xc7\xd3\x4c\x3e\x72\x8f"),
  .aad = TEST_DATA_STR (
    "\x31\x4a\x33\x07\xa6\x41\x8a\xd2\x29\xaf\x5b\x03\x25\xd2\xbd\x41\x98\xfe"
    "\x82\xd8\xc5\xa8\x96\x02\xe9\x26\x84\x8c\x09\x6f\xd0\x1e\xa3\x94\x84\xdf"
    "\x6e\x4a\xae\xd1\x8f\x2e\x2b\x07\x0c\xa3\x6e\xe5\xed\x66\xcd\xa3\xc0\x4a"
    "\xb6\xeb\x41\xb3\x27\x52\x49\x4b\xa3\x56\xef\x13\x27\xd8\xfd\x6a\x83\x52"
    "\xa6\x21\xe1\xbb\x0b\x20\x66\x3f\xc7\x04\x89\x9a\x85\x5d\x32\x77\x77"
    "\x0c"),
  .tag = TEST_DATA_STR (
    "\x8f\xda\x0e\x49\x52\xbe\xef\x47\xbe\xa6\xf4\x8d\x9b\xdb\x3e\x79"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad720_tc10) = {
  .name = "192-GMAC 720-aad NIST CAVS TC10",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x69\xd3\xd0\x0e\x9a\xef\xe5\xb3\xa9\xaf\x64\x83\x8b"
			"\x40\x45\x79\xd4\x59\x2f\x9c\xfe\xe8\x64\x57"),
  .iv = TEST_DATA_STR ("\x05\x4c\xc3\x74\x8c\xd8\x44\x24\x10\x50\x3a\xd5"),
  .aad = TEST_DATA_STR (
    "\x10\x31\xcc\x7d\x96\x77\xc5\xf9\x57\x45\xc3\xdc\xc2\x6d\x62\x52\x76\x32"
    "\x35\x56\x7d\x56\xc6\x13\x86\x7b\xce\x17\xec\x09\x9d\xef\x27\x8a\x64\x37"
    "\xd1\xb7\x02\x64\x2b\xea\x5c\xfd\xed\x9a\xf6\xd0\xc5\xe0\x20\xf7\x04\x92"
    "\xad\x7f\x04\xa1\xb4\xba\xd3\x95\x3b\x96\x13\x57\x4c\x2a\x18\xce\x5f\x14"
    "\xd4\x36\x68\x79\xd1\x1e\x0b\x0a\x58\xfe\x09\x2f\x3c\xf0\xe0\x1a\xc0"
    "\x3d"),
  .tag = TEST_DATA_STR (
    "\x1b\xff\xd2\x07\x47\xb2\x5e\x87\x25\x18\x44\x68\x28\x81\xf5\x3e"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad720_tc11) = {
  .name = "192-GMAC 720-aad NIST CAVS TC11",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x03\x5e\x18\x64\xfc\xaa\x90\x78\xd1\xc8\x30\x99\xb2"
			"\x3f\xd7\x17\x8c\x94\x6a\x58\x69\xc3\x15\x77"),
  .iv = TEST_DATA_STR ("\xe2\xc4\x07\xa6\xaa\xd6\xd9\x04\x0e\x5b\x67\x49"),
  .aad = TEST_DATA_STR (
    "\x2f\xc4\x1f\x0f\xd5\xe3\xec\xef\x75\xa1\xf1\xa0\xf0\x33\x51\x5e\x6f\x96"
    "\x19\xb8\x7a\x8c\xa1\x68\x7b\xb2\xd6\x37\x52\xcc\x3d\x47\x36\x77\xdb\x30"
    "\x0e\x76\x97\x8c\xd3\x42\xc5\x1f\x57\x6b\x15\x98\x56\x75\x02\xaf\x0e\xd1"
    "\xca\x85\xc5\xde\x2d\x84\xc2\xa3\x21\x19\x61\x53\x8d\xf5\x15\x25\x0a\x69"
    "\xe8\xd6\x7e\xa2\xe8\x77\xd8\xf5\x2e\x69\x7f\xc9\x0b\xad\x33\x0b\x97"
    "\xe4"),
  .tag = TEST_DATA_STR (
    "\x3c\x90\x68\x75\x7b\xda\x60\x22\xea\xb5\xb1\x98\x75\x0b\xad\xc4"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad720_tc12) = {
  .name = "192-GMAC 720-aad NIST CAVS TC12",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x47\xcf\x37\x7a\x1d\xb1\x10\x6f\x8d\xd4\x54\x88\x4f"
			"\x71\xef\x93\xf4\xa6\x45\xe0\xe3\xc9\xd4\x30"),
  .iv = TEST_DATA_STR ("\xf4\x39\x46\xec\x30\x3f\x1e\xfc\x19\xdc\x21\xc2"),
  .aad = TEST_DATA_STR (
    "\x2e\xab\xfa\xdb\x99\x7d\x15\x4b\xea\x95\xd4\x5f\x7c\x4d\x5c\x5f\x18\x2b"
    "\x1e\xd9\x89\x7a\xb0\x12\x41\xf6\x15\xf0\x4b\x8a\x16\xf7\xa9\x65\x2b\x34"
    "\xa0\xee\x70\x52\xff\x5a\x20\x9a\xd4\xd2\x4a\x2b\xfc\x5e\x5e\xbc\x42\x4f"
    "\x6d\xbb\xf0\x33\xf0\x59\x51\x24\x7a\xb3\x73\xcb\x9c\xce\x73\x5d\x7f\xb1"
    "\x80\xa4\xf6\x2a\xd5\xa4\x12\x1e\xb7\xaa\x47\x26\x9f\x95\x41\xbd\xd9"
    "\x5a"),
  .tag = TEST_DATA_STR (
    "\xfe\xed\xe5\x21\x2f\x35\xea\xa8\xfa\xa9\xe2\xe6\xbb\x7b\x1e\x18"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad720_tc13) = {
  .name = "192-GMAC 720-aad NIST CAVS TC13",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\x64\x98\xf9\x61\x00\xe7\xb0\xb6\xed\xd7\x2b\x61\xf8"
			"\x64\xd3\x38\x23\xbc\xbd\x0b\x58\x51\xc5\x2a"),
  .iv = TEST_DATA_STR ("\x81\xf0\x05\xdf\x39\x2a\xc0\x25\x0a\xe0\x7a\x69"),
  .aad = TEST_DATA_STR (
    "\xd1\x83\x82\x41\x68\x23\x15\xdc\x27\x3a\xe8\xc2\xd5\x9d\x71\x27\x17\x48"
    "\xbf\x1e\xf0\x38\x5d\xe4\x05\xfc\x5c\x2f\xe5\xca\xcf\x57\xc8\xd5\x1d\x72"
    "\xdf\x09\x6d\x2c\x3e\x46\x63\xf1\xc5\x9b\xd4\xda\x3c\xfe\xe9\x4e\x53\xab"
    "\xa8\x7e\x49\x3a\xad\x38\x6b\xb3\x28\x3d\xd3\x37\xa0\xba\x57\xb8\x4f\x2d"
    "\x35\xa8\xb6\xbf\xb2\x07\x7d\x22\xb8\x23\x98\xff\x6c\x34\x31\xec\xc4"
    "\xf6"),
  .tag = TEST_DATA_STR (
    "\xe1\x49\xfb\xaa\x73\xf0\x50\x9d\x34\xbd\xdf\x03\x1c\x4c\xc4\x76"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac192_aad720_tc14) = {
  .name = "192-GMAC 720-aad NIST CAVS TC14",
  .alg = VNET_CRYPTO_ALG_AES_192_NULL_GMAC,
  .key = TEST_DATA_STR ("\xba\xff\x99\xa6\xdd\x4d\x29\x81\x04\x3a\x48\xb5\x2f"
			"\x36\xba\x5d\xbb\x73\x80\xca\xa7\x5b\xc6\x5d"),
  .iv = TEST_DATA_STR ("\x98\x38\xd9\xf9\xb8\x63\x2c\xbd\x48\xa2\xba\x35"),
  .aad = TEST_DATA_STR (
    "\xe7\x81\xf8\xf1\xf5\xbf\xad\x3a\x50\xc4\x7e\x36\x33\x5e\x7a\x22\x5d\xbf"
    "\x32\xbc\x15\x96\x7d\x66\xdd\x30\x06\xdd\x42\x4b\xa9\x71\xd8\xf1\xa9\xca"
    "\x90\x61\x94\x50\xbd\xa4\x56\x29\x39\x01\x5f\x75\xb4\x67\xd6\x33\xbb\x57"
    "\x43\xbb\xf3\x7c\x9a\x2b\x24\x15\xd7\x30\x65\xfa\xd7\x1d\xa3\x31\x2d\x81"
    "\x7b\xa2\xe6\x24\xc6\x88\x63\xf7\x22\x78\x05\x2a\x4d\xb0\xe7\x3d\xbf"
    "\x10"),
  .tag = TEST_DATA_STR (
    "\xf8\xed\xe3\x60\x48\x26\x1d\x8a\x3b\xf7\x8b\x19\x33\xf3\x3b\x22"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad0_tc0) = {
  .name = "256-GMAC 0-aad NIST CAVS TC0",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xb5\x2c\x50\x5a\x37\xd7\x8e\xda\x5d\xd3\x4f\x20\xc2\x25\x40\xea\x1b\x58"
    "\x96\x3c\xf8\xe5\xbf\x8f\xfa\x85\xf9\xf2\x49\x25\x05\xb4"),
  .iv = TEST_DATA_STR ("\x51\x6c\x33\x92\x9d\xf5\xa3\x28\x4f\xf4\x63\xd7"),
  .tag = TEST_DATA_STR (
    "\xbd\xc1\xac\x88\x4d\x33\x24\x57\xa1\xd2\x66\x4f\x16\x8c\x76\xf0"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad0_tc1) = {
  .name = "256-GMAC 0-aad NIST CAVS TC1",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x5f\xe0\x86\x1c\xdc\x26\x90\xce\x69\xb3\x65\x8c\x7f\x26\xf8\x45\x8e\xec"
    "\x1c\x92\x43\xc5\xba\x08\x45\x30\x5d\x89\x7e\x96\xca\x0f"),
  .iv = TEST_DATA_STR ("\x77\x0a\xc1\xa5\xa3\xd4\x76\xd5\xd9\x69\x44\xa1"),
  .tag = TEST_DATA_STR (
    "\x19\x6d\x69\x1e\x10\x47\x09\x3c\xa4\xb3\xd2\xef\x4b\xab\xa2\x16"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad0_tc2) = {
  .name = "256-GMAC 0-aad NIST CAVS TC2",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x76\x20\xb7\x9b\x17\xb2\x1b\x06\xd9\x70\x19\xaa\x70\xe1\xca\x10\x5e\x1c"
    "\x03\xd2\xa0\xcf\x8b\x20\xb5\xa0\xce\x5c\x39\x03\xe5\x48"),
  .iv = TEST_DATA_STR ("\x60\xf5\x6e\xb7\xa4\xb3\x8d\x4f\x03\x39\x55\x11"),
  .tag = TEST_DATA_STR (
    "\xf5\x70\xc3\x82\x02\xd9\x45\x64\xba\xb3\x9f\x75\x61\x7b\xc8\x7a"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad0_tc3) = {
  .name = "256-GMAC 0-aad NIST CAVS TC3",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x7e\x2d\xb0\x03\x21\x18\x94\x76\xd1\x44\xc5\xf2\x7e\x78\x70\x87\x30\x2a"
    "\x48\xb5\xf7\x78\x6c\xd9\x1e\x93\x64\x16\x28\xc2\x32\x8b"),
  .iv = TEST_DATA_STR ("\xea\x9d\x52\x5b\xf0\x1d\xe7\xb2\x23\x4b\x60\x6a"),
  .tag = TEST_DATA_STR (
    "\xdb\x9d\xf5\xf1\x4f\x6c\x9f\x2a\xe8\x1f\xd4\x21\x41\x2d\xdb\xbb"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad0_tc4) = {
  .name = "256-GMAC 0-aad NIST CAVS TC4",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xa2\x3d\xfb\x84\xb5\x97\x6b\x46\xb1\x83\x0d\x93\xbc\xf6\x19\x41\xca\xe5"
    "\xe4\x09\xe4\xf5\x55\x1d\xc6\x84\xbd\xce\xf9\x87\x64\x80"),
  .iv = TEST_DATA_STR ("\x5a\xa3\x45\x90\x80\x48\xde\x10\xa2\xbd\x3d\x32"),
  .tag = TEST_DATA_STR (
    "\xf2\x82\x17\x64\x92\x30\xbd\x7a\x40\xa9\xa4\xdd\xab\xc6\x7c\x43"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad0_tc5) = {
  .name = "256-GMAC 0-aad NIST CAVS TC5",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xdf\xe9\x28\xf8\x64\x30\xb7\x8a\xdd\x7b\xb7\x69\x60\x23\xe6\x15\x3d\x76"
    "\x97\x7e\x56\x10\x3b\x18\x02\x53\x49\x0a\xff\xb9\x43\x1c"),
  .iv = TEST_DATA_STR ("\x1d\xd0\x78\x5a\xf9\xf5\x89\x79\xa1\x0b\xd6\x2d"),
  .tag = TEST_DATA_STR (
    "\xa5\x5e\xb0\x9e\x9e\xde\xf5\x8d\x9f\x67\x1d\x72\x20\x7f\x8b\x3c"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad0_tc6) = {
  .name = "256-GMAC 0-aad NIST CAVS TC6",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x34\x04\x8d\xb8\x15\x91\xee\x68\x22\x49\x56\xbd\x69\x89\xe1\x63\x0f\xcf"
    "\x06\x8d\x7f\xf7\x26\xae\x81\xe5\xb2\x9f\x54\x8c\xfc\xfb"),
  .iv = TEST_DATA_STR ("\x16\x21\xd3\x4c\xff\x2a\x5b\x25\x0c\x7b\x76\xfc"),
  .tag = TEST_DATA_STR (
    "\x49\x92\xec\x3d\x57\xcc\xcf\xa5\x8f\xd8\x91\x6c\x59\xb7\x0b\x11"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad0_tc7) = {
  .name = "256-GMAC 0-aad NIST CAVS TC7",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xa1\x11\x4f\x87\x49\xc7\x2b\x8c\xef\x62\xe7\x50\x3f\x1a\xd9\x21\xd3\x3e"
    "\xee\xde\x32\xb0\xb5\xb8\xe0\xd6\x80\x7a\xa2\x33\xd0\xad"),
  .iv = TEST_DATA_STR ("\xa1\x90\xed\x3f\xf2\xe2\x38\xbe\x56\xf9\x0b\xd6"),
  .tag = TEST_DATA_STR (
    "\xc8\x46\x4d\x95\xd5\x40\xfb\x19\x11\x56\xfb\xbc\x16\x08\x84\x2a"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad0_tc8) = {
  .name = "256-GMAC 0-aad NIST CAVS TC8",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xdd\xbb\x99\xdc\x31\x02\xd3\x11\x02\xc0\xe1\x4b\x23\x85\x18\x60\x57\x66"
    "\xc5\xb2\x3d\x9b\xea\x52\xc7\xc5\xa7\x71\x04\x2c\x85\xa0"),
  .iv = TEST_DATA_STR ("\x95\xd1\x5e\xd7\x5c\x6a\x10\x9a\xac\x1b\x1d\x86"),
  .tag = TEST_DATA_STR (
    "\x81\x3d\x1d\xa3\x77\x5c\xac\xd7\x8e\x96\xd8\x6f\x03\x6c\xff\x96"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad0_tc9) = {
  .name = "256-GMAC 0-aad NIST CAVS TC9",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x1f\xaa\x50\x6b\x8f\x13\xa2\xe6\x66\x0a\xf7\x8d\x92\x91\x5a\xdf\x33\x36"
    "\x58\xf7\x48\xf4\xe4\x8f\xa2\x01\x35\xa2\x9e\x9a\xbe\x5f"),
  .iv = TEST_DATA_STR ("\xe5\x0f\x27\x8d\x36\x62\xc9\x9d\x75\x0f\x60\xd3"),
  .tag = TEST_DATA_STR (
    "\xae\xc7\xec\xe6\x6b\x73\x44\xaf\xd6\xf6\xcc\x74\x19\xcf\x60\x27"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad0_tc10) = {
  .name = "256-GMAC 0-aad NIST CAVS TC10",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xf3\x0b\x59\x42\xfa\xf5\x7d\x4c\x13\xe7\xa8\x24\x95\xae\xdf\x1b\x4e\x60"
    "\x35\x39\xb2\xe1\x59\x93\x17\xcc\x6e\x53\x22\x5a\x24\x93"),
  .iv = TEST_DATA_STR ("\x33\x6c\x38\x8e\x18\xe6\xab\xf9\x2b\xb7\x39\xa9"),
  .tag = TEST_DATA_STR (
    "\xdd\xaf\x8e\xf4\xcb\x2f\x8a\x6d\x40\x1f\x3b\xe5\xff\x0b\xaf\x6a"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad0_tc11) = {
  .name = "256-GMAC 0-aad NIST CAVS TC11",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xda\xf4\xd9\xc1\x2c\x5d\x29\xfc\x3f\xa9\x36\x53\x2c\x96\x19\x6e\x56\xae"
    "\x84\x2e\x47\x06\x3a\x4b\x29\xbf\xff\x2a\x35\xed\x92\x80"),
  .iv = TEST_DATA_STR ("\x53\x81\xf2\x11\x97\xe0\x93\xb9\x6c\xda\xc4\xfa"),
  .tag = TEST_DATA_STR (
    "\x7f\x18\x32\xc7\xf7\xcd\x78\x12\xa0\x04\xb7\x9c\x3d\x39\x94\x73"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad0_tc12) = {
  .name = "256-GMAC 0-aad NIST CAVS TC12",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x6b\x52\x47\x54\x14\x9c\x81\x40\x1d\x29\xa4\xb8\xa6\xf4\xa4\x78\x33\x37"
    "\x28\x06\xb2\xd4\x08\x3f\xf1\x7f\x2d\xb3\xbf\xc1\x7b\xca"),
  .iv = TEST_DATA_STR ("\xac\x7d\x3d\x61\x8a\xb6\x90\x55\x5e\xc2\x44\x08"),
  .tag = TEST_DATA_STR (
    "\xdb\x07\xa8\x85\xe2\xbd\x39\xda\x74\x11\x6d\x06\xc3\x16\xa5\xc9"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad0_tc13) = {
  .name = "256-GMAC 0-aad NIST CAVS TC13",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xcf\xf0\x83\x30\x3f\xf4\x0a\x1f\x66\xc4\xae\xd1\xac\x7f\x50\x62\x8f\xe7"
    "\xe9\x31\x1f\x5d\x03\x7e\xbf\x49\xf4\xa4\xb9\xf0\x22\x3f"),
  .iv = TEST_DATA_STR ("\x45\xd4\x6e\x1b\xaa\xdc\xfb\xc8\xf0\xe9\x22\xff"),
  .tag = TEST_DATA_STR (
    "\x16\x87\xc6\xd4\x59\xea\x48\x1b\xf8\x8e\x4b\x22\x63\x22\x79\x06"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad0_tc14) = {
  .name = "256-GMAC 0-aad NIST CAVS TC14",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x39\x54\xf6\x0c\xdd\xbb\x39\xd2\xd8\xb0\x58\xad\xf5\x45\xd5\xb8\x24\x90"
    "\xc8\xae\x92\x83\xaf\xa5\x27\x86\x89\x04\x1d\x41\x5a\x3a"),
  .iv = TEST_DATA_STR ("\x8f\xb3\xd9\x8e\xf2\x4f\xba\x03\x74\x6a\xc8\x4f"),
  .tag = TEST_DATA_STR (
    "\x7f\xb1\x30\x85\x5d\xfe\x7a\x37\x33\x13\x36\x1f\x33\xf5\x52\x37"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad128_tc0) = {
  .name = "256-GMAC 128-aad NIST CAVS TC0",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x78\xdc\x4e\x0a\xaf\x52\xd9\x35\xc3\xc0\x1e\xea\x57\x42\x8f\x00\xca\x1f"
    "\xd4\x75\xf5\xda\x86\xa4\x9c\x8d\xd7\x3d\x68\xc8\xe2\x23"),
  .iv = TEST_DATA_STR ("\xd7\x9c\xf2\x2d\x50\x4c\xc7\x93\xc3\xfb\x6c\x8a"),
  .aad = TEST_DATA_STR (
    "\xb9\x6b\xaa\x8c\x1c\x75\xa6\x71\xbf\xb2\xd0\x8d\x06\xbe\x5f\x36"),
  .tag = TEST_DATA_STR (
    "\x3e\x5d\x48\x6a\xa2\xe3\x0b\x22\xe0\x40\xb8\x57\x23\xa0\x6e\x76"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad128_tc1) = {
  .name = "256-GMAC 128-aad NIST CAVS TC1",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x44\x57\xff\x33\x68\x3c\xca\x6c\xa4\x93\x87\x8b\xdc\x00\x37\x38\x93\xa9"
    "\x76\x34\x12\xee\xf8\xcd\xdb\x54\xf9\x13\x18\xe0\xda\x88"),
  .iv = TEST_DATA_STR ("\x69\x9d\x1f\x29\xd7\xb8\xc5\x53\x00\xbb\x1f\xd2"),
  .aad = TEST_DATA_STR (
    "\x67\x49\xda\xee\xa3\x67\xd0\xe9\x80\x9e\x2d\xc2\xf3\x09\xe6\xe3"),
  .tag = TEST_DATA_STR (
    "\xd6\x0c\x74\xd2\x51\x7f\xde\x4a\x74\xe0\xcd\x47\x09\xed\x43\xa9"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad128_tc2) = {
  .name = "256-GMAC 128-aad NIST CAVS TC2",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x4d\x01\xc9\x6e\xf9\xd9\x8d\x4f\xb4\xe9\xb6\x1b\xe5\xef\xa7\x72\xc9\x78"
    "\x85\x45\xb3\xea\xc3\x9e\xb1\xca\xcb\x99\x7a\x5f\x07\x92"),
  .iv = TEST_DATA_STR ("\x32\x12\x4a\x4d\x9e\x57\x6a\xea\x25\x89\xf2\x38"),
  .aad = TEST_DATA_STR (
    "\xd7\x2b\xad\x0c\x38\x49\x5e\xda\x50\xd5\x58\x11\x94\x5e\xe2\x05"),
  .tag = TEST_DATA_STR (
    "\x6d\x63\x97\xc9\xe2\x03\x0f\x5b\x80\x53\xbf\xe5\x10\xf3\xf2\xcf"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad128_tc3) = {
  .name = "256-GMAC 128-aad NIST CAVS TC3",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x83\x78\x19\x3a\x4c\xe6\x41\x80\x81\x4b\xd6\x05\x91\xd1\x05\x4a\x04\xdb"
    "\xc4\xda\x02\xaf\xde\x45\x37\x99\xcd\x68\x88\xee\x0c\x6c"),
  .iv = TEST_DATA_STR ("\xbd\x8b\x4e\x35\x2c\x7f\x69\x87\x8a\x47\x54\x35"),
  .aad = TEST_DATA_STR (
    "\x1c\x6b\x34\x3c\x4d\x04\x5c\xbb\xa5\x62\xba\xe3\xe5\xff\x1b\x18"),
  .tag = TEST_DATA_STR (
    "\x08\x33\x96\x7a\x6a\x53\xba\x24\xe7\x5c\x03\x72\xa6\xa1\x7b\xda"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad128_tc4) = {
  .name = "256-GMAC 128-aad NIST CAVS TC4",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x22\xfc\x82\xdb\x5b\x60\x69\x98\xad\x45\x09\x9b\x79\x78\xb5\xb4\xf9\xdd"
    "\x4e\xa6\x01\x7e\x57\x37\x0a\xc5\x61\x41\xca\xaa\xbd\x12"),
  .iv = TEST_DATA_STR ("\x88\x0d\x05\xc5\xee\x59\x9e\x5f\x15\x1e\x30\x2f"),
  .aad = TEST_DATA_STR (
    "\x3e\x3e\xb5\x74\x7e\x39\x0f\x7b\xc8\x0e\x74\x82\x33\x48\x4f\xfc"),
  .tag = TEST_DATA_STR (
    "\x2e\x12\x2a\x47\x8e\x64\x46\x32\x86\xf8\xb4\x89\xdc\xdd\x09\xc8"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad128_tc5) = {
  .name = "256-GMAC 128-aad NIST CAVS TC5",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xfc\x00\x96\x0d\xdd\x69\x8d\x35\x72\x8c\x5a\xc6\x07\x59\x6b\x51\xb3\xf8"
    "\x97\x41\xd1\x4c\x25\xb8\xba\xda\xc9\x19\x76\x12\x0d\x99"),
  .iv = TEST_DATA_STR ("\xa4\x24\xa3\x2a\x23\x7f\x0d\xf5\x30\xf0\x5e\x30"),
  .aad = TEST_DATA_STR (
    "\xcf\xb7\xe0\x5e\x31\x57\xf0\xc9\x05\x49\xd5\xc7\x86\x50\x63\x11"),
  .tag = TEST_DATA_STR (
    "\xdc\xdc\xb9\xe4\x00\x4b\x85\x2a\x0d\xa1\x2b\xdf\x25\x5b\x4d\xdd"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad128_tc6) = {
  .name = "256-GMAC 128-aad NIST CAVS TC6",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x69\x74\x99\x43\x09\x2f\x56\x05\xbf\x97\x1e\x18\x5c\x19\x1c\x61\x82\x61"
    "\xb2\xc7\xcc\x16\x93\xcd\xa1\x08\x0c\xa2\xfd\x8d\x51\x11"),
  .iv = TEST_DATA_STR ("\xbd\x0d\x62\xc0\x2e\xe6\x82\x06\x9b\xd1\xe1\x28"),
  .aad = TEST_DATA_STR (
    "\x69\x67\xdc\xe8\x78\xf0\x3b\x64\x3b\xf5\xcd\xba\x59\x6a\x7a\xf3"),
  .tag = TEST_DATA_STR (
    "\x37\x8f\x79\x6a\xe5\x43\xe1\xb2\x91\x15\xcc\x18\xac\xd1\x93\xf4"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad128_tc7) = {
  .name = "256-GMAC 128-aad NIST CAVS TC7",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xfc\x48\x75\xdb\x84\x81\x98\x34\xb1\xcb\x43\x82\x8d\x2f\x0a\xe3\x47\x3a"
    "\xa3\x80\x11\x1c\x27\x37\xe8\x2a\x9a\xb1\x1f\xea\x1f\x19"),
  .iv = TEST_DATA_STR ("\xda\x6a\x68\x4d\x3f\xf6\x3a\x2d\x10\x9d\xec\xd6"),
  .aad = TEST_DATA_STR (
    "\x91\xb6\xfa\x2a\xb4\xde\x44\x28\x2f\xfc\x86\xc8\xcd\xe6\xe7\xf5"),
  .tag = TEST_DATA_STR (
    "\x50\x4e\x81\xd2\xe7\x87\x7e\x4d\xad\x6f\x31\xcd\xeb\x07\xbd\xbd"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad128_tc8) = {
  .name = "256-GMAC 128-aad NIST CAVS TC8",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x9f\x9f\xe7\xd2\xa2\x6d\xcf\x59\xd6\x84\xf1\xc0\x94\x5b\x5f\xfa\xfe\x0a"
    "\x47\x46\x84\x5e\xd3\x17\xd3\x5f\x3e\xd7\x6c\x93\x04\x4d"),
  .iv = TEST_DATA_STR ("\x13\xb5\x99\x71\xcd\x4d\xd3\x6b\x19\xac\x71\x04"),
  .aad = TEST_DATA_STR (
    "\x19\x0a\x69\x34\xf4\x5f\x89\xc9\x00\x67\xc2\xf6\x2e\x04\xc5\x3b"),
  .tag = TEST_DATA_STR (
    "\x4f\x63\x6a\x29\x4b\xfb\xf5\x1f\xc0\xe1\x31\xd6\x94\xd5\xc2\x22"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad128_tc9) = {
  .name = "256-GMAC 128-aad NIST CAVS TC9",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xab\x91\x55\xd7\xd8\x1b\xa6\xf3\x31\x93\x69\x5c\xf4\x56\x6a\x9b\x6e\x97"
    "\xa3\xe4\x09\xf5\x71\x59\xae\x6c\xa4\x96\x55\xcc\xa0\x71"),
  .iv = TEST_DATA_STR ("\x26\xa9\xf8\xd6\x65\xd1\x63\xdd\xb9\x2d\x03\x5d"),
  .aad = TEST_DATA_STR (
    "\x4a\x20\x3a\xc2\x6b\x95\x1a\x1f\x67\x3c\x66\x05\x65\x3e\xc0\x2d"),
  .tag = TEST_DATA_STR (
    "\x43\x7e\xa7\x7a\x38\x79\xf0\x10\x69\x1e\x28\x8d\x62\x69\xa9\x96"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad128_tc10) = {
  .name = "256-GMAC 128-aad NIST CAVS TC10",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x0f\x1c\x62\xdd\x80\xb4\xa6\xd0\x9e\xe9\xd7\x87\xb1\xb0\x43\x27\xaa\x36"
    "\x15\x29\xff\xa3\x40\x75\x60\x41\x4a\xc4\x7b\x7e\xf7\xbc"),
  .iv = TEST_DATA_STR ("\xc8\x76\x13\xa3\xb7\x0d\x2a\x04\x8f\x32\xcb\x9a"),
  .aad = TEST_DATA_STR (
    "\x8f\x23\xd4\x04\xbe\x2d\x9e\x88\x8d\x21\x9f\x1b\x40\xaa\x29\xe8"),
  .tag = TEST_DATA_STR (
    "\x36\xd8\xa3\x09\xac\xbb\x87\x16\xc9\xc0\x8c\x7f\x5d\xe4\x91\x1e"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad128_tc11) = {
  .name = "256-GMAC 128-aad NIST CAVS TC11",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xf3\xe9\x54\xa3\x89\x56\xdf\x89\x02\x55\xf0\x17\x09\xe4\x57\xb3\x3f\x4b"
    "\xfe\x7e\xcb\x36\xd0\xee\x50\xf2\x50\x04\x71\xee\xbc\xde"),
  .iv = TEST_DATA_STR ("\x97\x99\xab\xd3\xc5\x21\x10\xc7\x04\xb0\xf3\x6a"),
  .aad = TEST_DATA_STR (
    "\xdd\xb7\x01\x73\xf4\x41\x57\x75\x5b\x6c\x9b\x70\x58\xf4\x0c\xb7"),
  .tag = TEST_DATA_STR (
    "\xb3\x23\xae\x3a\xbc\xb4\x15\xc7\xf4\x20\x87\x6c\x98\x0f\x48\x58"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad128_tc12) = {
  .name = "256-GMAC 128-aad NIST CAVS TC12",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x06\x25\x31\x65\x34\xfb\xd8\x2f\xe8\xfd\xea\x50\xfa\x57\x3c\x46\x20\x22"
    "\xc4\x2f\x79\xe8\xb2\x13\x60\xe5\xa6\xdc\xe6\x6d\xde\x28"),
  .iv = TEST_DATA_STR ("\xda\x64\xa6\x74\x90\x7c\xd6\xcf\x24\x8f\x5f\xbb"),
  .aad = TEST_DATA_STR (
    "\xf2\x4d\x48\xe0\x4f\x5a\x0d\x98\x7b\xa7\xc7\x45\xb7\x3b\x03\x64"),
  .tag = TEST_DATA_STR (
    "\xdf\x36\x0b\x81\x0f\x27\xe7\x94\x67\x3a\x8b\xb2\xdc\x0d\x68\xb0"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad128_tc13) = {
  .name = "256-GMAC 128-aad NIST CAVS TC13",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x28\xf0\x45\xac\x7c\x4f\xe5\xd4\xb0\x1a\x9d\xcd\x5f\x1a\xd3\xef\xff\x1c"
    "\x4f\x17\x0f\xc8\xab\x87\x58\xd9\x72\x92\x86\x8d\x58\x28"),
  .iv = TEST_DATA_STR ("\x5d\x85\xde\x95\xb0\xbd\xc4\x45\x14\x14\x39\x19"),
  .aad = TEST_DATA_STR (
    "\x60\x1d\x21\x58\xf1\x7a\xb3\xc7\xb4\xdc\xb6\x95\x0f\xbd\xcd\xde"),
  .tag = TEST_DATA_STR (
    "\x42\xc3\xf5\x27\x41\x8c\xf2\xc3\xf5\xd5\x01\x0c\xcb\xa8\xf2\x71"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad128_tc14) = {
  .name = "256-GMAC 128-aad NIST CAVS TC14",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x19\x31\x0e\xed\x5f\x5f\x44\xeb\x47\x07\x5c\x10\x5e\xb3\x1e\x36\xbb\xfd"
    "\x13\x10\xf7\x41\xb9\xba\xa6\x6a\x81\x13\x8d\x35\x72\x42"),
  .iv = TEST_DATA_STR ("\xa1\x24\x71\x20\x13\x8f\xa4\xf0\xe9\x6c\x99\x2c"),
  .aad = TEST_DATA_STR (
    "\x29\xd7\x46\x41\x43\x33\xe0\xf7\x2b\x4c\x3f\x44\xec\x6b\xfe\x42"),
  .tag = TEST_DATA_STR (
    "\xd5\x99\x7e\x2f\x95\x6d\xf3\xfa\x2c\x23\x88\xe2\x0f\x30\xc4\x80"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad160_tc0) = {
  .name = "256-GMAC 160-aad NIST CAVS TC0",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x88\x6c\xff\x5f\x3e\x6b\x8d\x0e\x1a\xd0\xa3\x8f\xcd\xb2\x6d\xe9\x7e\x8a"
    "\xcb\xe7\x9f\x6b\xed\x66\x95\x9a\x59\x8f\xa5\x04\x7d\x65"),
  .iv = TEST_DATA_STR ("\x3a\x8e\xfa\x1c\xd7\x4b\xba\xb5\x44\x8f\x99\x45"),
  .aad = TEST_DATA_STR ("\x51\x9f\xee\x51\x9d\x25\xc7\xa3\x04\xd6\xc6\xaa\x18"
			"\x97\xee\x1e\xb8\xc5\x96\x55"),
  .tag = TEST_DATA_STR (
    "\xf6\xd4\x75\x05\xec\x96\xc9\x8a\x42\xdc\x3a\xe7\x19\x87\x7b\x87"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad160_tc1) = {
  .name = "256-GMAC 160-aad NIST CAVS TC1",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x69\x37\xa5\x7d\x35\xfe\x6d\xc3\xfc\x42\x0b\x12\x3b\xcc\xdc\xe8\x74\xbd"
    "\x4c\x18\xf2\xe7\xc0\x1c\xe2\xfa\xf3\x3d\x39\x44\xfd\x9d"),
  .iv = TEST_DATA_STR ("\xa8\x72\x47\x79\x7b\x75\x84\x67\xb9\x63\x10\xf3"),
  .aad = TEST_DATA_STR ("\xea\xd9\x61\x93\x9a\x33\xdd\x57\x8f\x8e\x93\xdb\x8b"
			"\x28\xa1\xc8\x53\x62\x90\x5f"),
  .tag = TEST_DATA_STR (
    "\x59\x9d\xe3\xec\xf2\x2c\xb8\x67\xf0\x3f\x7f\x6d\x9f\xd7\x42\x8a"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad160_tc2) = {
  .name = "256-GMAC 160-aad NIST CAVS TC2",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xe6\x5a\x33\x17\x76\xc9\xdc\xdf\x5e\xba\x6c\x59\xe0\x5e\xc0\x79\xd9\x74"
    "\x73\xbc\xdc\xe8\x4d\xaf\x83\x6b\xe3\x23\x45\x62\x63\xa0"),
  .iv = TEST_DATA_STR ("\xca\x73\x1f\x76\x8d\xa0\x1d\x02\xeb\x8e\x72\x7e"),
  .aad = TEST_DATA_STR ("\xd7\x27\x45\x86\x51\x7b\xf1\xd8\xda\x86\x6f\x4a\x47"
			"\xad\x0b\xcf\x29\x48\xa8\x62"),
  .tag = TEST_DATA_STR (
    "\xa8\xab\xe7\xa8\x08\x5f\x25\x13\x0a\x72\x06\xd3\x7a\x8a\xaf\x6d"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad160_tc3) = {
  .name = "256-GMAC 160-aad NIST CAVS TC3",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x77\xbb\x1b\x6e\xf8\x98\x68\x3c\x98\x1b\x2f\xc8\x99\x31\x9f\xfb\xb6\x00"
    "\x0e\xdc\xa2\x25\x66\xb6\x34\xdb\x3a\x3c\x80\x40\x59\xe5"),
  .iv = TEST_DATA_STR ("\x35\x4a\x19\x28\x37\x69\xb3\xb9\x91\xb0\x5a\x4c"),
  .aad = TEST_DATA_STR ("\xb5\x56\x62\x51\xa8\xa8\xbe\xc2\x12\xdc\x08\x11\x32"
			"\x29\xff\x85\x90\x16\x88\x00"),
  .tag = TEST_DATA_STR (
    "\xe5\xc2\xdc\xcf\x8f\xc7\xf2\x96\xca\xc9\x5d\x70\x71\xcb\x8d\x7d"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad160_tc4) = {
  .name = "256-GMAC 160-aad NIST CAVS TC4",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x2a\x43\x30\x8d\x52\x0a\x59\xed\x51\xe4\x7a\x3a\x91\x5e\x1d\xbf\x20\xa9"
    "\x1f\x08\x86\x50\x6e\x48\x1a\xd3\xde\x65\xd5\x09\x75\xb4"),
  .iv = TEST_DATA_STR ("\xbc\xbf\x99\x73\x3d\x8e\xc9\x0c\xb2\x3e\x6c\xe6"),
  .aad = TEST_DATA_STR ("\xeb\x88\x28\x87\x29\x28\x9d\x26\xfe\x0e\x75\x7a\x99"
			"\xad\x8e\xec\x96\x10\x60\x53"),
  .tag = TEST_DATA_STR (
    "\x01\xb0\x19\x69\x33\xaa\x49\x12\x3e\xab\x4e\x15\x71\x25\x03\x83"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad160_tc5) = {
  .name = "256-GMAC 160-aad NIST CAVS TC5",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x23\x79\xb3\x5f\x85\x10\x2d\xb4\xe7\xae\xcc\x52\xb7\x05\xbc\x69\x5d\x47"
    "\x68\xd4\x12\xe2\xd7\xbe\xbe\x99\x92\x36\x78\x39\x72\xff"),
  .iv = TEST_DATA_STR ("\x91\x89\x98\xc4\x80\x10\x37\xb1\xcd\x10\x2f\xaa"),
  .aad = TEST_DATA_STR ("\xb3\x72\x23\x09\xe0\xf0\x66\x22\x5e\x8d\x16\x59\x08"
			"\x4e\xbb\x07\xa9\x3b\x43\x5d"),
  .tag = TEST_DATA_STR (
    "\xdf\xb1\x8a\xee\x99\xd1\xf6\x7f\x57\x48\xd4\xb4\x84\x3c\xb6\x49"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad160_tc6) = {
  .name = "256-GMAC 160-aad NIST CAVS TC6",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x98\xb3\xcb\x75\x37\x16\x7e\x6d\x14\xa2\xa8\xb2\x31\x0f\xe9\x4b\x71\x5c"
    "\x72\x9f\xdf\x85\x21\x65\x68\x15\x0b\x55\x6d\x07\x97\xba"),
  .iv = TEST_DATA_STR ("\xbc\xa5\xe2\xe5\xa6\xb3\x0f\x18\xd2\x63\xc6\xb2"),
  .aad = TEST_DATA_STR ("\x26\x0d\x3d\x72\xdb\x70\xd6\x77\xa4\xe3\xe1\xf3\xe1"
			"\x14\x31\x21\x7a\x2e\x47\x13"),
  .tag = TEST_DATA_STR (
    "\xd6\xb7\x56\x0f\x8a\xc2\xf0\xa9\x0b\xad\x42\xa6\xa0\x72\x04\xbc"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad160_tc7) = {
  .name = "256-GMAC 160-aad NIST CAVS TC7",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x30\x34\x1a\xe0\xf1\x99\xb1\x0a\x15\x17\x5d\x00\x91\x3d\x50\x29\x52\x6a"
    "\xb7\xf7\x61\xc0\xb9\x36\xa7\xdd\x5f\x1b\x15\x83\x42\x9d"),
  .iv = TEST_DATA_STR ("\xdb\xe1\x09\xa8\xce\x5f\x7b\x24\x1e\x99\xf7\xaf"),
  .aad = TEST_DATA_STR ("\xfe\x4b\xde\xe5\xca\x9c\x48\x06\xfa\x02\x47\x15\xfb"
			"\xf6\x6a\xb8\x45\x28\x5f\xa7"),
  .tag = TEST_DATA_STR (
    "\xae\x91\xda\xed\x65\x8e\x26\xc0\xd1\x26\x57\x51\x47\xaf\x98\x99"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad160_tc8) = {
  .name = "256-GMAC 160-aad NIST CAVS TC8",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x82\x32\xb6\xa1\xd2\xe3\x67\xe9\xce\x1e\xa8\xd4\x2f\xcf\xc8\x3a\x4b\xc8"
    "\xbd\xec\x46\x5c\x6b\xa3\x26\xe3\x53\xad\x92\x55\xf2\x07"),
  .iv = TEST_DATA_STR ("\xcd\x2f\xb5\xff\x9c\xf0\xf3\x98\x68\xad\x86\x85"),
  .aad = TEST_DATA_STR ("\x02\x41\x8b\x3d\xde\x54\x92\x4a\x96\x28\xde\x06\x00"
			"\x4c\x08\x82\xae\x4e\xc3\xbb"),
  .tag = TEST_DATA_STR (
    "\xd5\x30\x8f\x63\x70\x86\x75\xce\xd1\x9b\x27\x10\xaf\xd2\xdb\x49"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad160_tc9) = {
  .name = "256-GMAC 160-aad NIST CAVS TC9",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xf9\xa1\x32\xa5\x0a\x50\x81\x45\xff\xd8\x29\x4e\x68\x94\x4e\xa4\x36\xce"
    "\x0f\x9a\x97\xe1\x81\xf5\xe0\xd6\xc5\xd2\x72\x31\x1f\xc1"),
  .iv = TEST_DATA_STR ("\x89\x29\x91\xb5\x4e\x94\xb9\xd5\x74\x42\xcc\xaf"),
  .aad = TEST_DATA_STR ("\x4e\x0f\xbd\x37\x99\xda\x25\x0f\xa2\x79\x11\xb7\xe6"
			"\x8d\x76\x23\xbf\xe6\x0a\x53"),
  .tag = TEST_DATA_STR (
    "\x89\x88\x1d\x5f\x78\x6e\x6d\x53\xe0\xd1\x9c\x3b\x4e\x68\x87\xd8"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad160_tc10) = {
  .name = "256-GMAC 160-aad NIST CAVS TC10",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x0e\x37\x46\xe5\x06\x46\x33\xea\x93\x11\xb2\xb8\x42\x7c\x53\x6a\xf9\x27"
    "\x17\xde\x20\xee\xb6\x26\x0d\xb1\x33\x3c\x3d\x8a\x81\x14"),
  .iv = TEST_DATA_STR ("\xf8\x4c\x3a\x1c\x94\x53\x3f\x7f\x25\xce\xc0\xac"),
  .aad = TEST_DATA_STR ("\x8c\x0d\x41\xe6\x13\x53\x38\xc8\xd3\xe6\x3e\x2a\x5f"
			"\xa0\xa9\x66\x7e\xc9\xa5\x80"),
  .tag = TEST_DATA_STR (
    "\x47\x9c\xcf\xe9\x24\x1d\xe2\xc4\x74\xf2\xed\xeb\xbb\x38\x5c\x09"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad160_tc11) = {
  .name = "256-GMAC 160-aad NIST CAVS TC11",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xb9\x97\xe9\xb0\x74\x6a\xba\xae\xd6\xe6\x4b\x63\xbd\xf6\x48\x82\x52\x6a"
    "\xd9\x2e\x24\xa2\xf5\x64\x9d\xf0\x55\xc9\xec\x0f\x1d\xaa"),
  .iv = TEST_DATA_STR ("\xf1\x41\xd8\xd7\x1b\x03\x37\x55\x02\x2f\x0a\x7d"),
  .aad = TEST_DATA_STR ("\x68\x1d\x65\x83\xf5\x27\xb1\xa9\x2f\x66\xca\xae\x9b"
			"\x1d\x4d\x02\x8e\x2e\x63\x1e"),
  .tag = TEST_DATA_STR (
    "\xb3\x04\x42\xa6\x39\x5e\xc1\x32\x46\xc4\x8b\x21\xff\xc6\x55\x09"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad160_tc12) = {
  .name = "256-GMAC 160-aad NIST CAVS TC12",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x87\x66\x0e\xc1\x70\x0d\x4e\x9f\x88\xa3\x23\xa4\x9f\x0b\x87\x1e\x6a\xaf"
    "\x43\x4a\x2d\x84\x48\xd0\x4d\x4a\x22\xf6\x56\x10\x28\xe0"),
  .iv = TEST_DATA_STR ("\x2a\x07\xb4\x25\x93\xcd\x24\xf0\xa6\xfe\x40\x6c"),
  .aad = TEST_DATA_STR ("\x1d\xd2\x39\xb5\x71\x85\xb7\xe4\x57\xce\xd7\x3e\xbb"
			"\xa0\x43\x05\x7f\x04\x9e\xdd"),
  .tag = TEST_DATA_STR (
    "\xdf\x7a\x50\x10\x49\xb3\x7a\x53\x40\x98\xcb\x45\xcb\x9c\x21\xb7"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad160_tc13) = {
  .name = "256-GMAC 160-aad NIST CAVS TC13",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xea\x47\x92\xe1\xf1\x71\x7b\x77\xa0\x0d\xe4\xd1\x09\xe6\x27\x54\x9b\x16"
    "\x5c\x82\xaf\x35\xf3\x3c\xa7\xe1\xa6\xb8\xed\x62\xf1\x4f"),
  .iv = TEST_DATA_STR ("\x74\x53\xcc\x8b\x46\xfe\x4b\x93\xbc\xc4\x83\x81"),
  .aad = TEST_DATA_STR ("\x46\xd9\x89\x70\xa6\x36\xe7\xcd\x7b\x76\xfc\x36\x2a"
			"\xe8\x82\x98\x43\x6f\x83\x4f"),
  .tag = TEST_DATA_STR (
    "\x51\x8d\xba\xcd\x36\xbe\x6f\xba\x5c\x12\x87\x16\x78\xa5\x55\x16"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad160_tc14) = {
  .name = "256-GMAC 160-aad NIST CAVS TC14",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x34\x89\x2c\xdd\x1d\x48\xca\x16\x6f\x7b\xa7\x31\x82\xcb\x97\x33\x6c\x2c"
    "\x75\x4a\xc1\x60\xa3\xe3\x71\x83\xd6\xfb\x50\x78\xce\xc3"),
  .iv = TEST_DATA_STR ("\xed\x31\x98\xc5\x86\x1b\x78\xc7\x1a\x6a\x4e\xec"),
  .aad = TEST_DATA_STR ("\xa6\xfa\x6d\x0d\xd1\xe0\xb9\x5b\x46\x09\x95\x1b\xbb"
			"\xe7\x14\xde\x0a\xe0\xcc\xfa"),
  .tag = TEST_DATA_STR (
    "\xc6\x38\x77\x95\x09\x6b\x34\x8e\xcf\x1d\x1f\x6c\xaa\xa3\xc8\x13"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad384_tc0) = {
  .name = "256-GMAC 384-aad NIST CAVS TC0",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xf4\x06\x9b\xb7\x39\xd0\x7d\x0c\xaf\xdc\xbc\x60\x9c\xa0\x15\x97\xf9\x85"
    "\xc4\x3d\xb6\x3b\xba\xaa\x0d\xeb\xbb\x04\xd3\x84\xe4\x9c"),
  .iv = TEST_DATA_STR ("\xd2\x5f\xf3\x0f\xdc\x3d\x46\x4f\xe1\x73\xe8\x05"),
  .aad = TEST_DATA_STR (
    "\x3e\x14\x49\xc4\x83\x7f\x08\x92\xf9\xd5\x51\x27\xc7\x5c\x4b\x25\xd6\x9b"
    "\xe3\x34\xba\xf5\xf1\x93\x94\xd2\xd8\xbb\x46\x0c\xbf\x21\x20\xe1\x47\x36"
    "\xd0\xf6\x34\xaa\x79\x2f\xec\xa2\x0e\x45\x5f\x11"),
  .tag = TEST_DATA_STR (
    "\x80\x5e\xc2\x93\x1c\x21\x81\xe5\xbf\xb7\x4f\xa0\xa9\x75\xf0\xcf"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad384_tc1) = {
  .name = "256-GMAC 384-aad NIST CAVS TC1",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x62\x18\x9d\xcc\x4b\xeb\x97\x46\x2d\x6c\x09\x27\xd8\xa2\x70\xd3\x9a\x1b"
    "\x07\xd7\x2d\x0a\xd2\x88\x40\xba\xdd\x4f\x68\xcf\x9c\x8b"),
  .iv = TEST_DATA_STR ("\x85\x9f\xda\x52\x47\xc8\x88\x82\x3a\x4b\x80\x32"),
  .aad = TEST_DATA_STR (
    "\xb2\x8d\x16\x21\xee\x11\x0f\x4c\x9d\x70\x9f\xad\x76\x4b\xba\x2d\xd6\xd2"
    "\x91\xbc\x00\x37\x48\xfa\xac\x6d\x90\x19\x37\x12\x0d\x41\xc1\xb7\xce\x67"
    "\x63\x37\x63\xe9\x9e\x05\xc7\x13\x63\xfc\xec\xa8"),
  .tag = TEST_DATA_STR (
    "\x27\x33\x09\x07\xd0\x00\x28\x80\xbb\xb4\xc1\xa1\xd2\x3c\x0b\xe2"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad384_tc2) = {
  .name = "256-GMAC 384-aad NIST CAVS TC2",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x59\x01\x2d\x85\xa1\xb9\x0a\xeb\x03\x59\xe6\x38\x4c\x99\x91\xe7\xbe\x21"
    "\x93\x19\xf5\xb8\x91\xc9\x2c\x38\x4a\xde\x2f\x37\x18\x16"),
  .iv = TEST_DATA_STR ("\x3c\x9c\xde\x00\xc2\x39\x12\xcf\xf9\x68\x9c\x7c"),
  .aad = TEST_DATA_STR (
    "\xe5\xda\xf4\x73\xa4\x70\x86\x0b\x55\x21\x0a\x48\x3c\x0d\x1a\x97\x8d\x8a"
    "\xdd\x84\x3c\x2c\x09\x7f\x73\xa3\xcd\xa4\x9a\xc4\xa6\x14\xc8\xe8\x87\xd9"
    "\x4e\x66\x92\x30\x9d\x2e\xd9\x7e\xbe\x1e\xaf\x5d"),
  .tag = TEST_DATA_STR (
    "\x04\x82\x39\xe4\xe5\xc2\xc8\xb3\x38\x90\xa7\xc9\x50\xcd\xa8\x52"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad384_tc3) = {
  .name = "256-GMAC 384-aad NIST CAVS TC3",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x4b\xe0\x9b\x40\x8a\xd6\x8b\x89\x0f\x94\xbe\x5e\xfa\x7f\xe9\xc9\x17\x36"
    "\x27\x12\xa3\x48\x0c\x57\xcd\x38\x44\x93\x5f\x35\xac\xb7"),
  .iv = TEST_DATA_STR ("\x8f\x35\x0b\xd3\xb8\xee\xa1\x73\xfc\x73\x70\xbc"),
  .aad = TEST_DATA_STR (
    "\x28\x19\xd6\x5a\xec\x94\x21\x98\xca\x97\xd4\x43\x5e\xfd\x9d\xd4\xd4\x39"
    "\x3b\x96\xcf\x5b\xa4\x4f\x09\xbc\xe4\xba\x13\x5f\xc8\x63\x6e\x82\x75\xdc"
    "\xb5\x15\x41\x4b\x8b\xef\xd3\x2f\x91\xfc\x48\x22"),
  .tag = TEST_DATA_STR (
    "\xa1\x33\xcb\x7a\x7d\x04\x71\xdb\xac\x61\xfb\x41\x58\x9a\x2e\xfe"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad384_tc4) = {
  .name = "256-GMAC 384-aad NIST CAVS TC4",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x13\xcb\x96\x5a\x4d\x9d\x1a\x36\xef\xad\x9f\x6c\xa1\xba\x76\x38\x6a\x5b"
    "\xb1\x60\xd8\x0b\x09\x17\x27\x71\x02\x35\x7a\xc7\xaf\xc8"),
  .iv = TEST_DATA_STR ("\xf3\x13\xad\xec\x42\xa6\x6d\x13\xc3\x95\x81\x80"),
  .aad = TEST_DATA_STR (
    "\x71\x7b\x48\x35\x88\x98\xe5\xcc\xfe\xa4\x28\x90\x49\xad\xcc\x1b\xb0\xdb"
    "\x3b\x3e\xbd\x17\x67\xac\x24\xfb\x2b\x7d\x37\xdc\x80\xea\x23\x16\xc1\x7f"
    "\x14\xfb\x51\xb5\xe1\x8c\xd5\xbb\x09\xaf\xe4\x14"),
  .tag = TEST_DATA_STR (
    "\x81\xb4\xef\x7a\x84\xdc\x4a\x0b\x1f\xdd\xbe\xfe\x37\xf5\x38\x52"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad384_tc5) = {
  .name = "256-GMAC 384-aad NIST CAVS TC5",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xd2\x7f\x1b\xeb\xbb\xde\xf0\xed\xca\x39\x3a\x62\x61\xb0\x33\x8a\xbb\xc4"
    "\x91\x26\x2e\xab\x07\x37\xf5\x52\x46\x45\x8f\x66\x68\xcc"),
  .iv = TEST_DATA_STR ("\xfc\x06\x2f\x85\x78\x86\xe2\x78\xf3\xa5\x67\xd2"),
  .aad = TEST_DATA_STR (
    "\x2b\xae\x92\xde\xa6\x4a\xa9\x91\x89\xde\x8e\xa4\xc0\x46\x74\x53\x06\x00"
    "\x2e\x02\xcf\xb4\x6a\x41\x44\x4c\xe8\xbf\xcc\x32\x9b\xd4\x20\x59\x63\xd9"
    "\xab\x53\x57\xb0\x26\xa4\xa3\x4b\x1a\x86\x17\x71"),
  .tag = TEST_DATA_STR (
    "\x5c\x5a\x6c\x46\x13\xf1\xe5\x22\x59\x63\x30\xd4\x5f\x24\x3f\xdd"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad384_tc6) = {
  .name = "256-GMAC 384-aad NIST CAVS TC6",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x7b\x4d\x19\xcd\x35\x69\xf7\x4c\x7b\x5d\xf6\x1a\xb7\x83\x79\xee\x6b\xfa"
    "\x15\x10\x5d\x21\xb1\x0b\xf6\x09\x66\x99\x53\x90\x06\xd0"),
  .iv = TEST_DATA_STR ("\xfb\xed\x56\x95\xc4\xa7\x39\xed\xed\x97\xb1\xe3"),
  .aad = TEST_DATA_STR (
    "\xc6\xf2\xe5\xd6\x63\xbf\xaf\x66\x8d\x01\x45\x50\xef\x2e\x66\xbf\x89\x97"
    "\x87\x99\xa7\x85\xf1\xf2\xc7\x9a\x2c\xb3\xeb\x3f\x2f\xd4\x07\x62\x07\xd5"
    "\xf7\xe1\xc2\x84\xb4\xaf\x5c\xff\xc4\xe4\x61\x98"),
  .tag = TEST_DATA_STR (
    "\x71\x01\xb4\x34\xfb\x90\xc7\xf9\x5b\x9b\x7a\x0d\xee\xeb\x5c\x81"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad384_tc7) = {
  .name = "256-GMAC 384-aad NIST CAVS TC7",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xd3\x43\x14\x88\xd8\xf0\x48\x59\x0b\xd7\x6e\xc6\x6e\x71\x42\x1e\xf0\x9f"
    "\x65\x5d\x7c\xf8\x04\x3b\xf3\x2f\x75\xb4\xb2\xe7\xef\xcc"),
  .iv = TEST_DATA_STR ("\xcc\x76\x6e\x98\xb4\x0a\x81\x51\x9f\xa4\x63\x92"),
  .aad = TEST_DATA_STR (
    "\x93\x32\x01\x79\xfd\xb4\x0c\xbc\x1c\xcf\x00\xb8\x72\xa3\xb4\xa5\xf6\xc7"
    "\x0b\x56\xe4\x3a\x84\xfc\xac\x5e\xb4\x54\xa0\xa1\x9a\x74\x7d\x45\x20\x42"
    "\x61\x1b\xf3\xbb\xaa\xfd\x92\x5e\x80\x6f\xfe\x8e"),
  .tag = TEST_DATA_STR (
    "\x3a\xfc\xc3\x36\xce\x8b\x71\x91\xea\xb0\x4a\xd6\x79\x16\x3c\x2a"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad384_tc8) = {
  .name = "256-GMAC 384-aad NIST CAVS TC8",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xa4\x40\x94\x8c\x03\x78\x56\x1c\x39\x56\x81\x3c\x03\x1f\x81\x57\x32\x08"
    "\xc7\xff\xa8\x15\x11\x4e\xf2\xee\xe1\xeb\x64\x2e\x74\xc6"),
  .iv = TEST_DATA_STR ("\xc1\xf4\xff\xe5\x4b\x86\x80\x83\x2e\xed\x88\x19"),
  .aad = TEST_DATA_STR (
    "\x25\x34\x38\xf1\x32\xb1\x8e\x84\x83\x07\x45\x61\x89\x8c\x56\x52\xb4\x3a"
    "\x82\xcc\x94\x1e\x8b\x4a\xe3\x7e\x79\x2a\x8e\xd6\xec\x5c\xe2\xbc\xec\x9f"
    "\x1f\xfc\xf4\x21\x6e\x46\x69\x63\x07\xbb\x77\x4a"),
  .tag = TEST_DATA_STR (
    "\x12\x94\x45\xf0\xa3\xc9\x79\xa1\x12\xa3\xaf\xb1\x0a\x24\xe2\x45"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad384_tc9) = {
  .name = "256-GMAC 384-aad NIST CAVS TC9",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x79\x87\x06\xb6\x51\x03\x3d\x9e\x9b\xf2\xce\x06\x4f\xb1\x2b\xe7\xdf\x73"
    "\x08\xcf\x45\xdf\x44\x77\x65\x88\xcd\x39\x1c\x49\xff\x85"),
  .iv = TEST_DATA_STR ("\x5a\x43\x36\x8a\x39\xe7\xff\xb7\x75\xed\xfa\xf4"),
  .aad = TEST_DATA_STR (
    "\x92\x6b\x74\xfe\x63\x81\xeb\xd3\x57\x57\xe4\x2e\x8e\x55\x76\x01\xf2\x28"
    "\x7b\xfc\x13\x3a\x13\xfd\x86\xd6\x1c\x01\xaa\x84\xf3\x97\x13\xbf\x99\xa8"
    "\xdc\x07\xb8\x12\xf0\x27\x4c\x9d\x32\x80\xa1\x38"),
  .tag = TEST_DATA_STR (
    "\x89\xfe\x48\x1a\x3d\x95\xc0\x3a\x0a\x9d\x4e\xe3\xe3\xf0\xed\x4a"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad384_tc10) = {
  .name = "256-GMAC 384-aad NIST CAVS TC10",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xc3\xaa\x2a\x39\xa9\xfe\xf4\xa4\x66\x61\x8d\x12\x88\xbb\x62\xf8\xda\x7b"
    "\x1c\xb7\x60\xcc\xc8\xf1\xbe\x3e\x99\xe0\x76\xf0\x8e\xff"),
  .iv = TEST_DATA_STR ("\x99\x65\xba\x5e\x23\xd9\x45\x3d\x72\x67\xca\x5b"),
  .aad = TEST_DATA_STR (
    "\x93\xef\xb6\xa2\xaf\xfc\x30\x4c\xb2\x5d\xfd\x49\xaa\x3e\x3c\xcd\xb2\x5c"
    "\xea\xc3\xd3\xce\xa9\x0d\xd9\x9e\x38\x97\x69\x78\x21\x7a\xd5\xf2\xb9\x90"
    "\xd1\x0b\x91\x72\x5c\x7f\xd2\x03\x5e\xcc\x6a\x30"),
  .tag = TEST_DATA_STR (
    "\x00\xa9\x4c\x18\xa4\x57\x2d\xcf\x4f\x9e\x22\x26\xa0\x3d\x4c\x07"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad384_tc11) = {
  .name = "256-GMAC 384-aad NIST CAVS TC11",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x14\xe0\x68\x58\x00\x8f\x7e\x77\x18\x6a\x2b\x3a\x79\x28\xa0\xc7\xfc\xee"
    "\x22\x13\x6b\xc3\x6f\x53\x55\x3f\x20\xfa\x5c\x37\xed\xcd"),
  .iv = TEST_DATA_STR ("\x32\xeb\xe0\xdc\x9a\xda\x84\x9b\x5e\xda\x7b\x48"),
  .aad = TEST_DATA_STR (
    "\x6c\x01\x52\xab\xfa\x48\x5b\x8c\xd6\x7c\x15\x4a\x5f\x04\x11\xf2\x21\x21"
    "\x37\x97\x74\xd7\x45\xf4\x0e\xe5\x77\xb0\x28\xfd\x0e\x18\x82\x97\x58\x15"
    "\x61\xae\x97\x22\x23\xd7\x5a\x24\xb4\x88\xae\xd7"),
  .tag = TEST_DATA_STR (
    "\x26\x25\xb0\xba\x6e\xe0\x2b\x58\xbc\x52\x9e\x43\xe2\xeb\x47\x1b"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad384_tc12) = {
  .name = "256-GMAC 384-aad NIST CAVS TC12",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xfb\xb5\x6b\x11\xc5\x1a\x09\x3c\xe1\x69\xa6\x99\x03\x99\xc4\xd7\x41\xf6"
    "\x2b\x3c\xc6\x1f\x9e\x8a\x60\x9a\x1b\x6a\xe8\xe7\xe9\x65"),
  .iv = TEST_DATA_STR ("\x9c\x5a\x95\x32\x47\xe9\x1a\xce\xce\xb9\xde\xfb"),
  .aad = TEST_DATA_STR (
    "\x46\xcb\x5c\x4f\x61\x79\x16\xa9\xb1\xb2\xe0\x32\x72\xcb\x05\x90\xce\x71"
    "\x64\x98\x53\x30\x47\xd7\x3c\x81\xe4\xcb\xe9\x27\x8a\x36\x86\x11\x6f\x56"
    "\x32\x75\x3e\xa2\xdf\x52\xef\xb3\x55\x1a\xea\x2d"),
  .tag = TEST_DATA_STR (
    "\x4f\x3b\x82\xe6\xbe\x4f\x08\x75\x60\x71\xf2\xc4\x6c\x31\xfe\xdf"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad384_tc13) = {
  .name = "256-GMAC 384-aad NIST CAVS TC13",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xb3\x03\xbf\x02\xf6\xa8\xdb\xb5\xbc\x4b\xac\xca\xb0\x80\x0d\xb5\xee\x06"
    "\xde\x64\x8e\x2f\xae\x29\x9b\x95\xf1\x35\xc9\xb1\x07\xcc"),
  .iv = TEST_DATA_STR ("\x90\x64\x95\xb6\x7e\xf4\xce\x00\xb4\x44\x22\xfa"),
  .aad = TEST_DATA_STR (
    "\x87\x2c\x6c\x37\x09\x26\x53\x5c\x3f\xa1\xba\xec\x03\x1e\x31\xe7\xc6\xc8"
    "\x28\x08\xc8\xa0\x60\x74\x2d\xbe\xf1\x14\x96\x1c\x31\x4f\x19\x86\xb2\x13"
    "\x1a\x9d\x91\xf3\x0f\x53\x06\x7e\xc0\x12\xc6\xb7"),
  .tag = TEST_DATA_STR (
    "\x64\xdd\xe3\x71\x69\x08\x2d\x18\x1a\x69\x10\x7f\x60\xc5\xc6\xbb"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad384_tc14) = {
  .name = "256-GMAC 384-aad NIST CAVS TC14",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x29\xf5\xf8\x07\x59\x03\x06\x3c\xb6\xd7\x05\x06\x69\xb1\xf7\x4e\x08\xa3"
    "\xf7\x9e\xf5\x66\x29\x2d\xfd\xef\x1c\x06\xa4\x08\xe1\xab"),
  .iv = TEST_DATA_STR ("\x35\xf2\x5c\x48\xb4\xb5\x35\x5e\x78\xb9\xfb\x3a"),
  .aad = TEST_DATA_STR (
    "\x10\x7e\x2e\x23\x15\x9f\xc5\xc0\x74\x8c\xa7\xa0\x77\xe5\xcc\x05\x3f\xa5"
    "\xc6\x82\xff\x52\x69\xd3\x50\xee\x81\x7f\x8b\x5d\xe4\xd3\x97\x20\x41\xd1"
    "\x07\xb1\xe2\xf2\xe5\x4c\xa9\x3b\x72\xcd\x04\x08"),
  .tag = TEST_DATA_STR (
    "\xfe\xe5\xa9\xba\xeb\xb5\xbe\x01\x65\xde\xaa\x86\x7e\x96\x7a\x9e"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad720_tc0) = {
  .name = "256-GMAC 720-aad NIST CAVS TC0",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x03\xcc\xb7\xdb\xc7\xb8\x42\x54\x65\xc2\xc3\xfc\x39\xed\x05\x93\x92\x9f"
    "\xfd\x02\xa4\x5f\xf5\x83\xbd\x89\xb7\x9c\x6f\x64\x6f\xe9"),
  .iv = TEST_DATA_STR ("\xfd\x11\x99\x85\x53\x3b\xd5\x52\x0b\x30\x1d\x12"),
  .aad = TEST_DATA_STR (
    "\x98\xe6\x8c\x10\xbf\x4b\x5a\xe6\x2d\x43\x49\x28\xfc\x64\x05\x14\x7c\x63"
    "\x01\x41\x73\x03\xef\x3a\x70\x3d\xcf\xd2\xc0\xc3\x39\xa4\xd0\xa8\x9b\xd2"
    "\x9f\xe6\x1f\xec\xf1\x06\x6a\xb0\x6d\x7a\x5c\x31\xa4\x8f\xfb\xfe\xd2\x2f"
    "\x74\x9b\x17\xe9\xbd\x0d\xc1\xc6\xf8\xfb\xd6\xfd\x45\x87\x18\x4d\xb9\x64"
    "\xd5\x45\x61\x32\x10\x6d\x78\x23\x38\xc3\xf1\x17\xec\x05\x22\x9b\x08"
    "\x99"),
  .tag = TEST_DATA_STR (
    "\xcf\x54\xe7\x14\x13\x49\xb6\x6f\x24\x81\x54\x42\x78\x10\xc8\x7a"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad720_tc1) = {
  .name = "256-GMAC 720-aad NIST CAVS TC1",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x57\xe1\x12\xcd\x45\xf2\xc5\x7d\xdb\x81\x9e\xa6\x51\xc2\x06\x76\x31\x63"
    "\xef\x01\x6c\xee\xad\x5c\x4e\xae\x40\xf2\xbb\xe0\xe4\xb4"),
  .iv = TEST_DATA_STR ("\x18\x80\x22\xc2\x12\x5d\x2b\x1f\xcf\x9e\x47\x69"),
  .aad = TEST_DATA_STR (
    "\x09\xc8\xf4\x45\xce\x5b\x71\x46\x56\x95\xf8\x38\xc4\xbb\x2b\x00\x62\x4a"
    "\x1c\x91\x85\xa3\xd5\x52\x54\x6d\x9d\x2e\xe4\x87\x00\x07\xaa\xf3\x00\x70"
    "\x08\xf8\xae\x9a\xff\xb7\x58\x8b\x88\xd0\x9a\x90\xe5\x8b\x45\x7f\x88\xf1"
    "\xe3\x75\x2e\x3f\xb9\x49\xce\x37\x86\x70\xb6\x7a\x95\xf8\xcf\x7f\x5c\x7c"
    "\xeb\x65\x0e\xfd\x73\x5d\xbc\x65\x2c\xae\x06\xe5\x46\xa5\xdb\xd8\x61"
    "\xbd"),
  .tag = TEST_DATA_STR (
    "\x9e\xfc\xdd\xfa\x0b\xe2\x15\x82\xa0\x57\x49\xf4\x05\x0d\x29\xfe"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad720_tc2) = {
  .name = "256-GMAC 720-aad NIST CAVS TC2",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xa4\xdd\xf3\xca\xb7\x45\x3a\xae\xfa\xd6\x16\xfd\x65\xd6\x3d\x13\x00\x5e"
    "\x94\x59\xc1\x7d\x31\x73\xcd\x6e\xd7\xf2\xa8\x6c\x92\x1f"),
  .iv = TEST_DATA_STR ("\x06\x17\x7b\x24\xc5\x8f\x3b\xe4\xf3\xdd\x49\x20"),
  .aad = TEST_DATA_STR (
    "\xf9\x5b\x04\x6d\x80\x48\x5e\x41\x1c\x56\xb8\x34\x20\x9d\x3a\xbd\x5a\x8a"
    "\x9d\xdf\x72\xb1\xb9\x16\x67\x9a\xdf\xdd\xe8\x93\x04\x43\x15\xa5\xf4\x96"
    "\x7f\xd0\x40\x5e\xc2\x97\xaa\x33\x2f\x67\x6f\xf0\xfa\x5b\xd7\x95\xeb\x60"
    "\x9b\x2e\x4f\x08\x8d\xb1\xcd\xf3\x7c\xcf\xf0\x73\x5a\x5e\x53\xc4\xc1\x21"
    "\x73\xa0\x02\x6a\xea\x42\x38\x8a\x7d\x71\x53\xa8\x83\x0b\x8a\x90\x1c"
    "\xf9"),
  .tag = TEST_DATA_STR (
    "\x9d\x1b\xd8\xec\xb3\x27\x69\x06\x13\x8d\x0b\x03\xfc\xb8\xc1\xbb"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad720_tc3) = {
  .name = "256-GMAC 720-aad NIST CAVS TC3",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x24\xa9\x2b\x24\xe8\x59\x03\xcd\x4a\xaa\xbf\xe0\x7c\x31\x0d\xf5\xa4\xf8"
    "\xf4\x59\xe0\x3a\x63\xcb\xd1\xb4\x78\x55\xb0\x9c\x0b\xe8"),
  .iv = TEST_DATA_STR ("\x22\xe7\x56\xdc\x89\x8d\x4c\xf1\x22\x08\x06\x12"),
  .aad = TEST_DATA_STR (
    "\x2e\x01\xb2\x53\x6d\xbe\x37\x6b\xe1\x44\x29\x6f\x5c\x38\xfb\x09\x9e\x00"
    "\x8f\x96\x2b\x9f\x0e\x89\x63\x34\xb6\x40\x83\x93\xbf\xf1\x02\x0a\x0e\x44"
    "\x24\x77\xab\xfd\xb1\x72\x72\x13\xb6\xcc\xc5\x77\xf5\xe1\x6c\xb0\x57\xc8"
    "\x94\x5a\x07\xe3\x07\x26\x4b\x65\x97\x9a\xed\x96\xb5\x99\x5f\x40\x25\x0f"
    "\xfb\xaa\xa1\xa1\xf0\xec\xcf\x39\x40\x15\xf6\x29\x0f\x5e\x64\xdf\xe5"
    "\xca"),
  .tag = TEST_DATA_STR (
    "\x0d\x7f\x1a\xed\x47\x08\xa0\x3b\x0c\x80\xb2\xa1\x87\x85\xc9\x6d"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad720_tc4) = {
  .name = "256-GMAC 720-aad NIST CAVS TC4",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x15\x27\x6f\xc6\x44\x38\x57\x8e\x0e\xc5\x33\x66\xb9\x0a\x0e\x23\xd9\x39"
    "\x10\xfe\xc1\x0d\xc3\x00\x3d\x9b\x3f\x3f\xa7\x2d\xb7\x02"),
  .iv = TEST_DATA_STR ("\xc5\xe9\x31\x94\x6d\x5c\xae\xbc\x22\x76\x56\xd2"),
  .aad = TEST_DATA_STR (
    "\x3f\x96\x7c\x83\xba\x02\xe7\x7c\x14\xe9\xd4\x11\x85\xeb\x87\xf1\x72\x25"
    "\x0e\x93\xed\xb0\xf8\x2b\x67\x42\xc1\x24\x29\x8a\xb6\x94\x18\x35\x8e\xdd"
    "\xef\xa3\x9f\xed\xc3\xca\xde\x9d\x80\xf0\x36\xd8\x64\xa5\x9e\xad\x37\xc8"
    "\x77\x27\xc5\x6c\x70\x1a\x8c\xd9\x63\x44\x69\xff\x31\xc7\x04\xf5\xee\x39"
    "\x35\x41\x57\xe6\x55\x84\x67\xb9\x28\x24\xda\x36\xb1\xc0\x71\xbe\xdf"
    "\xe9"),
  .tag = TEST_DATA_STR (
    "\xa0\xff\xa1\x9a\xdc\xf3\x1d\x06\x1c\xd0\xdd\x46\xd2\x40\x15\xef"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad720_tc5) = {
  .name = "256-GMAC 720-aad NIST CAVS TC5",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xec\x09\x80\x4a\x04\x8b\xb8\x54\xc7\x16\x18\xb5\xa3\xa1\xc5\x90\x91\x0f"
    "\xc8\xa6\x84\x55\x13\x9b\x71\x94\x86\xd2\x28\x0e\xa5\x9a"),
  .iv = TEST_DATA_STR ("\xd0\xb1\x24\x7e\x71\x21\xa9\x27\x6a\xc1\x8c\xa3"),
  .aad = TEST_DATA_STR (
    "\x66\xb1\xd3\x9d\x41\x45\x96\x30\x8e\x86\x6b\x04\x47\x6e\x05\x3b\x71\xac"
    "\xd1\xcd\x07\xce\x80\x93\x95\x77\xeb\xbe\xac\xe0\x43\x0f\x7e\x4c\x0c\x18"
    "\x5f\xe1\xd9\x7a\xc7\x56\x99\x50\xc8\x3d\xb4\x0b\xbe\xd0\xf1\xd1\x73\xe1"
    "\xaa\x0d\xc2\x8b\x47\x73\x70\x50\x32\xd9\x75\x51\xf7\xfc\xef\x7f\x55\xe4"
    "\xb6\x9f\x88\xdf\x65\x00\x32\xdf\xc5\x23\x2c\x15\x66\x41\x10\x4b\x53"
    "\x97"),
  .tag = TEST_DATA_STR (
    "\x84\x40\xe6\xd8\x64\xab\x77\x8f\x9b\xe4\x78\xf2\x03\x16\x2d\x86"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad720_tc6) = {
  .name = "256-GMAC 720-aad NIST CAVS TC6",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x4a\xdf\x86\xbf\xa5\x47\x72\x5e\x4b\x80\x36\x5a\x5a\x32\x7c\x10\x70\x40"
    "\xfa\xcf\xff\x00\x7d\xc3\x51\x02\x06\x6b\xd6\xa9\x95\xc4"),
  .iv = TEST_DATA_STR ("\xb1\x01\x8c\xc3\x31\x91\x12\x55\xa5\x5a\x07\x95"),
  .aad = TEST_DATA_STR (
    "\x05\x3c\xa4\x42\x8c\x99\x0b\x44\x56\xd3\xc1\x89\x5d\x5d\x52\xde\xff\x67"
    "\x58\x96\xde\x9f\xaa\x53\xd8\xcf\x24\x12\x55\xf4\xa3\x1d\xc3\x39\x9f\x15"
    "\xd8\x3b\xe3\x80\x25\x66\x16\xe5\xaf\x04\x3a\xbf\xb3\x75\x52\x65\x5a\xdf"
    "\x4f\x2e\x68\xdd\xa2\x4b\xc3\x73\x69\x51\x13\x4f\x35\x9d\x9c\x0e\x28\x8b"
    "\xb7\x98\xb6\xc3\xea\x46\x23\x92\x31\xa3\xcb\x28\x00\x66\xdb\x98\x62"
    "\xe7"),
  .tag = TEST_DATA_STR (
    "\xc7\x42\x4f\x38\x08\x49\x30\xbf\xc5\xed\xc1\xfc\xf1\xe7\x60\x8d"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad720_tc7) = {
  .name = "256-GMAC 720-aad NIST CAVS TC7",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x3c\x92\xe0\xd1\xe3\x9a\x3c\x76\x65\x73\xc4\x64\x6c\x76\x8c\x40\x2c\xcf"
    "\xf4\x8a\x56\x68\x2a\x93\x43\x35\x12\xab\xf0\x45\x6e\x00"),
  .iv = TEST_DATA_STR ("\xd5\x7f\x31\x9e\x59\x01\x91\x84\x1d\x2b\x98\xbd"),
  .aad = TEST_DATA_STR (
    "\x84\x0d\x93\x94\xaa\x24\x0e\x52\xba\x15\x21\x51\xc1\x2a\xcd\x1c\xd4\x48"
    "\x81\xe8\x54\x9d\xc8\x32\xb7\x1a\x45\xda\x7e\xfc\xc7\x4f\xb7\xe8\x44\xd9"
    "\xfe\xc2\x5e\x5d\x49\x7b\x8f\xb8\xf4\x7f\x32\x8c\x8d\x99\x04\x5a\x19\xe3"
    "\x66\xe6\xce\x5e\x19\xdc\x26\xf6\x7a\x81\xa9\x4f\xa6\xc9\x7c\x31\x4d\x88"
    "\x6e\x7b\x56\xef\xf1\x44\xc0\x9f\x6f\xa5\x19\xdb\x63\x08\xbc\x73\x42"
    "\x2e"),
  .tag = TEST_DATA_STR (
    "\xcb\x4e\xf7\x2d\xbd\xa4\x91\x4d\x74\x34\xf9\x68\x6f\x82\x3e\x2f"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad720_tc8) = {
  .name = "256-GMAC 720-aad NIST CAVS TC8",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xb6\x6b\xa3\x97\x33\x88\x8a\x9e\x0a\x2e\x30\x45\x28\x44\x16\x1d\xc3\x3c"
    "\xb3\x83\xc0\x2c\xe1\x6c\x4e\xfa\xd5\x45\x25\x09\xb5\xb5"),
  .iv = TEST_DATA_STR ("\x93\x7c\xb6\x65\xe3\x70\x59\xb2\xe4\x03\x59\xf2"),
  .aad = TEST_DATA_STR (
    "\xdb\xcd\x96\x94\xa8\x83\x48\x60\x03\x4e\x8e\xde\x3a\x5b\xd4\x19\xfc\xf9"
    "\x1c\x00\x5a\xd9\x9f\x48\x8a\xa6\x23\xf5\x81\x62\x20\x93\xf9\xd4\x1e\x6a"
    "\x68\xe2\x0f\xd2\x02\xf3\x02\xbc\xfc\x44\x17\xca\x89\x09\x0b\xfc\xd4\xd5"
    "\x22\x4e\x8f\xf4\xeb\x5b\xba\xe4\xec\xb2\x7b\xaa\x23\x9f\x59\xc2\xf9\x9c"
    "\xd4\x7c\x0a\x26\x9c\x49\x79\x06\xb4\x1a\x8f\x32\x0a\x3d\xd2\xdc\x2d"
    "\xe2"),
  .tag = TEST_DATA_STR (
    "\xbd\xc8\x24\x93\x02\xd9\xd6\x66\xcf\x71\x68\x31\x7c\x11\x87\x43"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad720_tc9) = {
  .name = "256-GMAC 720-aad NIST CAVS TC9",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x2f\x9f\xcd\x10\x43\x45\x56\x95\x63\x8c\x99\x1a\x1b\x1d\x35\xad\x57\xc1"
    "\x8e\xf0\x72\x73\x22\x74\x7b\x79\x91\xab\xc3\xd7\x87\xf3"),
  .iv = TEST_DATA_STR ("\xd0\x6c\xf5\x48\xf6\x28\x69\xf4\xbe\xd7\xa3\x18"),
  .aad = TEST_DATA_STR (
    "\x43\x20\x23\xc1\x2c\xf1\xf6\x14\xe1\x00\x51\x12\xa1\x7d\xbe\x6c\x5d\x54"
    "\x02\x2a\x95\xcf\x63\x35\xa5\xbc\x55\x00\x4c\x75\xf0\x9a\x56\x99\x73\x9e"
    "\xcf\x92\x8e\x1c\x78\xd0\x3d\xad\x50\x96\xa1\x7a\x08\x4a\xfe\x1c\xc2\x20"
    "\x41\xbb\xdf\xb5\x98\x5b\xd0\x8b\x0d\xcc\x59\xd2\xb0\x8c\xd8\x6b\x7a\xad"
    "\x59\x7c\x4c\xd7\xb4\xba\x6d\x6a\x73\x70\xb8\x39\x95\xa6\x51\x1a\x1f"
    "\x9e"),
  .tag = TEST_DATA_STR (
    "\x32\x2e\xb8\x4f\xb6\x88\x4f\x10\xcf\xb7\x66\xc2\xe3\xec\x77\x9e"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad720_tc10) = {
  .name = "256-GMAC 720-aad NIST CAVS TC10",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x21\xc5\x83\x9a\x63\xe1\x23\x0c\x06\xb0\x86\x34\x1c\x96\xab\x74\x58\x5e"
    "\x69\xbc\xed\x94\x33\x2c\xae\xb1\xfa\x77\xd5\x10\xc2\x4f"),
  .iv = TEST_DATA_STR ("\x5a\xb6\xe5\xed\x6e\xe7\x33\xbe\x72\x50\x85\x8c"),
  .aad = TEST_DATA_STR (
    "\xc9\x2f\x08\xe3\x0f\x67\xd4\x25\x16\x13\x3c\x48\xe9\x7b\x65\xcc\x9e\x12"
    "\x43\x65\xe1\x10\xab\xa5\xe7\xb2\xcb\xe8\x3d\xeb\xcc\x99\xed\xf4\xeb\x00"
    "\x07\xaf\x05\x2b\xda\x22\xd8\x59\x00\x27\x1b\x18\x97\xaf\x4f\xd9\xac\xe6"
    "\xa2\xd0\x9d\x98\x4a\xc3\xde\x79\xd0\x5d\xe0\xb1\x05\xa8\x1b\x12\x54\x2b"
    "\x2c\x48\xe2\x7d\x40\x9f\xd6\x99\x2d\xd0\x62\xd6\x05\x5d\x6f\xc6\x68"
    "\x42"),
  .tag = TEST_DATA_STR (
    "\x53\xb0\xe4\x50\x30\x9d\x14\x64\x59\xf2\xa1\xe4\x6c\x9d\x9e\x23"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad720_tc11) = {
  .name = "256-GMAC 720-aad NIST CAVS TC11",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x25\xa1\x44\xf0\xfd\xba\x18\x41\x25\xd8\x1a\x87\xe7\xed\x82\xfa\xd3\x3c"
    "\x70\x1a\x09\x4a\x67\xa8\x1f\xe4\x69\x2d\xc6\x9a\xfa\x31"),
  .iv = TEST_DATA_STR ("\x8b\xf5\x75\xc5\xc2\xb4\x5b\x4e\xfc\x67\x46\xe4"),
  .aad = TEST_DATA_STR (
    "\x2a\x36\x7c\xb0\xd3\xb7\xc5\xb8\x32\x0b\x3c\xf9\x5e\x82\xb6\xba\x0b\xba"
    "\x1d\x09\xa2\x05\x58\x85\xde\xdd\x9e\xf5\x64\x16\x23\x68\x22\x12\x10\x32"
    "\x38\xb8\xf7\x75\xcc\xe4\x2d\xdf\xd4\xf6\x63\x82\xf2\xc3\xa5\xe8\xd6\xdf"
    "\xf9\x16\x3c\xed\x83\x58\x0a\x75\x70\x55\x74\x02\x6b\x55\xdb\x90\xf7\x5f"
    "\x8a\xbb\x30\x14\xc9\xa7\x07\x02\x1d\xed\xc0\x75\xda\x38\xbe\xbb\xf0"
    "\xa0"),
  .tag = TEST_DATA_STR (
    "\x0e\x2c\xe9\xca\xc8\xdf\xce\xdb\x05\x72\xec\x6c\xab\x62\x1e\xfd"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad720_tc12) = {
  .name = "256-GMAC 720-aad NIST CAVS TC12",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\x42\xbc\x84\x1b\x3b\x03\xa8\x07\xcd\x36\x6a\x35\xec\xec\x8a\x6a\xeb\xef"
    "\x7c\x4c\xba\x0e\xc8\xcb\x8d\xa0\xda\x41\xdf\x8c\xce\xf1"),
  .iv = TEST_DATA_STR ("\x1b\xd4\x6f\x85\xdf\x5f\x4b\x3a\x12\x6e\xe3\x15"),
  .aad = TEST_DATA_STR (
    "\xed\xe3\xdc\xdd\xbd\xc7\xd8\xe5\xd0\x34\xc0\x16\x61\x33\x2e\xc3\x49\xcb"
    "\x4e\x7a\x9f\xba\xaf\x7a\xbe\x2c\x64\x75\x87\xdb\x86\xcd\x42\x7c\xe6\x69"
    "\x08\xe0\x70\xbc\x49\xef\x83\x87\x47\xe0\x6b\x45\xac\x48\x6d\xfb\xea\x6f"
    "\x86\x98\xb4\x62\x5e\x21\xe6\x9d\xb8\x32\x7e\xc0\x5c\xfd\x74\xac\xcb\xe6"
    "\x7a\xb6\x44\x94\x8c\xdb\x55\x4a\xf1\x79\xa1\xe2\x64\xe0\x8f\xe1\x66"
    "\x41"),
  .tag = TEST_DATA_STR (
    "\x63\x3a\xb6\xaa\xf5\xb3\x2b\x53\xa7\x94\xf6\xbe\x62\x62\xfc\x5f"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad720_tc13) = {
  .name = "256-GMAC 720-aad NIST CAVS TC13",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xc2\x5b\x85\x00\xbe\x73\x21\x05\x96\xfc\x4a\x9f\xb4\xd8\x4d\x1a\x33\x79"
    "\xa9\x1e\x3f\x0a\x6c\xc4\x17\x7d\x99\x60\x46\x62\x76\x79"),
  .iv = TEST_DATA_STR ("\xb5\x6c\x48\xc0\xc4\xcd\x31\x8b\x20\x43\x70\x02"),
  .aad = TEST_DATA_STR (
    "\xbc\xd1\x4d\xd0\x43\xfd\xc8\xc3\x27\x95\x7e\x1c\x14\x28\x69\x85\x43\xec"
    "\x86\x02\x52\x1a\x7c\x74\x78\x8d\x29\x6d\x37\xd4\x82\x8f\x10\xf9\x06\x56"
    "\x88\x3d\x25\x31\xc7\x02\xeb\xda\x2d\xc0\xa6\x8d\xab\x00\x15\x45\x77\x45"
    "\x44\x55\xfa\xd9\x86\xff\x8e\x09\x73\x09\x8d\xbf\x37\x0f\xf7\x03\xed\x98"
    "\x22\x2b\x94\x57\x26\xed\x9b\xe7\x90\x92\x10\xdd\xbc\x67\x2e\x99\xfd"
    "\xd9"),
  .tag = TEST_DATA_STR (
    "\x81\x71\xd4\xff\x60\xfe\x7e\xf6\xde\x02\x88\x32\x6a\xa7\x32\x23"),
};

UNITTEST_REGISTER_CRYPTO_TEST (aes_gmac256_aad720_tc14) = {
  .name = "256-GMAC 720-aad NIST CAVS TC14",
  .alg = VNET_CRYPTO_ALG_AES_256_NULL_GMAC,
  .key = TEST_DATA_STR (
    "\xdd\x95\x25\x9b\xc8\xee\xfa\x3e\x49\x3c\xb1\xa6\xba\x1d\x8e\xe2\xb3\x41"
    "\xd5\x23\x0d\x50\x36\x30\x94\xa2\xcc\x34\x33\xb3\xd9\xb9"),
  .iv = TEST_DATA_STR ("\xa1\xa6\xce\xd0\x84\xf4\xf1\x39\x90\x75\x0a\x9e"),
  .aad = TEST_DATA_STR (
    "\xd4\x6d\xb9\x0e\x13\x68\x4b\x26\x14\x9c\xb3\xb7\xf7\x76\xe2\x28\xa0\x53"
    "\x8f\xa1\x89\x2c\x41\x8a\xaa\xd0\x7a\xa0\x8d\x30\x76\xf4\xa5\x2b\xee\x8f"
    "\x13\x0f\xf5\x60\xdb\x2b\x8d\x10\x09\xe9\x26\x0f\xa6\x23\x3f\xc2\x27\x33"
    "\xe0\x50\xc9\xe4\xf7\xcc\x69\x90\x62\x76\x5e\x26\x1d\xff\xff\x11\x59\xe9"
    "\x06\x0b\x26\xc8\x06\x5d\xfa\xb0\x40\x55\xb5\x8c\x82\xc3\x40\xd9\x87"
    "\xc9"),
  .tag = TEST_DATA_STR (
    "\x9e\x12\x0b\x01\x89\x9f\xe2\xcb\x3e\x3a\x0b\x0c\x05\x04\x59\x40"),
};

/*
 * fd.io coding-style-patch-verification: ON
 *
 * Local Variables:
 * eval: (c-set-style "gnu")
 * End:
 */
